define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-file-upload'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.agenda', [
        'ui.router',
        'ngResource',
        'ngFileUpload'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.agenda', {
                url: '/agenda',                
                data: {
                    title: 'Agenda',
                    permissions: {
                        only: ['app.agenda']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/lista.html',
                        controller: 'AgendaCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/AgendaCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.agenda.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Agenda',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/form.html',
                        controller: 'AgendaCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/AgendaCtrl',
                                'modules/agenda/directives/agendaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Agenda',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/form.html',
                        controller: 'AgendaCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/AgendaCtrl',
                                'modules/agenda/directives/agendaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Agenda',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/form.html',
                        controller: 'AgendaCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/AgendaCtrl',
                                'modules/agenda/directives/agendaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.upload', {
                url: '/upload',
                data: {
                    title: 'Importar contatos',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/upload.html',
                        controller: 'AgendaCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/AgendaCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.grupos', {
                url: '/agenda/grupos',
                data: {
                    title: 'Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/lista.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.grupos_editar', {
                url: '/grupos/editar/:id',
                data: {
                    title: 'Editar Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.agenda.grupos_novo', {
                url: '/grupos/novo',
                data: {
                    title: 'Novo Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/agenda/views/form.grupos.html',
                        controller: 'GruposCtrl',
                        parent: 'app.agenda',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/agenda/controllers/GruposCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$rootScope) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.agenda', function () {            
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.agenda",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: $rootScope.getWord("Você não tem acesso a Agenda.")
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});