define(['modules/agentes/module'], function (module) {
    "use strict";

    return module.registerController('AgentesCtrl', function ($scope, $http, clientSidePaginationService, $rootScope, $localStorage) {
        $scope.searchText = '';
        $scope.row = {};
        // Injeta paginação
        $scope.pag = clientSidePaginationService;
        $scope.pag.dados = [];
        $scope.pag.sortBy = 'fullname';

        $scope.keycodes = {
            8: "BACKSPACE",
            9: "TAB",
            13: "ENTER",
            16: "SHIFT",
            17: "CTRL",
            18: "ALT",
            19: "PAUSE/BREAK",
            20: "CAPS LOCK",
            27: "ESC",
            32: "ESPAÇO",
            33: "PAGE UP",
            34: "PAGE DOWN",
            35: "END",
            36: "HOME",
            37: "SETA PARA ESQUERDA",
            38: "SETA PARA CIMA",
            39: "SETA PARA DIREITA",
            40: "SETA PARA BAIXO",
            44: "PRINT SCREEN",
            45: "INSERT",
            46: "DELETE",
            59: "PONTO E VIRGULA",
            91: "TECLA WINDOWS ESQUERDA",
            92: "TECLA WINDOWS DIREITA",
            96: "NUMPAD 0",
            97: "NUMPAD 1",
            98: "NUMPAD 2",
            99: "NUMPAD 3",
            100: "NUMPAD 4",
            101: "NUMPAD 5",
            102: "NUMPAD 6",
            103: "NUMPAD 7",
            104: "NUMPAD 8",
            105: "NUMPAD 9",
            106: "MULTIPLICAR",
            107: "ADICIONAR",
            108: "NUMPAD PONTO",
            109: "SUBTRAIR",
            110: "PONTO DECIMAL",
            111: "DIVIDIR",
            48: "0",
            49: "1",
            50: "2",
            51: "3",
            52: "4",
            53: "5",
            54: "6",
            55: "7",
            56: "8",
            57: "9",
            65: "A",
            66: "B",
            67: "C",
            68: "D",
            69: "E",
            70: "F",
            71: "G",
            72: "H",
            73: "I",
            74: "J",
            75: "K",
            76: "L",
            77: "M",
            78: "N",
            79: "O",
            80: "P",
            81: "Q",
            82: "R",
            83: "S",
            84: "T",
            85: "U",
            86: "V",
            87: "W",
            88: "X",
            89: "Y",
            90: "Z",
            112: "F1",
            113: "F2",
            114: "F3",
            115: "F4",
            116: "F5",
            117: "F6",
            118: "F7",
            119: "F8",
            120: "F9",
            121: "F10",
            122: "F11",
            123: "F12",
        };

        $scope.options = {
            height: 200,
            focus: false,
            airMode: false,
            dialogsInBody: true,
            dialogsFade: true,

        };

        $scope.onIntegracaoKeypress = function (event) {
            // Deixa o macaco apagar a tecla!!            
            if (event.keyCode == 8 || event.keyCode == 46) {
                $scope.row.atalho_url = '';
                return;
            }

            let evento = $scope.keycodes[event.keyCode] || event.keyCode;
            $scope.row.atalho_url = evento;

            event.preventDefault();
            return false;
        };

        $scope.imageUpload = function (files) {
            console.log('image upload:', files);
            for (var i = 0; i < files.length; i++) {
                var acl = {};
                acl[$scope.UserId] = { "read": true, "write": true };
                acl["*"] = { "read": true };
                var req = {
                    method: 'POST',
                    url: 'https://api.leancloud.cn/1.1/files/' + files[i].name,
                    headers: {
                        'X-LC-Id': $rootScope.LeanCloudId,
                        'X-LC-Key': $rootScope.LeanCloudKey,
                        'Content-Type': files[i].type
                    },
                    data: {
                        'file': files[i],
                        "ACL": acl
                    }
                };
                $http(req).then(function successCallback (resp) {
                    toastr.success('Success! You have uploaded image(s).', $rootScope.message_title);
                    console.log(resp.data.url);
                    $scope.comment_text += '<img src="' + resp.data.url + '">'
                }, function errorCallback (resp) {
                    toastr.error(resp.data.error, $rootScope.message_title);
                });
            }
        };

        $scope.fechapop = function () {
            console.log('Clicou no close');
            $scope.showModalPopUp = false;
        };

        $scope.filterFila = (query) => {
            return $scope.filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        }

        $scope.filterGrupo = (query) => {
            return $scope.gruposNomes.filter(function (grupo) {
                return grupo.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        }

        $scope.onGrupoAdded = function (tag) {
            // Encontre o grupo de filas correspondente ao tag adicionado
            const grupo = $scope.grupos.find(grupo => grupo.nome === tag.text);
            // Verifique se o grupo foi encontrado antes de continuar
            if (!grupo) return;
            // Verifique se o grupo já está na lista de grupos de filas atuais
            const gruposAtuaisNomes = $scope.row.grupo_filas.map(grupo => grupo.text);
            if (!gruposAtuaisNomes.includes(grupo.nome)) {
                $scope.row.grupo_filas.push({ text: grupo.nome });
            }
        };

        $scope.onGrupoRemoved = function (tag) {
            // Filtra para manter apenas os grupos atuais que ainda estão na lista de grupo_filas
            const gruposAtuaisNomes = $scope.row.grupo_filas.map(grupo => grupo.text);
            const gruposAtuais = $scope.grupos.filter(grupo => gruposAtuaisNomes.includes(grupo.nome));
            // Atualiza a lista de grupo_filas removendo o grupo removido
            $scope.row.grupo_filas = gruposAtuais.map(grupo => ({ text: grupo.nome }));
        };

        $scope.onToggleGrupo = function () {
            $scope.row.filas = []
            $scope.row.grupo_filas = []
        }

        function gerarSenhaAleatoria () {
            const caracteres = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*';
            const tamanho = 16;
            let senha = '';
            for (let i = 0; i < tamanho; i++) {
                senha += caracteres.charAt(Math.floor(Math.random() * caracteres.length));
            }
            return senha
        };

        $scope.showpopup = async function (id) {
            $scope.showModalPopUp = true;
            $scope.loading = true;

            const fontes = await $http.get(`${options.api.restv4}/autenticacao_fontes`);
            $scope.fontes_autenticacao = fontes.data ? fontes.data : [];

            const getfilas = await $http.get(options.api.restv2 + "/filas?max=-1");
            $scope.filas = $localStorage.User.filas;
            $scope.instancias = getfilas.data.dados.filter(f => f.instance && $scope.filas.includes(f.name));

            const ordem = [{ 'campo': 'nome', 'ordem': 'ASC' }]
            const getGrupos = await $http.get(options.api.restv4 + "/gruposFilas?max=-1&order=" + encodeURI(JSON.stringify(ordem)) + '&filas=' + encodeURI(JSON.stringify($scope.filas)))
            $scope.grupos = getGrupos.data.dados
            $scope.gruposNomes = $scope.grupos.map(grupo => grupo.nome)

            $scope.row = (await $http.get(options.api.restv4 + `/agente/${id}`)).data;
            $scope.row.filas = $scope.row.agenteFilas.map(agf => { return agf.fila })
            $scope.row.grupo_filas = $scope.row.agenteGruposFila.map(agg => { return agg.fila_grupo })
            $scope.row.senha = gerarSenhaAleatoria()

            delete $scope.row.agenteFilas;
            delete $scope.row.agenteGruposFila;

            $scope.loading = false;
            $scope.senha_disable = $scope.row.autenticacao_fonte ? true : false;

            $scope.senhaantiga = angular.copy($scope.row.senha);

            $scope.row.possui_limite_de_sessao = $scope.row.tempo_maximo_sessao !== null && $scope.row.tempo_maximo_sessao !== "";
            $scope.row.tempo_maximo_sessao = $scope.row.tempo_maximo_sessao ? FloatTotime($scope.row.tempo_maximo_sessao) : null;

            $scope.row.possui_auto_pausa = $scope.row.tempo_auto_pausa !== null && $scope.row.tempo_auto_pausa !== "";
            $scope.row.tempo_auto_pausa = $scope.row.tempo_auto_pausa;

            if ($scope.row.atalho_url && !$scope.row.abrir_url) {
                $scope.row.atalho_url = $scope.row.atalho_url in $scope.keycodes ? $scope.keycodes[$scope.row.atalho_url] : $scope.row.atalho_url;
                $scope.row.abrir_url = false;
            } else {
                $scope.row.atalho_url = null;
                $scope.row.abrir_url = true;
            }
        };

        $scope.new = async function () {
            $scope.showModalPopUp = true;
            $scope.loading = true;
            $scope.row = {};
            $scope.row.ramal = 'SIP/'

            const fontes = await $http.get(`${options.api.restv4}/autenticacao_fontes`);
            $scope.fontes_autenticacao = fontes.data ? fontes.data : [];

            const getfilas = await $http.get(options.api.restv2 + "/filas?max=-1");
            $scope.filas = $localStorage.User.filas;
            $scope.instancias = getfilas.data.dados.filter(f => f.instance && $scope.filas.includes(f.name));

            const ordem = [{ 'campo': 'nome', 'ordem': 'ASC' }]
            const getGrupos = await $http.get(options.api.restv4 + "/gruposFilas?max=-1&order=" + encodeURI(JSON.stringify(ordem)) + '&nomes=' + encodeURIComponent(JSON.stringify($localStorage.User.gruposFila)))
            $scope.grupos = getGrupos.data.dados
            $scope.gruposNomes = $scope.grupos.map(grupo => grupo.nome)

            $scope.loading = false;

            $scope.row.modo = 'F';
            $scope.row.abrir_url = false;
        };

        //senha_disable
        $scope.FonteSelec = () => {
            $scope.senha_disable = $scope.row.autenticacao_fonte ? true : false;
        }

        $scope.filter = async function () {
            let filteredRT = angular.copy($scope.reminderTypes);

            if ($scope.searchText.length > 0) {
                filteredRT = filteredRT.filter(function (obj) {
                    return Object.values(obj).find(v => {
                        if (v) {
                            return v.toString().toLowerCase().includes($scope.searchText.toLowerCase());
                        }
                    })
                });
            }

            // Pagina os dados filtrados
            $scope.pag.dados = filteredRT;
            // Ordena
            $scope.pag.order();
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $http.get(options.api.restv4 + "/agente").
                success(function (data) {
                    $scope.loading = false;
                    $scope.reminderTypes = data.map(agente => {
                        agente.autenticacao_fonte = agente.autenticacaoFontes.length > 0 ? agente.autenticacaoFontes[0].nome : 'Xcontact';
                        return agente;
                    })
                    $scope.filter();
                })
                .error(er => {
                    $scope.loading = false;
                    console.log(`Deu erro no get`);
                    console.log(er);
                })
        };

        $scope.check = function (id) {
            console.log('Marcou ' + id);
            $scope.row.modo = id;
        };

        $scope.getFilasAgente = (filas) => {
            if (!filas) return;
            return filas.map(fila => {
                return fila.fila;
            }).join(', ');
        }

        function timeToFloat (timestr) {
            if (!timestr) return;

            const time_format = timestr.split(':');

            const hours = parseInt(time_format[0]);
            const minutes = parseFloat(time_format[1] / 60);
            const time = hours + minutes;
            return parseFloat(time).toFixed(2);
        }

        $scope.validarCampo = () => {
            $scope.row.fullname = $scope.row.fullname.replace(/[^a-zA-Z0-9\s\-]+/g, '');
        };

        $scope.$watch('row.fullname', () => {
            if (!$scope.row.id && $scope.row.fullname) $scope.validarCampo();
        });

        function padTime (time) {
            return (time < 10 ? "0" : "") + time;
        }

        function FloatTotime (num) {
            if (!num) return;

            var min = 1 / 60;

            var intpart = Math.floor(num);
            var decpart = num - intpart;

            decpart = min * Math.round(decpart / min);

            var minutes = Math.floor(decpart * 60);
            return padTime(intpart) + ':' + padTime(minutes);
        }

        $scope.Salvar = function (f1) {
            if (f1.$valid) {
                let payload = angular.copy($scope.row);
                payload.nome = payload.nome.trim();
                payload.fullname = payload.fullname.trim();

                if ($scope.senhaantiga == $scope.row.senha) {
                    delete payload.senha;
                }

                payload.tempo_maximo_sessao = payload.possui_limite_de_sessao ? timeToFloat(payload.tempo_maximo_sessao) : null;
                delete payload.possui_limite_de_sessao;

                payload.tempo_auto_pausa = payload.tempo_auto_pausa && payload.possui_auto_pausa ? parseFloat(payload.tempo_auto_pausa) : null;
                delete payload.possui_auto_pausa;

                // Delete trash
                delete payload.datastatus;
                delete payload.status;
                delete payload.edita_rota;
                delete payload.autenticacaoFontes;
                delete payload.profileImage;
                delete payload.token;
                delete payload.visualizar_dado_1;
                delete payload.visualizar_dado_2;

                if (!payload.usa_grupo_fila && payload.filas?.length > 0) payload.filas = payload.filas[0].text ? payload.filas.map(fila => fila.text) : $scope.row.filas;
                else payload.filas = []
                if (payload.grupo_filas) payload.grupo_filas = payload.grupo_filas.map(grupo => grupo.text);

                if (payload.abrir_url) {
                    if (payload.atalho_url) {
                        // Verifica se o atalho existe no key codes e está sendo mascarado
                        let i = Object.values($scope.keycodes).indexOf(payload.atalho_url);
                        if (i !== -1) payload.atalho_url = Object.keys($scope.keycodes)[i].toString();
                        else payload.atalho_url = payload.atalho_url.toString();
                    }
                    else {
                        payload.atalho_url = null;
                    }
                } else {
                    payload.atalho_url = null;
                }

                if (payload.id) {
                    delete payload.id;

                    $http.put(`${options.api.restv4}/agente/${$scope.row.id}`, payload)
                        .success(response => {
                            $scope.showModalPopUp = false;
                            $scope.load();
                            f1.$setPristine();
                            f1.$setUntouched();

                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Atualizado com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                        });

                } else {
                    $http.post(`${options.api.restv4}/agente`, payload)
                        .success(data => {
                            $scope.showModalPopUp = false;
                            $scope.load();
                            f1.$setPristine();
                            f1.$setUntouched();

                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                        });
                }
            }
        };

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.agentes');
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.fullname + "</b>?",
                content: "Tem certeza que quer remover " + row.fullname + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.confirmadesloga = function (row) {
            $.x5Box({
                title: "Deslogar <b>" + row.fullname + "</b>?",
                content: "Tem certeza que quer deslogar o agente " + row.fullname + " de todas as filas? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Deslogar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Deslogar") {
                    $scope.desloga(row.id);
                }
            });
        };

        $scope.desloga = function (id) {
            $http.put(`${options.api.restv4}/agente/${id}/logoffall`).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Agente deslogado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });
                });
        };

        $scope.delete = function (id) {
            $http.delete(`${options.api.restv4}/agente/${id}`).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Agentes apagado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });

                    $scope.load();
                })
        };
    });
});