define(['modules/campanhas/module'], function (module) {
    "use strict";
    const moment = require("moment");

    return module.registerController('CampanhaCtrl', function ($scope, $http, Upload, $timeout, $stateParams, SocketService, $filter, $rootScope, $localStorage, desktopNotification) {
        $scope.markall = false;
        $scope.marcados = [];
        $scope.row = {};
        $scope.campanha = {};
        $scope.carregando = false;
        $scope.layouts = [];
        $scope.row.data_final = null;
        $scope.row.hora_final = null;
        $scope.filtro = {};
        $scope.campos_disable = false;
        $scope.order = [{ "campo": "nome", "ordem": "ASC" }];

        $scope.vm = {};
        $scope.vm.permission = undefined;
        $scope.vm.isSupported = desktopNotification.isSupported();
        $scope.vm.permission = desktopNotification.currentPermission();
        $scope.vm.autoClose = true;

        let dataCreateInicial = new Date();
        dataCreateInicial.setDate(dataCreateInicial.getDate() - 30);

        $scope.filtros = {
            dataCreateInicial: dataCreateInicial,
            dataCreateFinal: new Date(),
            dataInicial: null,
            dataFinal: null,
            nome: "",
            status: ""
        };

        $scope.turnosSelecionados = [];
        $scope.turno = null;
        $scope.load_zenvia = true;

        $scope.selall = () => {
            $scope.table.dados.forEach(item => {
                item.marcado = $scope.markall;
            });
        };

        // Configura a visibilidade dos menus de acordo com o tipo de campanha
        $scope.roles = {
            /***
             * Essa função define o modo operante, ou seja, o tipo de campanha atual
             *
             * @returns {string}
             ***/
            getMode () {
                let fila = $scope.getFilaSelecionada();
                if (fila && fila.queue_type === 3) {
                    return "whatsapp";
                }
                else if (fila && (fila.queue_type === 5 || fila.queue_type === 7)) {
                    return "zenvia";
                }
                else {
                    return "normal";
                }
            },
            check (menuId, subMenuId) {
                let currentMode = $scope.roles.getMode();
                let menu = $scope.roles[menuId];
                if (menu) {
                    return menu[subMenuId || "menu"].indexOf(currentMode) !== -1
                } else {
                    return false;
                }
            },

            geral: {
                menu: ["normal", "whatsapp", "zenvia"],

                nome: ["normal", "whatsapp", "zenvia"],
                filaAssociadaACampanha: ["normal", "whatsapp", "zenvia"],
                status: ["normal", "whatsapp", "zenvia"],
                modo: ["normal"],
                ddi: ["whatsapp", "zenvia"],
                mensagemCampanha: ["whatsapp"],
            },

            regras: {
                menu: ["normal", "whatsapp", "zenvia"],

                tentativas: ["normal"],
                simultaneasPorAgente: ["normal"],
                desconectarSimultaneas: ["normal"],
                tempoMaximoChamando: ["normal"],
                layout: ["normal", "whatsapp", "zenvia"],
                intervaloEntreMensagens: ["whatsapp", "zenvia"],
                status_atendimento: ["whatsapp", "zenvia"],
                turnos: ["whatsapp", "zenvia"],
                autoFinalizar: ["normal", "whatsapp", "zenvia"],
                requerAgentesOnline: ["normal", "whatsapp", "zenvia"],
            },

            horarios: {
                menu: ["normal", "whatsapp", "zenvia"],

                agendamento: ["normal", "whatsapp", "zenvia"],
                turnos: ["normal", "whatsapp", "zenvia"],
            },

            rotas: {
                menu: ["normal"],

                grupoDeRotaDeSaida: ["normal"],
                destino: ["normal"],
                requerAgentesOnline: ["normal"],
                maximoLigacoesSimultaneas: ["normal"],
                tempoMinimoEmSegundos: ["normal"],
            },

            classificacoes: {
                menu: ["normal"],

                rediscar: ["normal", "whatsapp", "zenvia"],
                removemContato: ["normal", "whatsapp", "zenvia"],
                removemNumer: ["normal", "whatsapp", "zenvia"],
            },

            notificacoes: {
                menu: ["normal", "whatsapp", "zenvia"],

                notificacoes: ["normal", "whatsapp", "zenvia"]
            },

            zenvia: {
                menu: ["zenvia"]
            },
        }

        $scope.PediPermicao = function () {
            desktopNotification.requestPermission().then(function (permission) {
                $scope.vm.permission = permission;
                alert('Permissão concedida.');
            }, function (permission) {
                $scope.vm.permission = permission;

                if ($scope.vm.permission === 'denied') {
                    alert('Você não autorizou o uso de notificações de área de trabalho. Se tiver utilizando Google Chrome clique no ícone de informações da página que fica fica a esquerda do endereço desta página e autorize as notificações.');
                } else {
                    alert('Permissão negada.');
                }
            });
        };

        $scope.init = function () {
            $scope.filtro.page = { valor: 1 };
            $scope.load();
        };

        $scope.Limpa = function (r) {
            return JSON.stringify(r).replace(/[{}"\[\]]/g, '').replace(',', ', ');
        };

        $scope.row.ligacoes_agente = '1';
        $scope.row.max_fila = '2';
        $scope.row.max_chamar = '30';
        $scope.campanha_status = [
            { "nome": $scope.getWord('Ativa'), "valor": "A" },
            { "nome": $scope.getWord('Em pausa'), "valor": "P" },
            { "nome": $scope.getWord('Finalizada'), "valor": "F" }
        ];

        $scope.status_atendimento = [
            { "nome": $scope.getWord('Encerrado'), "valor": "Encerrado" },
            { "nome": $scope.getWord('Em atendimento'), "valor": "Entrou" },
            { "nome": $scope.getWord('Pesquisa'), "valor": "Pesquisa" },
            { "nome": $scope.getWord('Pré-atendimento'), "valor": "PreAtendimento" },
            { "nome": $scope.getWord('Confirmação'), "valor": "Confirmacao" },
        ];

        $scope.modos = [
            { "nome": $scope.getWord('Power'), "valor": "A" },
            { "nome": $scope.getWord('Preview'), "valor": "M" }
        ];

        $scope.setModo = function () {
            if ($scope.campanha.modo === 'M') {
                $scope.campanha.max_por_agente = 1;
                $scope.campanha.desconecta_multiplas = false;
                $scope.campanha.requer_agente_online = true;
            }
        };

        $scope.getPercentual = function (v1, v2, v3) {
            if (!v1 || v1 === 0) return 0;
            if (!v2 || v2 === 0) return 0;
            if (!v3 || v3 === 0) return 0;
            return Math.floor((v1 / (v2 * v3)) * 100);
        };

        $scope.getLayouts = function () {
            $http.get(options.api.restv2 + "/layouts?max=-1").
                success(function (data) {
                    $scope.layouts = data.dados;
                });
        };

        $scope.getTemplatesZenvia = function () {
            $scope.load_zenvia = true;
            let fila = $scope.campanha.fila;
            console.log(`Obtendo templates da zenvia para fila ${fila}`);
            $http.get(`${options.api.restv2}/zenvia/getTemplates/${fila}`).
                success(function (data) {
                    console.log(`RETORNO`, data);
                    if (data.templates) data = data.templates;
                    data.map(t => {
                        if (!t.name) t.name = t.vertical
                    });
                    $scope.zenvia_templates = data;
                    $scope.load_zenvia = false;
                    if ($scope.campanha.rota) {
                        console.log('Template definido');
                        console.log($scope.campanha.rota);
                        let tmp = $scope.zenvia_templates.findIndex(item => item.id == $scope.campanha.rota.id);
                        $scope.campanha.rota = $scope.zenvia_templates[tmp];
                        console.log(tmp);
                    }
                });
        };

        $scope.getClassificacoes = function () {
            $http.get(options.api.restv2 + "/classificacoes?max=-1").
                success(function (data) {
                    $scope.classificacoes = data.dados;
                });
        };

        $scope.filterClassificacoes = function (query) {
            return $scope.classificacoes.filter(function (t) {
                if (t.nome)
                    return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                else
                    return false;
            });
        };

        $scope.getTags = function () {
            $http.get(options.api.restv4 + "/tags?max=-1").
                success(function (data) {
                    $scope.tags = data;
                });
        };

        $scope.filterTags = function (query) {
            return $scope.tags.filter(function (t) {
                if (t.nome)
                    return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                else
                    return false;
            });
        };

        $scope.getTipoCampanha = function (tipo) {
            return [
                "Voz",      // Tipo 0
                "WhatsApp"  // Tipo 1
            ][tipo];
        };

        $scope.getFilas = function () {
            $http.get(options.api.restv2 + `/filas?max=-1&order=${encodeURI(JSON.stringify([{ campo: 'name', ordem: 'ASC' }]))}`).
                success(function (data) {
                    // Voz                  -> 1
                    // WebChat              -> 2
                    // WhatsApp             -> 3
                    // E-mail               -> 4
                    // WhatsApp (Oficial)   -> 5

                    const tiposDeFilasAceitas = [
                        1 /* Voz */,
                        3 /* WhatsApp */,
                        5 /* WhatsApp(Zenvia) */,
                        7 /* WhatsApp(Gupshup) */
                    ];

                    // Filtra as filas, ignorando filas de webchat e e-mail
                    $scope.filas = data.dados.filter(x => tiposDeFilasAceitas.indexOf(x.queue_type) !== -1);
                });
        };

        $scope.getFilaSelecionada = function () {
            if ($scope.filas) {
                return $scope.filas.find(x => x.name === $scope.campanha.fila);
            } else {
                return null;
            }
        };

        $scope.AddOpcaoReagenda = () => {
            $scope.campanha.classificacao_reagenda_contato.push({
                tags: []
            });
        }

        $scope.DelOpcaoReagenda = (index) => {
            $scope.campanha.classificacao_reagenda_contato.splice(index, 1);
        }

        $scope.getRotasGrupos = function () {
            var sentidos = ["Saida"];
            $http.get(options.api.restv2 + "/rotas/grupos?max=-1&sentido=" + encodeURI(JSON.stringify(sentidos))).
                success(function (data) {
                    $scope.rotas = data.dados;
                });
        };

        $scope.getStatus = function (st) {
            switch (st) {
                case 'A':
                    return $scope.getWord('Ativa');
                    break;
                case 'F':
                    return $scope.getWord('Finalizada');
                    break;
                case 'P':
                    return $scope.getWord('Pausada');
                    break;
            }
        };

        $scope.getDate = function (date) {
            if (date) {
                return moment(date).format('DD/MM/YYYY')
            } else {
                return ''
            }

        }

        $scope.getModo = function (st) {
            switch (st) {
                case 'A':
                    return $scope.getWord('Power');
                    break;
                case 'M':
                    return $scope.getWord('Preview');
                    break;
            }
        };

        SocketService.on('open', function () {
            $scope.tentativa = 0;
            console.log('Conexão com websocket aberta');
            var token = { token: "H91A+3zz_-.Y18DH1YeQP65l?G,Q7)F2agm|mq^ukdh,]k,Z[a@195uJhK3n-{" };
            SocketService.emit('authentication', JSON.stringify(token));
            //$scope.getQueueStatus(false);
            //if(timeout) $interval.cancel(timeout);
        });

        SocketService.on('authenticated', function () {
            console.log('Autenticou');
            //$scope.getQueueStatus(false);
        });

        SocketService.on('disconnect', function (err) {
            console.log('### CONEXÃO FECHADA ###');
            $scope.tentativa = 1;
            //timeout = $interval(Reconecta, 5000);
        });

        SocketService.on('connect_failed', function (err) {
            console.log('### FALHA NA CONEXAO, verifique se o serviço está rodando no servidor ###');
            $scope.tentativa = 1;
        });

        SocketService.on('connect_error', function (err) {
            console.log('### FALHA NA CONEXAO, verifique se o serviço está rodando no servidor ###');
            $scope.filas = [];
            $scope.tentativa = 1;
        });

        SocketService.on('message', function (event) {
            var response = JSON.parse(event);
            if (response.event === 'CampanhaFim') {
                NotificaFim(response.campanha);
            }
        });

        SocketService.on('cadastra_contato_campanha_conclusion', function (event) {
            console.log("Cadastra contato campanha conclusão", event);
            $.smallBox({
                title: "SUCESSO!!",
                content: "<i class='fa fa-check'></i> <i>Clientes cadastrados com sucesso!</i>",
                color: "#739E73",
                timeout: 3000
            });
        });

        SocketService.on('cadastra_contato_campanha_error', function (event) {
            console.log("Cadastra contato campanha erro", event);
            $.smallBox({
                title: "ERRO!!",
                content: "<i class='fa fa-check'></i> <i>Erro ao cadastrar clientes!</i>",
                color: "#C46A69",
                timeout: 3000
            });
        });

        SocketService.on('cadastra_contato_campanha_progress', function (event) {
            console.log("Cadastra contato campanha progresso", event);
        })

        SocketService.on('upsert_contato_campanha_conclusion', function (event) {
            console.log("Upsert contato campanha conclusão", event);
            $.smallBox({
                title: "SUCESSO!!",
                content: "<i class='fa fa-check'></i> <i>Clientes atualizados com sucesso!</i>",
                color: "#739E73",
                timeout: 3000
            });
        });

        SocketService.on('upsert_contato_campanha_error', function (event) {
            console.log("Upsert contato campanha erro", event);
            $.smallBox({
                title: "ERRO!!",
                content: "<i class='fa fa-check'></i> <i>Erro ao atualizar clientes!</i>",
                color: "#C46A69",
                timeout: 3000
            });
        })

        SocketService.on('upsert_contato_campanha_progress', function (event) {
            console.log("Upsert contato campanha progresso", event);
        })

        function NotificaFim (campanha) {
            var notif = desktopNotification.show('Xcontact - Fim da campanha', {
                icon: 'styles/img/logo-o.png',
                body: 'A campanha ' + campanha.nome + ' foi finalizada. Clique aqui para maiores informações.',
                autoClose: false,
                vibrate: true,
                onClick: function (event) {
                    window.focus(); notif.close();
                }
            });
        };

        function PadTime (num) {
            return ("0" + num).slice(-2);
        }

        $scope.CarregarItem = function (row) {
            $scope.popup = {};
            $http.get(options.api.restv2 + "/campanha/" + row.id).
                success(function (data) {
                    $scope.campanha = data.campanha;
                    $scope.campos_disable = false;
                    $scope.campanha.hora_ini = '';
                    $scope.campanha.hora_fim = '';
                    row.load = false;
                    $scope.showCampanhaPopup = true;
                    $scope.popup.go_contexto = $scope.campanha.exec_contexto ? true : false;
                    if ($scope.layouts.length > 0 && !$scope.campanha.id_layout || $scope.campanha.id_layout === null || $scope.campanha.id_layout === "null") $scope.campanha.id_layout = $scope.layouts[0].id;

                    if ($scope.campanha.data_ini) {
                        $scope.campanha.data_ini = new Date($scope.campanha.data_ini);
                        $scope.campanha.hora_ini = `${PadTime($scope.campanha.data_ini.getHours())}:${PadTime($scope.campanha.data_ini.getMinutes())}`;
                        $scope.campanha.agendamento = true;
                    }

                    if ($scope.campanha.data_fim) {
                        $scope.campanha.data_fim = new Date($scope.campanha.data_fim);
                        $scope.campanha.hora_fim = `${PadTime($scope.campanha.data_fim.getHours())}:${PadTime($scope.campanha.data_fim.getMinutes())}`;
                        $scope.campanha.agendamento = true;
                    }

                    if ($scope.campanha.ddi_padrao) {
                        $scope.campanha.nao_requer_ddi = false;
                        $scope.campanha.ddi = $scope.campanha.ddi_padrao;
                    } else {
                        $scope.campanha.nao_requer_ddi = true;
                    }

                    if ($scope.campanha.turnos) {
                        $scope.campanha.turnos = $scope.turnos.filter(x => $scope.campanha.turnos.find(y => y === x.id));
                    } else {
                        console.error("Turnos inválidos");
                        $scope.campanha.turnos = [];
                    }

                    // Marca por padrão as checkboxes de mensagens, imagens e arquivos
                    if ($scope.campanha.texto_padrao_campanha) $scope.campanha.mensagem_padrao_texto = true;
                    if ($scope.campanha.caminho_imagem_campanha) $scope.campanha.mensagem_padrao_imagem = true;
                    if ($scope.campanha.caminho_arquivo_campanha) $scope.campanha.mensagem_padrao_arquivo = true;

                    if ($scope.roles.getMode() == 'zenvia' && $scope.campanha.rota) {
                        $scope.campanha.rota = JSON.parse($scope.campanha.rota);
                        console.log($scope.campanha.rota);
                    }

                    // Busca tags da campanha
                    $http.get(options.api.restv4 + "/campanha/" + row.id + "/tags?max=-1")
                        .success(function (data) {
                            if (data.dados) {
                                $scope.campanha.classificacao_reabre_contato = [];
                                $scope.campanha.classificacao_remove_contato = [];
                                $scope.campanha.classificacao_remove_numero = [];
                                $scope.campanha.classificacao_reagenda_contato = [];
                                for (var i = 0; i < data.dados.length; ++i) {
                                    switch (data.dados[i].acao) {
                                        case 'AN':
                                            $scope.campanha.classificacao_reabre_contato.push(data.dados[i]);
                                            break;
                                        case 'RN':
                                            $scope.campanha.classificacao_remove_numero.push(data.dados[i]);
                                            break;
                                        case 'RC':
                                            $scope.campanha.classificacao_remove_contato.push(data.dados[i]);
                                            break;
                                        case 'RG':
                                            $scope.campanha.classificacao_reagenda_contato.push({ horas_apos: data.dados[i].horas_apos, tags: [{ ...data.dados[i] }] });
                                            break;
                                    }
                                }
                                console.log($scope.campanha.classificacao_reagenda_contato);
                                if ($scope.campanha.classificacao_reagenda_contato.length === 0)
                                    $scope.campanha.classificacao_reagenda_contato.push({ tags: [] });
                            }
                        });
                });
        };

        $scope.showedit = function (row) {
            $('#myTab1 a[href="#s1"]').tab('show');
            row.load = true;
            $scope.files = undefined;
            $scope.getFilas();
            $scope.getLayouts();
            $scope.getRotasGrupos();
            $scope.getTags();
            $scope.CarregarItem(row);
        };

        $scope.NovaCampanha = function () {
            $scope.files = undefined;
            $scope.getFilas();
            $scope.getLayouts();
            $scope.getRotasGrupos();
            $scope.getTags();
            $scope.campanha = {};
            $scope.popup = {};
            // Valores default

            $scope.popup.go_contexto = false;
            $scope.campanha.autofim = true;
            $scope.campanha.desconecta_multiplas = false;
            $scope.campanha.id_layout = 1;
            $scope.campanha.macro = 'fila';
            $scope.campanha.max_chamar = 30;
            $scope.campanha.max_por_agente = 1;
            $scope.campanha.max_simultaneas_campanha = 30;
            $scope.campanha.modo = 'A';
            $scope.campanha.status = 'A';
            $scope.campanha.requer_agente_online = false;
            $scope.campanha.rota = 'discador';
            $scope.campanha.tempo_minimo_completar = 0;
            $scope.campanha.tentativas = 3;
            $scope.campanha.classificacao_remove_contato = [];
            $scope.campanha.classificacao_remove_numero = [];
            $scope.campanha.classificacao_reabre_contato = [];
            $scope.campanha.classificacao_reagenda_contato = [{ tags: [] }];
            $scope.campanha.turnos = [];
            $scope.showCampanhaPopup = true;
        };

        $scope.clonar = function (row) {
            $scope.popup = {};
            $scope.getFilas();
            $scope.getLayouts();
            $scope.loadTurnos();
            $scope.getRotasGrupos();
            $scope.getTags();
            $http.get(options.api.restv2 + "/campanha/" + row.id).
                success(function (data) {
                    $scope.campanha = data.campanha;
                    $scope.campos_disable = false;
                    delete $scope.campanha.id;
                    delete $scope.campanha.data_ini;
                    delete $scope.campanha.data_fim;
                    delete $scope.campanha.descricao;
                    $scope.campanha.nome = '';
                    $scope.showCampanhaPopup = true;
                    $scope.popup.go_contexto = $scope.campanha.exec_contexto ? true : false;
                    if ($scope.campanha.turnos) {
                        $scope.campanha.turnos = $scope.turnos.filter(x => $scope.campanha.turnos.find(y => y === x.id));
                    } else {
                        console.error("Turnos inválidos");
                        $scope.campanha.turnos = [];
                    }

                    // Busca classificações da campanha
                    $http.get(options.api.restv2 + "/campanha/" + row.id + "/classificacoes?max=-1")
                        .success(function (data) {
                            if (data.dados) {
                                $scope.campanha.classificacao_reabre_contato = [];
                                $scope.campanha.classificacao_remove_contato = [];
                                $scope.campanha.classificacao_remove_numero = [];
                                $scope.campanha.classificacao_reagenda_contato = [];
                                for (var i = 0; i < data.dados.length; ++i) {
                                    switch (data.dados[i].acao) {
                                        case 'AN':
                                            $scope.campanha.classificacao_reabre_contato.push(data.dados[i]);
                                            break;
                                        case 'RN':
                                            $scope.campanha.classificacao_remove_numero.push(data.dados[i]);
                                            break;
                                        case 'RC':
                                            $scope.campanha.classificacao_remove_contato.push(data.dados[i]);
                                            break;
                                        case 'RG':
                                            $scope.campanha.classificacao_reagenda_contato.push({ horas_apos: data.dados[i].horas_apos, tags: [{ ...data.dados[i] }] });
                                            break;
                                    }
                                }
                            }
                        });
                });
        };

        $scope.gotoPage = function (page) {
            $scope.filtro.page = { valor: page };
            $scope.iniciarFiltragem();
        };

        $scope.getTurnosNaoSelecionados = function () {
            // Valida se o array existe, caso não exista, cria
            if (!$scope.campanha.turnos) $scope.campanha.turnos = [];

            return $scope.turnos.filter(x => $scope.campanha.turnos.indexOf(x) === -1);
        }

        $scope.extrairNomeArquivo = function (path) {
            try {
                return path.substring(path.lastIndexOf("/") + 1);
            } catch (e) {
                return path;
            }
        };

        $scope.downloadArquivo = async function (path) {
            fetch(`${options.api.gravacao}${path}`)
                .then(response => response.blob())
                .then(blob => {
                    const url = window.URL.createObjectURL(blob);

                    const link = document.createElement("a");
                    link.style.display = "none";
                    link.href = url;
                    link.download = $scope.extrairNomeArquivo(path);

                    document.body.appendChild(link);
                    link.click();

                    setTimeout(() => {
                        console.log(`Successfully downloaded file '${path}'`);

                        document.body.removeChild(link);
                        window.URL.revokeObjectURL(url);
                    }, 250);
                })
                .catch(error => {
                    console.error(error);

                    $.smallBox({
                        title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                        content: $scope.getWord('Erro ao fazer download de arquivo da campanha!<br>' + error),
                        color: "#C46A69",
                        icon: "fa fa-warning shake animated"
                    });
                });
        };

        $scope.loadTurnos = function () {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv2 + `/horarios?max=-1`)
                    .success(data => {
                        $scope.turnos = data.dados;
                        resolve();
                    })
                    .error(error => reject(error));
            });
        };

        $scope.filterTurnos = function (query) {
            return $scope.turnos.filter(function (t) {
                return t.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };
        /**
         * Carrega dados da lista
         * @returns void
         */
        $scope.load = async function () {
            $scope.carregando = true;

            // Chama o load turnos e espera voltar, para aproveitar o mesmo loader
            await $scope.loadTurnos();

            var par = [];

            $scope.filtro["data_create_ini"] = $scope.filtros.dataCreateInicial ? {
                valor: moment($scope.filtros.dataCreateInicial).format("YYYY-MM-DD 00:00:00"),
                date: $scope.filtros.dataCreateInicial
            } : null;
            $scope.filtro["data_create_fim"] = $scope.filtros.dataCreateFinal ? {
                valor: moment($scope.filtros.dataCreateFinal).format("YYYY-MM-DD 23:59:59"),
                date: $scope.filtros.dataCreateFinal
            } : null;

            $scope.filtro["data_agendamento_ini"] = $scope.filtros.dataInicial ? {
                valor: moment($scope.filtros.dataInicial).format("YYYY-MM-DD 00:00:00"),
                date: $scope.filtros.dataInicial
            } : null;
            $scope.filtro["data_agendamento_fim"] = $scope.filtros.dataFinal ? {
                valor: moment($scope.filtros.dataFinal).format("YYYY-MM-DD 23:59:59"),
                date: $scope.filtros.dataFinal
            } : null;

            if ($scope.filtro) {
                angular.forEach($scope.filtro, function (v, k) {
                    if (v && v.valor !== null) par.push(k + "=" + v.valor);
                });
            }

            if (options.importacaoJBS && $localStorage.User.filas && Array.isArray($localStorage.User.filas) && $localStorage.User.filas.length > 0) {
                par.push(`filas=${encodeURI(JSON.stringify($localStorage.User.filas))}`);
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
                console.log(pars);
            }
            $http.get(options.api.restv3 + "/campanha?max=-1&order=" + encodeURI(JSON.stringify($scope.order)) + pars).
                success(function (data) {
                    $scope.raw_table = data;

                    // DEBUG: Teste de filtragem com várias entradas
                    /* let d = []
                    for (let i = 0; i < 9000; i ++) {
                        for (let j = 0; j < data.dados.length; j++) {
                            let e = data.dados[j];
                            e.id = i + j * data.dados.length;
                            // e.nome += Math.floor(Math.random() * 1000).toString().padStart(4, '0');
                            d.push({ ...e });
                        }
                    }
                    $scope.raw_table.dados = d; */

                    setTimeout(filtrar, 0);
                });
        };

        async function filterAsync (array, predicate) {
            const res = [];

            await array.reduce(async (promise, n) => {
                await promise;
                if (await predicate(n)) {
                    res.push(n);
                }
            }, Promise.resolve());

            return res;
        }

        function filtrar () {
            return new Promise(async (resolve, reject) => {
                $scope.carregando = true;
                console.time(`Filtrar`);

                if (!$scope.raw_table) {
                    $scope.table = {
                        dados: 0,
                        total: 0,
                        pages: 0,
                        offset: 0
                    };
                } else {
                    let dados = $scope.raw_table.dados || [];
                    const PAGE_SIZE = 15;

                    let filtroNome = $scope.filtros.nome ? $scope.filtros.nome.toLowerCase() : null;

                    dados = await filterAsync(dados, async x =>
                        !(filtroNome && x.nome.toLowerCase().indexOf(filtroNome) === -1) &&
                        !($scope.filtros.status && $scope.filtros.status.valor.length > 0 && x.status !== $scope.filtros.status.valor) &&
                        !($scope.filtros.dataInicial && new Date(x.data_ini || x.data_create || x.data_start) < $scope.filtros.dataInicial) &&
                        !($scope.filtros.dataFinal && new Date(x.data_ini || x.data_create || x.data_start) > $scope.filtros.dataFinal)
                    );

                    let total = dados.length;
                    let page = Math.max($scope.filtro.page.valor - 1, 0) || 0;
                    let offset = page * PAGE_SIZE;
                    dados = dados.slice(offset, Math.min(PAGE_SIZE + offset, dados.length));

                    $scope.table = {
                        dados,
                        total: total,
                        page: $scope.filtro.page.valor,
                        pages: Math.max(Math.floor(total / PAGE_SIZE), 1),
                        offset: 0
                    };

                    console.timeEnd(`Filtrar`);
                }

                $scope.carregando = false;
            });

        }

        $scope.iniciarFiltragem = function () {
            $scope.table.dados = {}
            $scope.load();

            // let detectChanges = (a, b) => {
            //     if (a && b) {
            //         if (a.date !== b) return true;
            //     } else if (a !== b) return true;

            //     return false;
            // };

            // if (detectChanges($scope.filtro.data_agendamento_ini, $scope.filtros.dataInicial) ||
            //     detectChanges($scope.filtro.data_agendamento_fim, $scope.filtros.dataFinal) ||
            //     detectChanges($scope.filtro.data_create_ini, $scope.filtros.dataCreateInicial) ||
            //     detectChanges($scope.filtro.data_create_fim, $scope.filtros.dataCreateFinal)) {
            //     $scope.load();
            // } else {
            //     setTimeout(filtrar, 0);
            // }
        }

        $scope.pause = function (id) {
            var dados = {};
            dados.status = 'P';
            dados.data_status = new Date();
            $http.put(options.api.restv2 + "/campanha/" + id, dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>Campanha pausada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                });
        };

        function pauseAll () {
            var idMarcados = $scope.marcados.map(x => x.id);
            var dados = {};
            dados.CampanhaID = idMarcados
            dados.status = 'P';
            dados.data_status = new Date();
            $http.put(options.api.restv4 + "/campanha/status", dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: `<i class='fa fa-check'></i> <i>${idMarcados.length} campanhas pausadas com sucesso!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.markall = false;
                });
        };

        $scope.ConfirmaPausarTodos = () => {

            $scope.marcados = ($scope.table.dados || []).filter(x => x.marcado);

            if (!$scope.marcados || $scope.marcados.length <= 0) {
                return $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('Atenção!'),
                    content: $scope.getWord("Selecione ao menos uma campanha para pausar!"),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
            } else {
                $.x5Box({
                    title: $scope.getWord('Pausar todos os atendimento?'),
                    content: $scope.getWord(`<p>Tem certeza que deseja pausar ${$scope.marcados.length} campanhas ?</p>`) + "<p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                    color: "#c79121",
                    //timeout: 8000,
                    icon: "fa fa-warning shake animated"
                }, function (ButtonPressed) {
                    if (ButtonPressed === "Sim") {
                        pauseAll();
                    }
                });
            }
        };

        $scope.play = function (id) {
            var dados = {};
            dados.status = 'A';
            dados.data_status = new Date();
            $http.put(options.api.restv2 + "/campanha/" + id, dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>Campanha ativada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                });
        };

        function playAll (idMarcados) {
            var idMarcados = $scope.marcados.map(x => x.id);
            var dados = {};
            dados.CampanhaID = idMarcados;
            dados.status = 'A';
            dados.data_status = new Date();
            $http.put(options.api.restv4 + "/campanha/status", dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: `<i class='fa fa-check'></i> <i>${idMarcados.length} campanhas ativadas com sucesso!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.markall = false;
                });
        };

        $scope.ConfirmaAtivarTodos = () => {

            $scope.marcados = ($scope.table.dados || []).filter(x => x.marcado);

            if (!$scope.marcados || $scope.marcados.length <= 0) {
                return $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('Atenção!'),
                    content: $scope.getWord("Selecione ao menos uma campanha para ativar!"),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
            } else {
                $.x5Box({
                    title: $scope.getWord('Ativar todos os atendimento?'),
                    content: $scope.getWord(`<p>Tem certeza que deseja ativar ${$scope.marcados.length} campanhas ?</p>`) + "<p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                    color: "#c79121",
                    //timeout: 8000,
                    icon: "fa fa-warning shake animated"
                }, function (ButtonPressed) {
                    if (ButtonPressed === "Sim") {
                        playAll();
                    }
                });
            }
        };

        $scope.stop = function (id) {
            var dados = {};
            dados.status = 'F';
            dados.data_status = new Date();
            $http.put(options.api.restv2 + "/campanha/" + id, dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>Campanha parada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                });
        };

        function stopAll () {
            var idMarcados = $scope.marcados.map(x => x.id);
            var dados = {};
            dados.CampanhaID = idMarcados;
            dados.status = 'F';
            dados.data_status = new Date();
            $http.put(options.api.restv4 + "/campanha/status", dados)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: `<i class='fa fa-check'></i> <i>${idMarcados.length} campanhas paradas com sucesso!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.markall = false;
                });
        };

        $scope.ConfirmaPararTodos = () => {

            $scope.marcados = ($scope.table.dados || []).filter(x => x.marcado);

            if (!$scope.marcados || $scope.marcados.length <= 0) {
                return $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('Atenção!'),
                    content: $scope.getWord("Selecione ao menos uma campanha para parar!"),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
            } else {
                $.x5Box({
                    title: $scope.getWord('Parar todos os atendimento?'),
                    content: $scope.getWord(`<p>Tem certeza que deseja parar ${$scope.marcados.length} campanhas ?</p>`) + "<p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                    color: "#c79121",
                    //timeout: 8000,
                    icon: "fa fa-warning shake animated"
                }, function (ButtonPressed) {
                    if (ButtonPressed === "Sim") {
                        stopAll();
                    }
                });
            }
        };

        $scope.confirma_limpa = function (row) {
            $.x5Box({
                title: "Zerar a campanha <b>" + row.nome + "</b>?",
                content: $scope.getWord('<p>Essa ação zera os contatores da campanha para ela poder ser iniciada novamente.</p><p>Quando os contadores são zerados é como se a campanha nunca tivesse rodado.</p><p>Tem certeza que quer zerar essa campanha?</p>') + "<p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Sim") {
                    $scope.limpa(row.id);
                }
            });
        };

        $scope.limpa = function (id) {
            $http.delete(options.api.restv2 + "/campanha/" + id + "/zerar")
                .success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>Campanha zerada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.load();
                })
        };

        function zeraAll () {
            var idMarcados = $scope.marcados.map(x => x.id);
            $http.delete(options.api.restv4 + `/campanha/[${idMarcados}]/contatos`)
                .success(function (data) {
                    $scope.load();
                    $scope.showCampanhaPopup = false;
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: `<i class='fa fa-check'></i> <i>${idMarcados.length} campanhas zeradas com sucesso!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.markall = false;
                });
        };

        $scope.ConfirmaZerarTodos = () => {

            $scope.marcados = ($scope.table.dados || []).filter(x => x.marcado);

            if (!$scope.marcados || $scope.marcados.length <= 0) {
                return $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('Atenção!'),
                    content: $scope.getWord("Selecione ao menos uma campanha para zerar!"),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
            } else {
                $.x5Box({
                    title: $scope.getWord('Parar todos os atendimento?'),
                    content: $scope.getWord(`<p>Essa ação zera os contatores da campanha para ela poder ser iniciada novamente.</p><p>Quando os contadores são zerados é como se a campanha nunca tivesse rodado.</p><p>Tem certeza que quer zerar ${$scope.marcados.length} campanhas ?</p>`) + "<p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                    color: "#c79121",
                    //timeout: 8000,
                    icon: "fa fa-warning shake animated"
                }, function (ButtonPressed) {
                    if (ButtonPressed === "Sim") {
                        zeraAll();
                    }
                });
            }
        };

        $scope.ConfirmaDeletarTodos = async function () {

            $scope.marcados = ($scope.table.dados || []).filter(x => x.marcado);

            if (!$scope.marcados || $scope.marcados.length <= 0) {
                return $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('Atenção!'),
                    content: $scope.getWord("Selecione ao menos uma campanha para deletar!"),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });
            } else {
                let campanhaAtiva = [];
                campanhaAtiva = await ($scope.marcados || []).filter(x => x.status === "A" || x.status === "a");
                console.log(campanhaAtiva);
                if (campanhaAtiva && campanhaAtiva.length > 0) {
                    $.smallBox({
                        title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                        content: $scope.getWord('Não é possível deletar as campanhas!<br>Uma ou mais campanhas se encontram ativas, para excluir primeiro pare ela(s)!'),
                        color: "#C46A69",
                        icon: "fa fa-warning shake animated"
                    });
                    return;
                } else {
                    $.x5Box({
                        title: `Remover <b>${$scope.marcados.length}</b> campanhas ?`,
                        content: $scope.getWord('Tem certeza que deseja remover ') + $scope.marcados.length + " campanhas ? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                        color: "#C46A69",
                        //timeout: 8000,
                        icon: "fa fa-warning shake animated"
                    }, function (ButtonPressed) {
                        if (ButtonPressed === "Apagar") {
                            $scope.deleteAll();
                        }
                    });
                }
            };
        }

        $scope.deleteAll = function () {
            var idMarcados = $scope.marcados.map(x => x.id);
            $http.delete(options.api.restv4 + `/campanha/[${idMarcados}]`)
                .success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: `<i class='fa fa-check'></i> <i>${idMarcados.length} campanhas apagadas com sucesso!</i>`,
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.markall = false;
                    $scope.load();
                })
        };

        $scope.verificarStatusDaCampanha = function (id) {
            return new Promise((resolve, reject) => {
                $http.get(options.api.restv2 + "/campanha/" + id)
                    .success(data => {
                        try {
                            let statusCampanha = data.campanha.status;

                            // Verifica se a campanha está ativa
                            resolve(statusCampanha === "A" || statusCampanha === "a");
                        } catch (e) {
                            console.error(e);
                            reject(e);
                        }
                    });
            });
        };

        $scope.uploadImagem = function (id, file) {
            return new Promise((resolve, reject) => {
                file.upload = Upload.upload({
                    url: options.api.restv2 + "/campanha/" + id + "/imagem",
                    file: file
                });

                file.upload.then(
                    response => {
                        console.info("Imagem enviada com sucesso!", response);
                        resolve();
                    },
                    error => {
                        console.error(error);

                        $.smallBox({
                            title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                            content: $scope.getWord('Erro ao realizar upload da imagem!') + "\n" + error,
                            color: "#C46A69",
                            icon: "fa fa-warning shake animated"
                        });

                        reject(error);
                    },
                    event => {
                        file.progress = parseInt(100.0 * event.loaded / event.total);
                        console.log("File upload progress -> ", file.progress);
                    }
                );
            });
        };

        $scope.uploadArquivo = function (id, file) {
            return new Promise((resolve, reject) => {
                file.upload = Upload.upload({
                    url: options.api.restv2 + "/campanha/" + id + "/arquivo",
                    file: file
                });

                file.upload.then(
                    response => {
                        console.info("Arquivo enviado com sucesso!", response);
                        resolve();
                    },
                    error => {
                        console.error(error);

                        $.smallBox({
                            title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                            content: $scope.getWord('Erro ao realizar upload de arquivo!') + "\n" + error,
                            color: "#C46A69",
                            icon: "fa fa-warning shake animated"
                        });

                        reject(error);
                    },
                    event => {
                        file.progress = parseInt(100.0 * event.loaded / event.total);
                        console.log("File upload progress -> ", file.progress);
                    }
                );
            });
        };

        $scope.getImagemUrl = function () {
            return options.api.gravacao + $scope.campanha.caminho_imagem_campanha;
        };

        $scope.SaveCampanha = async function (f1) {
            if (f1.$valid) {
                // Verifica o status da campanha
                if ($scope.campanha && $scope.campanha.id) {
                    let campanhaAtiva = await $scope.verificarStatusDaCampanha($scope.campanha.id);
                    if (campanhaAtiva) {
                        $.smallBox({
                            title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                            content: $scope.getWord('Não é possível alterar a campanha!<br>A Campanha se encontra ativa, para fazer alterações na campanha, primeiramente pause ela!'),
                            color: "#C46A69",
                            icon: "fa fa-warning shake animated"
                        });

                        return;
                    }
                }

                // Verifica se o usuário marcou a checkbox de imagem e não a selecionou nenhuma imagem a enviar
                if ($scope.campanha.mensagem_padrao_imagem && (!$scope.campanha.imagem && !$scope.campanha.caminho_imagem_campanha)) {
                    return;
                }

                // Verifica se o usuário marcou o checkbox de arquivo e não selecionou nenhum arquivo a enviar
                if ($scope.campanha.mensagem_padrao_arquivo && (!$scope.campanha.arquivo && !$scope.campanha.caminho_arquivo_campanha)) {
                    return;
                }

                $scope.campanha.load = true;
                var dados = angular.copy($scope.campanha);
                dados.tempo_autopausa_agente = 0;
                delete dados.id;
                delete dados.processamento;
                delete dados.data_final;
                delete dados.data_inicial;
                delete dados.classificacao_reabre_contato;
                delete dados.classificacao_remove_contato;
                delete dados.classificacao_remove_numero;
                delete dados.classificacao_reagenda_contato;
                delete dados.total_contatos;
                delete dados.total_numeros;
                delete dados.total_discados;
                delete dados.completados;
                delete dados.falhas;
                delete dados.load;
                delete dados.turnoSelecionado;
                delete dados.imagem;
                delete dados.caminho_imagem_campanha;
                delete dados.caminho_arquivo_campanha;
                delete dados.arquivo;
                delete dados.data_create;
                delete dados.data_start;
                delete dados.total_discados_atual;

                if (!dados.mensagem_padrao_texto) {
                    dados.texto_padrao_campanha = null;
                }

                // Para campanhas já existentes, habilita a desativação das imagens
                if (!dados.mensagem_padrao_imagem && $scope.campanha.id) {
                    dados.caminho_imagem_campanha = null;
                    dados.legenda_imagem_campanha = null;
                }
                if (!dados.legenda_imagem_campanha || dados.legenda_imagem_campanha.length <= 0) dados.legenda_imagem_campanha = null;

                // Para campanhas já existentes, habilita a exclusão do arquivo
                if (!dados.mensagem_padrao_arquivo && $scope.campanha.id) {
                    dados.caminho_arquivo_campanha = null;
                }

                delete dados.mensagem_padrao_texto;
                delete dados.mensagem_padrao_imagem;
                delete dados.mensagem_padrao_arquivo;


                if (dados.turnos) dados.turnos = dados.turnos.map(x => x.id);
                else delete dados.turnos;

                dados.exec_contexto = $scope.popup && $scope.popup.go_contexto ? dados.exec_contexto : null;

                // Define o tipo de campanha a enviar para a API
                switch ($scope.roles.getMode()) {
                    case "normal": {
                        dados.tipo_campanha = 0;
                        dados.ddi_padrao = null;
                        delete dados.texto_padrao_campanha;
                        break;
                    }
                    case "zenvia":
                    case "whatsapp": {
                        dados.tipo_campanha = 1;

                        if (!dados.nao_requer_ddi) {
                            dados.ddi_padrao = dados.ddi.trim();
                        } else {
                            dados.ddi_padrao = null
                        }
                        break;
                    }
                    default: throw "Tipo de campanha não reconhecido!"; break;
                }

                delete dados.ddi;
                delete dados.nao_requer_ddi;

                if (!dados.data_ini || dados.data_ini === null || dados.data_ini === 'null') {
                    dados.data_ini = undefined
                } else {
                    dados.data_ini = dados.data_ini.toISOString()
                    let antiga = dados.data_ini.substr(11, 8)
                    dados.data_ini = dados.data_ini.replace(antiga, `${$scope.campanha.hora_ini}:00`);

                    dados.data_ini = moment(dados.data_ini).utc().format('YYYY-MM-DD HH:mm:ss')
                }

                if (!dados.data_fim || dados.data_fim === null || dados.data_fim === 'null') {
                    dados.data_fim = undefined
                } else {
                    dados.data_fim = dados.data_fim.toISOString()
                    let antiga = dados.data_fim.substr(11, 8)
                    dados.data_fim = dados.data_fim.replace(antiga, `${$scope.campanha.hora_fim}:59`);

                    dados.data_fim = moment(dados.data_fim).utc().format('YYYY-MM-DD HH:mm:ss')
                }

                // dados.data_ini = !dados.data_ini || dados.data_ini === null || dados.data_ini === 'null' ? undefined : dados.data_ini.toISOString();
                // dados.data_fim = !dados.data_fim || dados.data_fim === null || dados.data_fim === 'null' ? undefined : dados.data_fim.toISOString();
                dados.data_status = !dados.data_status || dados.data_status === null || dados.data_status === 'null' ? undefined : dados.data_status;
                dados.descricao = !dados.descricao || dados.descricao === null || dados.descricao === 'null' ? undefined : dados.descricao;
                dados.rota = (typeof dados.rota === 'object') ? JSON.stringify(dados.rota) : dados.rota;

                if (!dados.agendamento) {
                    dados.data_ini = null;
                    dados.data_fim = null;
                }
                delete dados.agendamento;
                delete dados.hora_ini;
                delete dados.hora_fim;

                if (!$scope.campanha.notificar_email) {
                    delete dados.notificar_email;
                }

                if ($scope.campanha.id) {
                    $http.put(options.api.restv2 + "/campanha/" + $scope.campanha.id, dados).
                        success(async data => {
                            /** Verifica se a campanha possui uma imagem {@link CampanhasWhatsapp} **/
                            if ($scope.campanha.imagem) await $scope.uploadImagem($scope.campanha.id, $scope.campanha.imagem);
                            if ($scope.campanha.arquivo) await $scope.uploadArquivo($scope.campanha.id, $scope.campanha.arquivo);

                            ajustaTags();
                        });
                } else {
                    $http.post(options.api.restv2 + "/campanha", dados).
                        success(async data => {
                            /** Verifica se a campanha possui uma imagem {@link CampanhasWhatsapp} **/
                            if ($scope.campanha.imagem) await $scope.uploadImagem(data.id, $scope.campanha.imagem);
                            if ($scope.campanha.arquivo) await $scope.uploadArquivo(data.id, $scope.campanha.arquivo);

                            ajustaTags(data.id);
                        });
                }
            } else {
                alert($scope.getWord('Existem campos obrigatórios não preenchidos.'));
                console.log('Form invalido');
            }
        };

        function ajustaTags (id) {
            if (id) $scope.campanha.id = id;

            if (($scope.campanha.classificacao_reabre_contato && $scope.campanha.classificacao_reabre_contato.length > 0)
                || ($scope.campanha.classificacao_remove_contato && $scope.campanha.classificacao_remove_contato.length > 0)
                || ($scope.campanha.classificacao_remove_numero && $scope.campanha.classificacao_remove_numero.length > 0)
                || ($scope.campanha.classificacao_reagenda_contato && $scope.campanha.classificacao_reagenda_contato.length > 0)
            ) {
                console.log('Temos tags para atualizar...lets go it');
                var classificacoes = [];
                for (var i = 0; i < $scope.campanha.classificacao_reabre_contato.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_reabre_contato[i].id,
                        acao: 'AN',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                for (var i = 0; i < $scope.campanha.classificacao_remove_contato.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_remove_contato[i].id,
                        acao: 'RC',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                for (var i = 0; i < $scope.campanha.classificacao_remove_numero.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_remove_numero[i].id,
                        acao: 'RN',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                for (var i = 0; i < $scope.campanha.classificacao_reagenda_contato.length; ++i) {
                    if (!$scope.campanha.classificacao_reagenda_contato[i].tags
                        || $scope.campanha.classificacao_reagenda_contato[i].tags.length <= 0
                        || !$scope.campanha.classificacao_reagenda_contato[i].horas_apos
                    ) continue;
                    var c1 = {
                        id: $scope.campanha.classificacao_reagenda_contato[i].tags[0].id,
                        acao: 'RG',
                        fila: 'f',
                        nome: 'c',
                        horas_apos: $scope.campanha.classificacao_reagenda_contato[i].horas_apos,
                    };
                    classificacoes.push(c1);
                }
                $http.put(options.api.restv4 + "/campanha/" + $scope.campanha.id + '/tags', classificacoes).
                    success(function (data) {
                        $scope.load();
                        $scope.campanha.load = false;
                        $scope.showCampanhaPopup = false;
                        $.smallBox({
                            title: $scope.getWord('SUCESSO') + "!!",
                            content: "<i class='fa fa-check'></i> <i>Campanha atualizada com sucesso!</i>",
                            color: "#739E73",
                            timeout: 2000
                        });
                    });
            }
            else {
                $scope.load();
                $scope.showCampanhaPopup = false;
                $.smallBox({
                    title: $scope.getWord('SUCESSO') + "!!",
                    content: "<i class='fa fa-check'></i> <i>Campanha editada com sucesso!</i>",
                    color: "#739E73",
                    timeout: 2000
                });
                //Vamos limpar as classificações, pois se não temos nenhuma, agora, quem disse que não tinhamos antes?
                /* $http.delete(options.api.restv2+"/campanha/"+$scope.campanha.id+'/classificacoes').
                    success(function(data) {
                        $scope.load();
                        $scope.campanha.load = false;
                        $scope.showCampanhaPopup = false;
                        $.smallBox({
                            title : $scope.getWord('SUCESSO')+"!!",
                            content : "<i class='fa fa-check'></i> <i>Campanha atualizada com sucesso!</i>",
                            color : "#739E73",
                            timeout : 2000
                        });
                    }); */
            }
        }

        function ajustaClassicacoes (id) {
            if (id) $scope.campanha.id = id;

            if (($scope.campanha.classificacao_reabre_contato && $scope.campanha.classificacao_reabre_contato.length > 0)
                || ($scope.campanha.classificacao_remove_contato && $scope.campanha.classificacao_remove_contato.length > 0)
                || ($scope.campanha.classificacao_remove_numero && $scope.campanha.classificacao_remove_numero.length > 0)
            ) {
                console.log('Temos classificações para atualizar...lets go it');
                var classificacoes = [];
                for (var i = 0; i < $scope.campanha.classificacao_reabre_contato.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_reabre_contato[i].id,
                        acao: 'AN',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                for (var i = 0; i < $scope.campanha.classificacao_remove_contato.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_remove_contato[i].id,
                        acao: 'RC',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                for (var i = 0; i < $scope.campanha.classificacao_remove_numero.length; ++i) {
                    var c1 = {
                        id: $scope.campanha.classificacao_remove_numero[i].id,
                        acao: 'RN',
                        fila: 'f',
                        nome: 'c'
                    };
                    classificacoes.push(c1);
                }
                $http.put(options.api.restv2 + "/campanha/" + $scope.campanha.id + '/classificacoes', classificacoes).
                    success(function (data) {
                        $scope.load();
                        $scope.campanha.load = false;
                        $scope.showCampanhaPopup = false;
                        $.smallBox({
                            title: $scope.getWord('SUCESSO') + "!!",
                            content: "<i class='fa fa-check'></i> <i>Campanha atualizada com sucesso!</i>",
                            color: "#739E73",
                            timeout: 2000
                        });
                    });
            }
            else {
                $scope.load();
                $scope.showCampanhaPopup = false;
                $.smallBox({
                    title: $scope.getWord('SUCESSO') + "!!",
                    content: "<i class='fa fa-check'></i> <i>Campanha editada com sucesso!</i>",
                    color: "#739E73",
                    timeout: 2000
                });
                //Vamos limpar as classificações, pois se não temos nenhuma, agora, quem disse que não tinhamos antes?
                /* $http.delete(options.api.restv2+"/campanha/"+$scope.campanha.id+'/classificacoes').
                    success(function(data) {
                        $scope.load();
                        $scope.campanha.load = false;
                        $scope.showCampanhaPopup = false;
                        $.smallBox({
                            title : $scope.getWord('SUCESSO')+"!!",
                            content : "<i class='fa fa-check'></i> <i>Campanha atualizada com sucesso!</i>",
                            color : "#739E73",
                            timeout : 2000
                        });
                    }); */
            }
        }

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.campanhas');
        };

        /* $scope.$watch('Pesquisa', function (t1, t2) {
            // Se os dois valores estirem em branco nao faz nada, isso aconte no load.
            if (!t1 && !t2)
                return false;

            setTimeout(function () {
                if (t1 === $scope.Pesquisa) {
                    if (t1 !== '') {
                        $scope.filtro.nome = { valor: t1 };
                    }
                    else {
                        $scope.filtro.nome = undefined;
                    }

                    $scope.load();
                }
            }, 1000);
        }); */

        $scope.confirma = async function (row) {
            // Verifica o status da campanha
            let campanhaAtiva = await $scope.verificarStatusDaCampanha(row.id);
            if (campanhaAtiva) {
                $.smallBox({
                    title: "<i class='glyphicon glyphicon-remove'></i> " + "Atenção!",
                    content: $scope.getWord('Não é possível deletar a campanha!<br>A Campanha se encontra ativa, para excluir primeiro pare ela!'),
                    color: "#C46A69",
                    icon: "fa fa-warning shake animated"
                });

                return;
            }

            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: $scope.getWord('Tem certeza que quer remover') + row.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.confirmalista = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.lista + "</b> da campanha?",
                content: "Tem certeza que quer remover a lista " + row.lista + " da campanha? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.deletelista(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            // Deleta a campanha
            $http.delete(options.api.restv2 + "/campanha/" + id).
                success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>Campanha apagada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 2000
                    });

                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                return $scope.order[0].ordem === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            }
            else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            //console.log($scope.order);
            $scope.load();
        };

        $scope.formatadata = function (d) {
            d = d.split(' ');
            var h = d[1].split('-');
            var d1 = d[0].split('-');

            var calldate = d1[2] + "/" + d1[1] + "/" + d1[0] + ' ' + h[0];
            return $filter("date")(calldate, 'dd-MM-yy HH:mm:ss')
        };

    });
});
