define(['modules/campanhas/module'], function (module) {
    "use strict";

    return module.registerController('DiscadorCtrl', function ($scope, SocketService, $http, Upload, $timeout, $stateParams, $filter, $rootScope) {

        // Definição de variáveis de escopo
        $scope.loading = true;
        $scope.campanha = {};
        $scope.status = null;
        $scope.tempoCorrido = 0;
        $scope.handleTempoCorrido = null;
        $scope.handleAtualizacaoStatus = null;

        $scope.onInit = function () {
            // Carrega a campanha no escopo
            $scope.loadCampanha();

            // Atualiza o tempo corrido de tempos em tempos
            $scope.handleTempoCorrido = setInterval(() => {
                if ($scope.status && $scope.status.data_status)
                    $scope.tempoCorrido = $scope.calcularTempoCorrido($scope.status.data_status)
            }, 1000);

            // De tempos em tempos requisita o objeto inteiro do status da campanha
            $scope.handleAtualizacaoStatus = setInterval(() => {
                SocketService.emit("message", { action: "DepuradorCampanhasIniciar", id: $stateParams.id });
            }, 10000);
        };

        $scope.$on("$destroy", () => {
            if ($scope.handleTempoCorrido) {
                clearInterval($scope.handleTempoCorrido);
            }

            if ($scope.handleAtualizacaoStatus) {
                clearInterval($scope.handleAtualizacaoStatus);
            }

            SocketService.emit("message", { action: "DepuradorCampanhasFinalizar", id: $stateParams.id });
        });

        $scope.loadCampanha = function () {
            return new Promise((resolve, reject) => {
                $scope.loading = true;

                $http.get(`${options.api.restv3}/campanha/${$stateParams.id}`)
                    .success(data => {
                        try {
                            $scope.campanha = $scope.status = data.dados;
                            resolve($scope.campanha);
                        } catch (e) {
                            $scope.loading = false;
                            console.error(e);
                            reject(e);
                        }
                    })
                    .error(e => {
                        console.error(e);
                        reject(e);
                    });
            });
        };

        $scope.calculateMaxLigacoesAtivas = function () {
            let maximo = 0;

            let ligacoesAtivas = $scope.status && $scope.status.ligacoes_ativas ? $scope.status.ligacoes_ativas : [];

            if ($scope.campanha.requer_agente_online) {
                let agentes = $scope.status && $scope.status.agentes ? $scope.status.agentes : [];
                if ($scope.campanha.max_por_agente > 1 && $scope.campanha.desconecta_multiplas) {
                    maximo = agentes.length - ligacoesAtivas.length;
                } else {
                    maximo = agentes.length * $scope.campanha.max_por_agente - ligacoesAtivas.length;
                }
            } else {
                maximo = $scope.campanha.max_simultaneas_campanha - ligacoesAtivas.length;
            }

            if (maximo > $scope.campanha.max_simultaneas_campanha) {
                maximo = $scope.campanha.max_simultaneas_campanha - ligacoesAtivas.length;
            }

            return maximo;
        };

        $scope.getStatusCampanha = function () {
            if ($scope.status.status_descricao) {
                return $scope.status.status_descricao;
            } else if ($scope.status.rodando) {
                return $scope.campanha.status === "A" ? "Stand by" : "Finalizada";
            } else {
                return "Parada";
            }
        };

        $scope.resetCampanha = async function () {
            $scope.loading = true;

            await $http.delete(options.api.restv2 + "/campanha/" + $stateParams.id + "/zerar")
            await $http.put(options.api.restv2 + "/campanha/" + $stateParams.id, { status: 'A', data_status: new Date() })
            $scope.loadCampanha();
        };

        $scope.calculateFailurePercent = function () {
            if (!$scope.status || !$scope.status.registros) return 0;

            let percent = Math.floor(($scope.status.registros.falhas / $scope.status.registros.total) * 100).toString();
            if (isNaN(percent) || !percent) percent = 0;
            return percent;
        };

        $scope.calcularTempoCorrido = function (inicio) {
            if (!inicio) return 0;
            try {
                var moment = require('moment');
                return moment().diff(moment(inicio), "seconds");
            } catch (e) {
                return 0;
            }
        };

        //#region Socket handling
        SocketService.on('open', function () {
            console.log('Conexão com websocket aberta');
            var token = { token: "H91A+3zz_-.Y18DH1YeQP65l?G,Q7)F2agm|mq^ukdh,]k,Z[a@195uJhK3n-{" };
            SocketService.emit('authentication', JSON.stringify(token));
            SocketService.emit("message", { action: "StatusDiscador", id: $stateParams.id });
        });

        SocketService.on('authenticated', function () {
            console.log('Autenticou');
            SocketService.emit("message", { action: "StatusDiscador", id: $stateParams.id });
        });

        SocketService.on('disconnect', function (err) {
            console.log('### CONEXÃO FECHADA ###');
        });

        SocketService.on('connect_failed', function (err) {
            console.log('### FALHA NA CONEXAO, verifique se o serviço está rodando no servidor ###');
        });

        SocketService.on('connect_error', function (err) {
            console.log('### FALHA NA CONEXAO, verifique se o serviço está rodando no servidor ###');
        });

        SocketService.on('cadastra_contato_campanha_conclusion', function (event) {
            console.log("Cadastra contato campanha conclusão", event);
            $.smallBox({
                title: "SUCESSO!!",
                content: "<i class='fa fa-check'></i> <i>Clientes cadastrados com sucesso!</i>",
                color: "#739E73",
                timeout: 3000
            });
        });

        SocketService.on('cadastra_contato_campanha_error', function (event) {
            console.log("Cadastra contato campanha erro", event);
            $.smallBox({
                title: "ERRO!!",
                content: "<i class='fa fa-check'></i> <i>Erro ao cadastrar clientes!</i>",
                color: "#C46A69",
                timeout: 3000
            });
        });

        SocketService.on('cadastra_contato_campanha_progress', function (event) {
            console.log("Cadastra contato campanha progresso", event);
        })

        SocketService.on('upsert_contato_campanha_conclusion', function (event) {
            console.log("Upsert contato campanha conclusão", event);
            $.smallBox({
                title: "SUCESSO!!",
                content: "<i class='fa fa-check'></i> <i>Clientes atualizados com sucesso!</i>",
                color: "#739E73",
                timeout: 3000
            });
        });

        SocketService.on('upsert_contato_campanha_error', function (event) {
            console.log("Upsert contato campanha erro", event);
            $.smallBox({
                title: "ERRO!!",
                content: "<i class='fa fa-check'></i> <i>Erro ao atualizar clientes!</i>",
                color: "#C46A69",
                timeout: 3000
            });
        })

        SocketService.on('upsert_contato_campanha_progress', function (event) {
            console.log("Upsert contato campanha progresso", event);
        })

        SocketService.on('message', function (event) {
            var response = JSON.parse(event);
            if (response.event === 'StatusDiscador' && response.id == $stateParams.id) {
                console.log("DEPURADOR -> ", response.alteracoes);
                if (!$scope.status) $scope.status = {};
                for (let property in response.alteracoes) {
                    $scope.status[property] = response.alteracoes[property];
                }

                $scope.loading = false;
            }
        });
        //#endregion


    });
});
