define(['modules/campanhas/module'], function (module) {
  "use strict";
  return module.registerController('RegistrosCtrl_old', function ($scope, $q, $http, $filter, exportFilesService, paginationService, $localStorage) {
    $scope.carregando = false;
    $scope.sortBy = 'data_status';
    $scope.sortDescending = false;
    $scope.searchText = '';

    $scope.maxSize = 3; // Máximo de páginas exibidas no bloco de páginação
    $scope.currentPage = 1; // Página Atual
    $scope.numPerPage = 10; // Registros por página 

    $scope.getCurPage = () => {
      return paginationService.getCurrentPage('pag');
    }

    $scope.getTotalPages = () => {
      return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
    }

    $scope.getTotalItens = () => {
      return paginationService.getCollectionLength('pag');
    }

    $scope.grafico_por = [
      $scope.getWord("status"),
      $scope.getWord("classificacao")
    ];

    $scope.grafico = $scope.grafico_por[0];

    $scope.onAddDesagrupar = (tag) => {
      console.log('Tag add');
      console.log(tag);
      $scope.grafico_por.push(tag.text);
    }

    $scope.onDelDesagrupar = (tag) => {
      console.log('Tag Del');
      console.log(tag);
      // Se a tag Removida é a atual do grafico volta pra padrão.
      if ($scope.grafico == tag.text) {
        $scope.grafico = $scope.grafico_por[0];
      }
      const i = $scope.grafico_por.indexOf(tag.text);
      $scope.grafico_por.splice(i, 1);
    }

    $scope.grafico_log = {
      chart: {
        plotBackgroundColor: null,
        plotBorderWidth: null,
        plotShadow: false,
        type: 'pie'
      },
      credits: {
        enabled: true,
        text: 'X5 Soluções - Xcontact',
        href: 'http://www.x5s.com.br'
      },
      title: {
        text: $scope.getWord('Estatísticas das campanhas'),
        useHTML: true
      },
      tooltip: {
        pointFormat: '{series.status}: <b>{point.percentage:.1f}%</b>'
      },
      plotOptions: {
        pie: {
          allowPointSelect: true,
          cursor: 'pointer',
          dataLabels: {
            enabled: true,
            format: '<b>{point.name}</b>: {point.percentage:.1f} %',
            style: {
              color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
            }
          },
          showInLegend: true
        }
      }
    };

    $scope.Limpa = function (r) {
      return JSON.stringify(r).replace(/[{}"\[\]]/g, '').replace(',', ', ');
    };

    $scope.order = function (keyname) {
      $scope.sortBy = keyname;
      $scope.sortDescending = !$scope.sortDescending;
      $scope.filter();
    }

    $scope.paginate = function (value) {
      var begin, end, index;
      begin = ($scope.currentPage - 1) * $scope.numPerPage;
      end = begin + $scope.numPerPage;
      index = $scope.filteredRT.indexOf(value);
      return (begin <= index && index < end);
    };

    $scope.ToCSV = () => {
      $scope.filter();

      $scope.carregando_export = true;
      let arr = [];

      // Formata cabeçalho
      let cab = [
        $scope.getWord('Data'),
        $scope.getWord('Campanha'),
        $scope.getWord('Numero'),
        $scope.getWord('Duracao'),
        $scope.getWord('Status'),
        $scope.getWord('Classificacao'),
        $scope.getWord('Observacao'),
      ];
      if ($scope.filtro.desagrupar && $scope.filtro.desagrupar.length > 0) {
        for (let campo of $scope.filtro.desagrupar) {
          cab.push(campo.text);
        }
      }
      arr.push(cab);

      // Formata dados
      $scope.filteredRT.forEach((e) => {
        var linha = [];
        linha.push($filter('date')(e.data_status, 'dd/MM/yyyy HH:mm:ss'));
        linha.push(e.campanha);
        linha.push(e.numero);
        linha.push(e.duracao);
        linha.push(e.status);
        linha.push(e.classificacao);
        linha.push(e.observacao);
        if ($scope.filtro.desagrupar && $scope.filtro.desagrupar.length > 0) {
          for (let campo of $scope.filtro.desagrupar) {
            linha.push(e.campos[campo.text]);
          }
        }
        arr.push(linha);
      });

      exportFilesService.gerarCSV(arr, "Registro das campanhas.csv").then(
        dados => {
          console.log(`Sucesso na geraCSV, ${dados}`);
          $scope.carregando_export = false;
        },
        erro => {
          console.log(`Erro na geraCSV ${erro}`);
          $scope.carregando_export = false;
        }
      )
    }

    $scope.filter = () => {
      console.log('Inicio');
      $scope.carregando_filtro = true;
      return $q.when(runFilter().then(function (value) {
        console.log('Fim');
        $scope.carregando_filtro = false;
        $scope.filteredRT = value;
        $scope.getGraf();
      }));
    }

    function runFilter () {
      return new Promise(function (resolve, reject) {
        try {
          console.log('Antes no copy');
          //let filteredRT = angular.copy($scope.reminderTypes);
          let filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
          console.log('Depois do copy');

          // Primeira coisa a fazer é desagrupar para poder aplicar os filtros depois
          if ($scope.filtro.desagrupar && $scope.filtro.desagrupar.length > 0) {
            let arrayDesagruparNomes = $scope.filtro.desagrupar.map(item => { return item.text; });

            filteredRT = filteredRT.map(item => {
              for (let i = 0; i < arrayDesagruparNomes.length; i++) {
                let campo = arrayDesagruparNomes[i];
                if (item.campos) {
                  item[campo] = item.campos[campo] ? item.campos[campo] : '';
                }
              }
              return item;
            });
          }

          if ($scope.filtro.desagrupado && Object.keys($scope.filtro.desagrupado).length > 0) {
            let keys = Object.keys($scope.filtro.desagrupado);
            for (let i = 0; i < keys.length; i++) {
              let key = keys[i];
              let arrayNomes = $scope.filtro.desagrupado[key].map(item => { return item.text; });
              if (arrayNomes.length > 0) {
                filteredRT = filteredRT.filter(item => arrayNomes.includes(item[key]));
              }
            }
          }

          if ($scope.filtro.numero && $scope.filtro.numero.length > 0) {
            let arrayNomes = $scope.filtro.numero.map(item => { return item.text; });
            filteredRT = filteredRT.filter(item => arrayNomes.includes(item.numero));
          }
          if ($scope.filtro.classificacao && $scope.filtro.classificacao.length > 0) {
            let arrayNomes = $scope.filtro.classificacao.map(item => { return item.text; });
            filteredRT = filteredRT.filter(item => arrayNomes.includes(item.classificacao));
          }

          console.log('Antes do filter');
          if ($scope.filtro.status && $scope.filtro.status.length > 0) {
            let arrayStatusNome = $scope.filtro.status.map(item => { return item.nome; });
            filteredRT = filteredRT.filter(item => arrayStatusNome.includes(item.status));
          }
          console.log('Depois do filter');

          if ($scope.filtro.campanhas && $scope.filtro.campanhas.length > 0) {
            let arrayCampanhasNome = $scope.filtro.campanhas.map(item => { return item.nome; });
            filteredRT = filteredRT.filter(item => arrayCampanhasNome.includes(item.campanha));
          }
          if ($scope.filtro.campo && $scope.filtro.campo.length > 0) {
            let arrayCampos = [];
            for (let i = 0; i < $scope.filtro.campo.length; i++) {
              let e = $scope.filtro.campo[i];
              if (!e.text.includes(':')) {
                $.smallBox({
                  title: "<i class='glyphicon glyphicon-remove'></i> " + $scope.getWord('ERRO'),
                  content: $scope.getWord("Formato inválido para") + ' ' + e.text + ' ' + $scope.getWord("Use: Campo:Valor"),
                  color: "#C46A69",
                  icon: "fa fa-warning shake animated"
                });
              }
              else {
                let ob = e.text.split(':');
                arrayCampos.push({ nome: ob[0].trim(), valor: ob[1].trim().toLocaleLowerCase() });
              }
            }
            if (arrayCampos.length > 0) {
              filteredRT = filteredRT.filter(e => {
                if (e.campos && Object.keys(e.campos).length > 0) {
                  return arrayCampos.find(campo => {
                    return e.campos[campo.nome] && e.campos[campo.nome].toLowerCase().includes(campo.valor);
                  });
                }
              });
            }
          }

          console.log('Antes do sort');
          filteredRT.sort(function (a, b) {
            var sortBy = $scope.sortBy;
            if (a[sortBy] < b[sortBy]) {
              return $scope.sortDescending ? 1 : -1;
            }
            if (a[sortBy] > b[sortBy]) {
              return $scope.sortDescending ? -1 : 1;
            }
            return 0;
          });
          console.log('Depois do sort');

          resolve(filteredRT);
        } catch (e) {
          reject(e);
        }
      });
    }

    $scope.init = function () {
      $scope.busca = {};
      $scope.filtro = {};
      $scope.busca.dataini = new Date();
      $scope.busca.datafim = new Date();
      getStatus();
      $scope.load();
      //$scope.getGraf();
    };

    function getCampanhas () {
      let filas = `filas=${encodeURI(JSON.stringify($localStorage.User.filas))}`
      var order = [{ "campo": "nome", "ordem": "ASC" }]
      return new Promise((resolve, reject) => {
        $http.get(options.api.restv3 + `/campanha?max=-1&order=${encodeURI(JSON.stringify(order))}&${filas}`).
          success(function (data) {
            try {
              $scope.campanhas = data.dados;
              resolve($scope.campanha)
            } catch (e) {
              console.error(e);
              reject(e);
            }
          });
      });
    };

    $scope.filterCampanhas = function (query) {
      return $scope.campanhas.filter(function (o) {
        return o.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
      });
    };

    function getStatus () {
      var order = [{ "campo": "nome", "ordem": "ASC" }]
      $http.get(options.api.restv2 + "/ligacoes/status?max=-1&order=" + encodeURI(JSON.stringify(order))).
        success(function (data) {
          $scope.status = data.dados;
        });
    };

    $scope.filterStatus = function (query) {
      return $scope.status.filter(function (o) {
        return o.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
      });
    };

    $scope.load = async function () {
      await getCampanhas();
      $scope.carregando = true;

      const busca = {};
      $scope.busca.dataini = $scope.busca.dataini ? $scope.busca.dataini : new Date();
      $scope.busca.datafim = $scope.busca.datafim ? $scope.busca.datafim : new Date();

      busca.data_ini = $scope.busca.horaini ? $filter('date')($scope.busca.dataini, 'yyyy-MM-dd') + ' ' + $scope.busca.horaini + ':00' : $filter('date')($scope.busca.dataini, 'yyyy-MM-dd') + ' 00:00:00';
      busca.data_fim = $scope.busca.horafim ? $filter('date')($scope.busca.datafim, 'yyyy-MM-dd') + ' ' + $scope.busca.horafim + ':00' : $filter('date')($scope.busca.datafim, 'yyyy-MM-dd') + ' 23:59:00';

      var par = [];
      angular.forEach(busca, function (v, k) {
        par.push(k + "=" + v);
      });

      let idsCampanhas = $scope.campanhas.map(c => c.id);

      $http.get(options.api.restv2 + "/campanhas/registros?max=-1&" + par.join("&")).
        success(function (data) {
          if (data.dados) {
            data.dados = data.dados.map((d) => {
              d.status = $scope.getWord(d.status);
              return d;
            })
            data.dados = data.dados.filter((d) => idsCampanhas.includes(d.id_campanha));
            console.log(data.dados)

            $scope.reminderTypes = data.dados;
            $scope.filter();
          }
          $scope.carregando = false;
        });
    };

    $scope.getClass = function (coluna) {
      if ($scope.sortBy === coluna) {
        return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
      }
      else {
        return 'sorting';
      }
    };

    $scope.getGraf = function () {
      console.log('Get Graf');
      $scope.loading_graf = true;

      var ini = $scope.busca.horaini ? $scope.busca.horaini : '';
      var fim = $scope.busca.horafim ? $scope.busca.horafim : '';
      $scope.grafico_log.subtitle = {
        text: $scope.getWord('De') + ' ' + $filter('date')($scope.busca.dataini, 'dd/MM/yyyy') + ' ' + ini + ' ' + $scope.getWord('até') + ' ' + $filter('date')($scope.busca.datafim, 'dd/MM/yyyy') + ' ' + fim
      };

      var serie = [];
      //console.log('Grafico por: ');
      //console.log($scope.grafico);

      var result = _.groupBy($scope.filteredRT, $scope.grafico === 'clasificación' ? 'classificacao' : $scope.grafico);


      angular.forEach(result, function (v, k) {
        var vet = [k, parseFloat(((parseInt(v.length) / $scope.filteredRT.length) * 100).toFixed(2))];
        serie.push(vet);
      });

      $scope.grafico_log.title = {
        text: $scope.getWord('Estatísticas das campanhas') + '- <b>' + $scope.filteredRT.length + ' ' + $scope.getWord('Ligações') + '</b></span>'
      };
      $scope.grafico_log.series = [
        {
          name: 'Total',
          data: serie
        }
      ];
      $scope.loading_graf = false;
    };

  });
});
