define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-csv',
    'highcharts',
    'highcharts-export',
    'angular-highcharts',
    'angular-ws',
    'angular-socket-io',
    'ng-file-upload',
    'underscore',
    'angular-desktop-notification',
    'angularUtils.directives.dirPagination',
], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.campanhas', [
        'ui.router',
        'ngResource',
        'ngCsv',
        'highcharts-ng',
        'ws',
        'btford.socket-io',
        'ngFileUpload',
        'ngDesktopNotification',
        'angularUtils.directives.dirPagination',
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.campanhas', {
                url: '/campanhas',
                data: {
                    title: 'Campanhas',
                    permissions: {
                        only: ['app.campanhas']
                    }
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/lista.html',
                        controller: 'CampanhaCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/servicos/SocketService',
                                'modules/campanhas/directives/campanhaPopup',
                                'modules/campanhas/controllers/CampanhaCtrl',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })
            .state('app.campanhas.campanha_contatos', {
                url: '/campanha_contatos/:id',
                data: {
                    title: 'Contatos da campanha'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/contatos.html',
                        controller: 'ContatosCtrl',
                        parent: 'app.campanhas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/servicos/SocketService',
                                'modules/campanhas/controllers/ContatosCtrl',
                                'modules/campanhas/directives/contatoPopup'
                            ])
                        }
                    }
                }
            })
            .state('app.campanhas.discador', {
                url: '/discador/:id',
                data: {
                    title: 'Depuração de discador'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/discador.html',
                        controller: 'DiscadorCtrl',
                        parent: 'app.campanhas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/controllers/DiscadorCtrl'
                            ])
                        }
                    }
                }
            })
            .state('app.campanhas.campanha_fila', {
                url: '/campanha_fila/:campanha',
                data: {
                    title: 'Fila da campanha'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/fila.html',
                        controller: 'FilaCtrl',
                        parent: 'app.campanhas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/controllers/FilaCtrl'
                            ])
                        }
                    }
                }
            })
            .state('app.campanhas.log', {
                url: '/log',
                data: {
                    title: 'Registros das campanhas',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/log.html',
                        controller: 'RegistrosCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/controllers/RegistrosCtrl',
                                'modules/forms/directives/input/smartSelect2',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })
            .state('app.campanhas.log_old', {
                url: '/log_old',
                data: {
                    title: 'Registros das campanhas (OLD)',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/campanhas/views/log_old.html',
                        controller: 'RegistrosCtrl_old',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/campanhas/controllers/RegistrosCtrl_old',
                                'modules/forms/directives/input/smartSelect2',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato, Permission, ModuloPermissao, $localStorage, $rootScope) {
        module.lazy = $couchPotato;
        Permission.defineRole('app.campanhas', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.campanhas", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: $rootScope.getWord("Você não tem acesso a Campanha.")
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;

});
