define(['modules/cdr/module'], function (module) {
    "use strict";

    return module.registerController('CdrCtrl', function ($scope, $http, $filter, $localStorage, exportFilesService) {
        $scope.filtro = {};
        $scope.filtro.sentido = [];
        $scope.filtro.pin = [];
        $scope.order = [{ "campo": "calldate", "ordem": "ASC" }];

        $scope.ordens = [
            { valor: "calldate", nome: $scope.getWord("Data") },
            { valor: "sentido", nome: $scope.getWord("Grupo") },
            { valor: "src", nome: $scope.getWord("Origem") },
            { valor: "dst", nome: $scope.getWord("Destino") },
            { valor: "duracao_total", nome: $scope.getWord("Duração total") },
            { valor: "espera", nome: $scope.getWord("Espera") },
            { valor: "falando", nome: $scope.getWord("Duração") },
            { valor: "status", nome: $scope.getWord("Status") },
            { valor: "gravacao", nome: $scope.getWord("Gravação") },
        ];

        $scope.status = [
            { valor: "Completada", nome: $scope.getWord("Completada") },
            { valor: "Não Completada", nome: $scope.getWord("Não Completada") },
            { valor: "Ocupado", nome: $scope.getWord("Ocupado") },
            { valor: "Falha", nome: $scope.getWord("Falha") },
        ];

        $scope.extra_filtro = [
            { valor: "contendo", nome: $scope.getWord("Contendo") },
            { valor: "igual", nome: $scope.getWord("Igual") },
            { valor: "iniciando", nome: $scope.getWord("Iniciando") },
            { valor: "finalizando", nome: $scope.getWord("Finalizando") },
            { valor: "nao_contendo", nome: $scope.getWord("Não contendo") },
            { valor: "diferente", nome: $scope.getWord("Diferente") }
        ];

        $scope.filterStatus = function (query) {
            return $scope.status.filter(function (o) {
                return o.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filterOrdem = function (query) {
            return $scope.ordens.filter(function (o) {
                return o.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $http.get(`${options.api.restv2}/contexto?max=-1&order=${encodeURI(JSON.stringify([{ "campo": "context" }]))}`).
            success(function (data) {
                $scope.contextos = data.dados;
            });

        $scope.getContexto = function (query) {
            return $scope.contextos.filter(function (op) {
                return op.context.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };


        $scope.getPin = function (query) {
            if (query.length > 0) {
                return $http.get(options.api.rest + "/pins/" + query + "?max=0");
            }
            else {
                return $http.get(options.api.rest + "/pins?max=0");
            }
        };


        function BuscaPermicaoExtraModulo (modulo, name, permicao) {
            if (modulo.nome === name && modulo.extra === permicao) {
                return modulo;
            }
            if (modulo.modulos && modulo.modulos.length > 0) {
                var ch = modulo.modulos;
                for (var i = 0; i < ch.length; i++) {
                    var found = BuscaPermicaoExtraModulo(ch[i], name, permicao);
                    if (found) {
                        return true;
                    }
                }
            }
            return false;
        }

        $scope.pode_ver_gravacao = false;
        //console.log($localStorage.User.modulos);

        for (var mi in $localStorage.User.modulos) {
            if (BuscaPermicaoExtraModulo($localStorage.User.modulos[mi], 'app.cdr', 'G')) {
                $scope.pode_ver_gravacao = true;
            }
        }

        $scope.getLinkGravacao = function (l1) {
            if (!l1) return;
            if (l1.indexOf('/var', 10) > 0) {
                let user = encodeURI($localStorage.User.token);
                let arquivo = encodeURI(l1.substring(0, l1.indexOf('/var', 10)));
                return `${options.api.restv4}/filas/gravacao?access_token=${user}&caminho=${arquivo}`;
            }
            else {
                let user = encodeURI($localStorage.User.token);
                let arquivo = encodeURI(l1);
                return `${options.api.restv4}/filas/gravacao?access_token=${user}&caminho=${arquivo}`;
                //return options.api.gravacao + l1;
            }
        }

        $scope.downloadGravacao = async function (l1) {
            console.log(l1)
            let user = encodeURI($localStorage.User.token);
            let filename = l1.substring(l1.lastIndexOf('/') + 1);
            let link = `${options.api.restv4}/filas/downloadgravacao?access_token=${user}&caminho=${l1}`;
            console.log(`Chamou download para tipo=audio/mpeg, nome=${filename}, link=${link}`);
            await exportFilesService.gerarDownload(link, 'audio/mpeg', filename);
            return `${options.api.restv4}/filas/downloadgravacao?access_token=${user}&caminho=${l1}`;
        }

        $scope.gotoPage = function (page) {
            $scope.filtro.page = page;
            $scope.load();
        };

        $scope.Listar = function (page) {
            $scope.filtro.page = 1;
            $scope.load();
        };

        $scope.init = () => {
            $scope.table = {};
            $scope.table.dados = [];

            $scope.filtro = {};
            $scope.filtro.gravacao = false;

            $scope.filtro.dataini = new Date();
            $scope.filtro.datafim = new Date();
            $scope.filtro.ordenar = [];
            $scope.filtro.origem_operador = $scope.extra_filtro[0];
            $scope.filtro.destino_operador = $scope.extra_filtro[0];
            $scope.filtro.ordenar.push($scope.ordens[0]);

            $scope.load();
        }

        function getDados (max) {
            var filtros = angular.copy($scope.filtro);

            filtros.data_ini = $filter('date')(filtros.dataini, 'yyyy-MM-dd');
            filtros.data_fim = $filter('date')(filtros.datafim, 'yyyy-MM-dd');
            delete filtros.dataini;
            delete filtros.datafim;

            if (!filtros.horaini || filtros.horaini.length == 0) {
                filtros.data_ini = filtros.data_ini + ' 00:00:00';
            } else {
                filtros.data_ini = filtros.data_ini + ' ' + filtros.horaini + ":00";
                delete filtros.horaini;
            }

            if (!filtros.horafim || filtros.horafim.length == 0) {
                filtros.data_fim = filtros.data_fim + ' 23:59:59';
            } else {
                filtros.data_fim = filtros.data_fim + ' ' + filtros.horafim + ":00";
                delete filtros.horafim;
            }

            if (filtros.status && filtros.status.length > 0) {
                filtros.status = [];
                angular.forEach($scope.filtro.status, function (o) {
                    filtros.status.push(o.valor);
                });
                filtros.status = JSON.stringify(filtros.status);
            }
            else {
                delete filtros.status;
            }

            if (filtros.sentido && filtros.sentido.length > 0) {
                filtros.sentido = [];
                angular.forEach($scope.filtro.sentido, function (o) {
                    filtros.sentido.push(o.context);
                });
                filtros.sentido = JSON.stringify(filtros.sentido);
            }
            else {
                delete filtros.sentido;
            }

            if (filtros.pin && filtros.pin.length > 0) {
                filtros.pin_id = [];
                for (var i = 0; i < filtros.pin.length; ++i) {
                    filtros.pin_id[i] = filtros.pin[i].id;
                }
            }
            delete filtros.pin;

            if (filtros.ordenar && filtros.ordenar.length > 0) {
                delete filtros.ordenar;
                $scope.order = [];
                angular.forEach($scope.filtro.ordenar, function (o) {
                    $scope.order.push({ "campo": o.valor, "ordem": "ASC" });
                });
            }
            else {
                delete filtros.ordenar;
            }
            if (filtros.sentido && filtros.sentido.length > 0) {
                //$scope.order = [];
                //angular.forEach($scope.filtro.ordenar, function (o) {
                //    $scope.order.push({ "campo": o.valor, "ordem": "ASC" });
                //});
            }
            else {
                delete filtros.sentido;
            }
            if (filtros.origem_operador) {
                console.log(filtros.origem_operador);
                filtros.origem_operador = filtros.origem_operador.valor;
            }
            if (filtros.destino_operador) {
                console.log(filtros.destino_operador);
                filtros.destino_operador = filtros.destino_operador.valor;
            }

            delete filtros.gravacao;

            var par = [];
            if (filtros) {
                angular.forEach(filtros, function (v, k) {
                    if (v) par.push(k + "=" + encodeURI(v));
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }

            return $http.get(`${options.api.restv2}/ligacoes?max=${max}&order=${encodeURI(JSON.stringify($scope.order))}${pars}`);
        }


        $scope.load = function () {

            $scope.loading = true;
            getDados(15)
                .then(data => {
                    $scope.table = data.data;
                    $scope.loading = false;
                })
                .catch(e => {
                    console.log('Tivemos um erro');
                    console.error(e);
                });
        };

        $scope.ToCSV = function () {
            $scope.carregando_export = true;
            let arr = [];

            // Formata cabeçalho
            let cab = [
                $scope.getWord('DataHora'),
                $scope.getWord('Grupo'),
                $scope.getWord('Origem'),
                $scope.getWord('Destino'),
                $scope.getWord('Ultimo registro'),
                $scope.getWord('PIN'),
                $scope.getWord('Espera'),
                $scope.getWord('Duracao'),
                $scope.getWord('Duracao Total'),
                $scope.getWord('Status'),
                $scope.getWord('Gravacao')
            ];

            arr.push(cab);

            // Formata dados
            getDados(-1).then(data => {
                data.data.dados.forEach((e) => {
                    var linha = [];
                    let canal = e.canal ? e.canal.split(',') : '';
                    linha.push($filter('date')(e.calldate, 'dd/MM/yyyy HH:mm:ss'));
                    linha.push(e.sentido);
                    linha.push(e.src);
                    linha.push(e.dst);
                    linha.push(canal[0]);
                    linha.push(e.pin_usuario);
                    linha.push(e.espera);
                    linha.push(e.falando);
                    linha.push(e.duracao_total);
                    linha.push(e.status);
                    linha.push(e.gravacao);
                    arr.push(linha);
                });
                exportFilesService.gerarCSV(arr, "Ligacoes.csv").then(
                    dados => {
                        console.log(`Sucesso na geraCSV, ${dados}`);
                        $scope.carregando_export = false;
                    },
                    erro => {
                        console.log(`Erro na geraCSV ${erro}`);
                        $scope.carregando_export = false;
                    }
                )
                $scope.carregando_export = false;
            });
        };

        $scope.ToPDF = function () {
            const filtros = angular.copy($scope.filtro);
            getDados(-1).then(data => {
                var linhas = [];
                var cabecalho = [
                    'DataHora',
                    'Grupo',
                    'Origem',
                    'Destino',
                    'Registro',
                    'Espera',
                    'Duração',
                    'Duração Total',
                    'Status'
                ];
                data.data.dados.forEach((ob) => {
                    var linha = [];
                    linha = [
                        $filter("datahora")(ob.calldate),
                        ob.sentido,
                        $filter("tel")(ob.src),
                        $filter("tel")(ob.dst),
                        ob.canal,
                        ob.espera,
                        ob.falando,
                        ob.duracao_total,
                        ob.status
                    ];
                    linhas.push(linha);
                })

                // Titulo
                const titulo = { texto: $scope.getWord('Ligações'), x: 40, y: 60 };

                // Subtitulo
                const subtitulo = [
                    { texto: `${$scope.getWord('De')} ${$filter('data')(filtros.dataini)} ${$scope.getWord('até')} ${$filter('data')(filtros.datafim)}`, x: 40, y: 80 }
                ];
                if (filtros.horaini && filtros.horafim) {
                    subtitulo.push({ texto: `${$scope.getWord('entre')} ${filtros.horaini} ${$scope.getWord('e')} ${filtros.horafim}`, x: 150, y: 80 });
                } else if (filtros.horaini) {
                    subtitulo.push({ texto: `${$scope.getWord('a partir das')} ${filtros.horaini}`, x: 150, y: 80 });
                } else if (filtros.horafim) {
                    subtitulo.push({ texto: `${$scope.getWord('até as')} ${filtros.horafim}`, x: 150, y: 80 });
                }
                if (filtros.sentido) {
                    var sentidos = [];
                    for (var s in filtros.sentido) {
                        sentidos.push(filtros.sentido[s].context);
                    }
                    subtitulo.push({ texto: $scope.getWord('Grupo(s)') + `: ${sentidos.join()}`, x: 40, y: 92 });
                }
                if (filtros.status) {
                    // doc.text('Status: ' + filtros.status.join(), 40, 92);
                    subtitulo.push({ texto: $scope.getWord('Status: ') + `: ${filtros.status.join()}`, x: 40, y: 104 });
                }

                exportFilesService.gerarPDF(cabecalho, linhas, titulo, subtitulo, "Ligações.pdf").then(ret => {
                    $scope.carregando_export = false;
                })
            });
        };

    });
});
