define(['modules/classificacoes/module'], function (module) {
    "use strict";

    return module.registerController('ClassificacoesCtrl', function ($scope, $http, $localStorage, paginationService) {
        $scope.carregando = false;
        $scope.sortBy = 'nome';
        $scope.sortDescending = false;
        $scope.searchText = '';
        $scope.filas = $localStorage.User.filas;

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.new = () => {
            $scope.popup = {};
            $scope.showClaPopup = true;
        };

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.filter = function () {
            var results = $scope.filteredRT;
            results.length = 0;
            var searchText = $scope.searchText;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].nome && reminderTypes[i].nome.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].fila && reminderTypes[i].fila.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    results.push(reminderTypes[i]);
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        $scope.init = function () {
            $scope.load();
        };

        $scope.load = function () {
            $scope.carregando = true;
            $http.get(options.api.restv2 + "/classificacoes?max=-1").
                success(function (data) {
                    $scope.reminderTypes = data.dados;
                    $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                    $scope.filter();
                    $scope.carregando = false;
                });
        };

        $scope.editar = (ob) => {
            ob.loading = true;
            $http.get(options.api.restv2 + "/classificacao/" + ob.id).
                success(function (data) {
                    ob.loading = false;
                    $scope.showClaPopup = true;
                    $scope.popup = data;
                });
        };

        $scope.Salvar = function (form) {
            if (form.$valid) {
                var pop = angular.copy($scope.popup);

                if (pop.id) {
                    delete pop.id;
                    $http.put(options.api.restv2 + "/classificacao/" + $scope.popup.id, pop)
                        .success(function (data) {
                            $scope.showClaPopup = false;
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Classificação " + pop.nome + " atualizada!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });

                            $scope.load();
                        });
                } else {
                    $http.post(options.api.restv2 + "/classificacao", pop)
                        .success(function (data) {
                            $scope.showClaPopup = false;
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Classificação " + pop.nome + " criada!</i>",
                                color: "#739E73",
                                timeout: 2000
                            });

                            $scope.load();
                        });
                }
            }
        };

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.classificacoes');
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover " + row.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv2 + "/classificacao/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Classificação apagada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });

                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.editarSubCla = (ob) => {
            ob.subloading = true;
            $scope.sub = {};
            $http.get(`${options.api.restv2}/classificacao/${ob.id}/subclassificacoes`).
                success(function (data) {
                    ob.subloading = false;
                    $scope.showpopupsub = true;
                    $scope.sub.subs = data;
                    $scope.sub.cla_nome = ob.nome;
                    $scope.sub.id = ob.id;

                    $scope.load();
                });
        };

        $scope.SalvarSub = function () {
            console.log($scope.sub);
            const sub = {
                update: $scope.sub.subs,
                apagar: $scope.sub.apagar
            }
            $http.put(`${options.api.restv2}/classificacao/${$scope.sub.id}/subclassificacoes`, sub).
                success(function (data) {
                    $scope.showpopupsub = false;
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                        color: "#739E73",
                        timeout: 1000
                    });
                })
        };

        $scope.NewSubCla = function (sub) {
            var ob = { idClassificacao: $scope.sub.id, nome: '' };
            $scope.sub.subs.push(ob);
        };

        $scope.delSubCla = (idx) => {
            if (!$scope.sub.apagar) $scope.sub.apagar = [];
            const sub = $scope.sub.subs.splice(idx, 1);
            console.log(sub);
            if (sub[0].id) $scope.sub.apagar.push(sub[0].id);
        }
    });
});