define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-file-upload',
    'ui.bootstrap',
    'angularUtils.directives.dirPagination'
], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.clientes', [
        'ui.router',
        'ngResource',
        'ngFileUpload',
        'ui.bootstrap',
        'angularUtils.directives.dirPagination'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.clientes', {
                url: '/clientes',
                data: {
                    title: 'Clientes',
                    permissions: {
                        only: ['app.clientes']
                    }
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/clientes/views/listanopag.html',
                        controller: 'ClientesCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'layout/directives/editPopup',
                                'modules/clientes/controllers/ClientesCtrl'
                            ])
                        },
                    }
                }
            })

            .state('app.clientes.grupos', {
                url: '/clientes/grupos',
                data: {
                    title: 'Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/clientes/views/lista.grupos.html',
                        controller: 'ClienteGruposCtrl',
                        parent: 'app.clientes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'layout/directives/editPopup',
                                'modules/clientes/controllers/ClienteGruposCtrl'
                            ])
                        }
                    }
                }
            })

            .state('app.clientes.layouts', {
                url: '/clientes/layouts',
                data: {
                    title: 'Layouts de importação'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/clientes/views/lista.layouts.html',
                        controller: 'ClienteLayoutsCtrl',
                        parent: 'app.clientes',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'layout/directives/editPopup',
                                'modules/clientes/controllers/ClienteLayoutsCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato, Permission, ModuloPermissao, $localStorage, $rootScope) {
        module.lazy = $couchPotato;
        Permission.defineRole('app.clientes', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.clientes", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: $rootScope.getWord("Você não tem acesso a clientes.")
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;

});