define(['modules/configuracoes/pesquisa_satisfacao/module'], function(module){
    "use strict";

    return module.registerController('PesquisaSatisfacaoCtrl', function($scope, $http, $stateParams, $rootScope){        
        $scope.row = {};
        $scope.campos_disable = false;
        $scope.filtro = {};
        $scope.order = [{"campo":"nome","ordem":"ASC"}];   
        $scope.loadingValor = false;
        
        $scope.init = function() {
            $scope.filtro.page = {valor:1};
            $scope.load();
        };
  
        $scope.getClass = function(coluna){
            if($scope.order === coluna){
                return $scope.order_sentido === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };
        
        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            }
            else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            $scope.load();
        };

        $scope.gotoPage = function(page){
            $scope.filtro.page= {valor: page?page:1};
            $scope.load();
        }

        $scope.load = function() {
          $scope.carregando = true;
          var par = [];
          if($scope.filtro){
              angular.forEach($scope.filtro, function(v, k) {
                  if(v) par.push(k+"="+v.valor);
              });
          }
          var pars = "";
          if(par.length > 0){
              pars = "&"+par.join("&");
          }

          $http.get(options.api.restv3+"/pesquisa?max=15&order="+encodeURI(JSON.stringify($scope.order))+pars).
              success(function(data) {
                  $scope.table = data;
                  $scope.carregando = false;
          });
        };        
        
        $scope.showpopup = function(row, disable) {
            $scope.campos_disable = disable;
            $scope.loading = true;
            $http.get(options.api.restv3+"/pesquisa/"+row.id).
            success(function(data) { 
                $scope.row = data.dados;
                if($scope.row.respostas==undefined) $scope.row.respostas=[];
                $scope.loading = false;
                $scope.showPopUp = true;
            });
        };
        
        $scope.new = function() {
            $scope.campos_disable = false;
            $scope.loading = true;
            $scope.row = {};
            $scope.row.respostas = [];
            $scope.loading = false;
            $scope.showPopUp = true;
        };       
        
        $scope.Salvar = function(f1) {
            $scope.campos_disable = true;
            if(f1.$valid){
                //Ajusta o modelo
                var model = angular.copy($scope.row);
                
                //trata as remoções
                if(model.respostas && Array.isArray(model.respostas) && model.respostas.length > 0)
                {
                    for (let i = 0; i < model.respostas.length; i++) {
                        const element = model.respostas[i];
                        if(!!element.deletar) model.respostas.splice(i,1);
                    }     
                }

                if(model.id){
                    delete model.id;
                    //Envia para o server    
                    $http.put(options.api.restv3 + '/pesquisa/' +$scope.row.id, model).
                    success(function (data) {
                        $scope.showPopUp = false;
                        $scope.load();
                        f1.$setPristine();
                        f1.$setUntouched();
                        $.smallBox({
                            title: "SUCESSO!!",
                            content: "<i class='fa fa-check'></i> <i>Salvo com sucesso!</i>",
                            color: "#739E73",
                            timeout: 6000
                        });
                    })
                }
                else{
                    //Envia para o server    
                    $http.post(options.api.restv3 + '/pesquisa', model).
                        success(function (data) {
                            $scope.showPopUp = false;
                            $scope.load();
                            f1.$setPristine();
                            f1.$setUntouched();
                            $.smallBox({
                                title: "SUCESSO!!",
                                content: "<i class='fa fa-check'></i> <i>Criado com sucesso!</i>",
                                color: "#739E73",
                                timeout: 6000
                            });
                        });
                }
            }
            $scope.campos_disable = false;
        };
               
        $scope.$watch('Pesquisa', function(t1, t2){
            // Se os dois valores estirem em branco nao faz nada, isso aconte no load.
            if (!t1 && !t2)
                return false;

                setTimeout(function () {
                    if (t1 === $scope.Pesquisa) {
                        if (t1 !== '')
                            $scope.filtro.nome = { valor: t1 };
                        else
                            delete $scope.filtro.nome;

                        $scope.load();
                    }
            }, 1000);
        });
        
        $scope.confirma =  function (row) {
            $.x5Box({
                title: "Remover <b>"+row.nome+"</b>?",
                content: "Tem certeza que quer remover "+row.nome+"? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };
        
        $scope.delete = function(id) {        
            $http.delete(options.api.restv3+"/pesquisa/"+id).
            success(function(s) {            
                $.smallBox({
                    title : "SUCESSO!!",
                    content : "<i class='fa fa-check'></i> <i>Pesquisa apagada com sucesso!</i>",
                    color : "#739E73",
                    timeout : 2000
                });

                $scope.load();
            })
        };

        $scope.AddResposta = function(row) {        
            var r = {
                id:null, 
                id_pesquisa: row.id,
                nota: null,
                significado: null
            };
            $scope.row.respostas.push(r);
        }

        $scope.DelResposta = function(r) {        
            $scope.row.respostas.splice($scope.row.respostas.indexOf(r),1);
        };

    });
});