define(['modules/configuracoes/pesquisa_satisfacao/module'], function (module) {
    "use strict";

    return module.registerController('PesquisaSatisfacaoCtrl', function ($scope, $http, $localStorage, $filter, paginationService, exportFilesService) {
        $scope.nps = [];
        $scope.notas = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10'];
        $scope.nivel = {};
        $scope.nivel.niveis = [];
        $scope.nivel.notas = [];
        $scope.nivel.cor = [];

        $scope.sortBy = 'data_registro';
        $scope.sortDescending = true;

        const getFiltrosToQuery = (comPage) => {
            var filtros = angular.copy($scope.filtro);
            delete filtros.desagrupar
            delete filtros.desagrupado
            delete filtros.usaGrupo
            delete filtros.grupos

            if ($scope.filtro.page && comPage) {
                filtros.page = $scope.filtro.page;
            } else delete filtros.page;

            filtros.dataini = $filter('date')(filtros.dataini, 'dd/MM/yyyy');
            filtros.datafim = $filter('date')(filtros.datafim, 'dd/MM/yyyy');
            if ($scope.filtro.fila && $scope.filtro.fila.length > 0) {
                filtros.fila = [];
                angular.forEach($scope.filtro.fila, function (f) {
                    filtros.fila.push(f.text);
                });
                filtros.fila = JSON.stringify(filtros.fila);
            }
            else {
                filtros.fila = [];
                filtros.fila = JSON.stringify($scope.filas);
                // delete filtros.fila;
            }

            if ($scope.filtro.agente && $scope.filtro.agente.length > 0) {
                filtros.agente = [];
                angular.forEach($scope.filtro.agente, function (a) {
                    filtros.agente.push(a.fullname);
                });
                filtros.agente = JSON.stringify(filtros.agente);
            }
            else delete filtros.agente;

            if ($scope.filtro.pesquisa && $scope.filtro.pesquisa.length > 0) {
                filtros.pesquisa = [];
                angular.forEach($scope.filtro.pesquisa, function (f) {
                    filtros.pesquisa.push(f.id);
                });
                filtros.pesquisa = JSON.stringify(filtros.pesquisa);
            }
            else delete filtros.pesquisa;

            if ($scope.filtro.resposta && $scope.filtro.resposta.length > 0) {
                filtros.resposta = [];
                angular.forEach($scope.filtro.resposta, function (f) {
                    filtros.resposta.push(f.nota);
                });
                filtros.resposta = JSON.stringify(filtros.resposta);
            }
            else delete filtros.resposta;

            if ($scope.filtro.protocolo && $scope.filtro.protocolo.length > 0) {
                filtros.protocolo = filtros.protocolo;
            } else delete filtros.protocolo;

            if ($scope.filtro.tags && $scope.filtro.tags.length > 0) {
                filtros.tags_id = [];
                angular.forEach($scope.filtro.tags, function (f) {
                    filtros.tags_id.push(f.id);
                });
                filtros.tags_id = JSON.stringify(filtros.tags_id);
                delete filtros.tags
            } else delete filtros.tags_ids;

            if ($scope.filtro.desagrupado && Object.keys($scope.filtro.desagrupado).length > 0) {
                // $scope.filtro.desagrupado é um objeto no seguinte formato
                // {teste 1: 'valor1', teste 2: 'valor2'}
                // vou remover as propriedades vazias e transformar em um array de objetos {nome: 'teste 1', valor: 'valor1'}
                filtros.camposAdicionais = [];
                for (let prop in $scope.filtro.desagrupado) {
                    if ($scope.filtro.desagrupado.hasOwnProperty(prop) && $scope.filtro.desagrupado[prop] !== '') {
                        filtros.camposAdicionais.push({ nome: prop, valor: $scope.filtro.desagrupado[prop] });
                    }
                }
                filtros.camposAdicionais = JSON.stringify(filtros.camposAdicionais);
            } else delete filtros.desagrupado;

            if ($scope.filtro.tipo) {
                filtros.tipo = $scope.filtro.tipo;
            } else delete filtros.tipo;


            var par = [];
            if (filtros) {
                angular.forEach(filtros, function (v, k) {
                    if (v) par.push(k + "=" + encodeURI(v));
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }

            var ord = "";
            //Ordenação
            if ($scope.ordem && Array.isArray($scope.ordem) && $scope.ordem.length > 0) {
                ord = "&order=" + encodeURI(JSON.stringify($scope.ordem.map((o) => {
                    return {
                        campo: o.campo,
                        ordem: o.ordem
                    }
                })));
            }

            return { pars, ord }
        }

        $scope.gotoPage = function (page) {
            $scope.filtro.page = page;
            $scope.getRel();
        };

        $scope.getGruposFila = async () => {
            if (!$localStorage.User.gruposFila || $localStorage.User.gruposFila.length === 0) return
            $scope.gruposNomes = $localStorage.User.gruposFila;
            await $http.get(options.api.restv4 + "/gruposFilas?max=-1&nomes=" + encodeURIComponent(JSON.stringify($localStorage.User.gruposFila))).
                success(function (data) {
                    $scope.gruposObj = data.dados;
                });
        }

        $scope.filterGrupos = (query) => {
            var grupos = $scope.gruposNomes
            return grupos.filter(function (grupo) {
                return grupo.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.groupAdded = (tag) => {
            const grupoObj = $scope.gruposObj.find(grupo => grupo.nome === tag.text)
            if (!$scope.filtro.fila) $scope.filtro.fila = grupoObj.filas
            else grupoObj.filas.forEach(fila => {
                if ($scope.filtro.fila.indexOf({ text: fila }) === -1) {
                    $scope.filtro.fila.push({ text: fila });
                }
            });
        }

        $scope.groupRemoved = () => {
            const gruposNomes = $scope.filtro.grupos.map(grupo => grupo.text)
            const gruposObj = $scope.gruposObj.filter(grupo => gruposNomes.includes(grupo.nome))
            const filas = gruposObj.flatMap(grupo => grupo.filas)
            $scope.filtro.fila = []
            if (filas && filas.length > 0) {
                filas.forEach(fila => {
                    if ($scope.filtro.fila.indexOf({ text: fila }) === -1) {
                        $scope.filtro.fila.push({ text: fila });
                    }
                })
            }
        }

        $scope.changeGroupOrQueue = () => {
            $scope.filtro.fila = []
            $scope.filtro.grupos = []
        }
        window.changeGroupOrQueue = $scope.changeGroupOrQueue

        $scope.returnValorAgrupado = (registro, campo) => {
            if (registro && registro.vCliente && registro.vCliente.CamposAdicionais && registro.vCliente.CamposAdicionais.length > 0) {
                return registro.vCliente.CamposAdicionais.find(ca => ca.nome === campo)?.valor;
            }
        }

        $scope.returnTags = (registro) => {
            if (registro && registro.tags && registro.tags.length > 0) {
                return registro.tags.map(t => t.nome).join(', ');
            }
        }

        $scope.returnClienteTipo = (registro) => {
            if (registro && registro.vCliente) {
                return registro.vCliente.isBusiness ? $scope.getWord('CNPJ') : $scope.getWord('CPF');
            }
        }

        $scope.returnPesquisaNome = (registro) => {
            if (registro && registro.id_pesquisa) {
                return $scope.pesquisas.find(p => p.id === registro.id_pesquisa)?.nome || '';
            } else return ''
        }

        $scope.onAddDesagrupar = (tag) => {
            console.log('Tag add', tag);
        }

        $scope.onDelDesagrupar = (tag) => {
            delete $scope.filtro.desagrupado[tag.text]
            console.log('Tag Del', tag);
        }

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.GrafNivelCat = [
            { valor: 'dia', nome: $scope.getWord('Dia') },
            { valor: 'dia_semana', nome: $scope.getWord('Dia da semana') },
            { valor: 'mes', nome: $scope.getWord('Mês') },
            { valor: 'ano', nome: $scope.getWord('Ano') },
        ]

        $scope.grafico = $scope.GrafNivelCat[2];

        $scope.filterNotas = (query) => {
            return $scope.notas.filter(function (ob) {
                return ob.indexOf(query) != -1;
            });
        }

        $scope.nps_grupo = {
            det: {
                count: 0,
                percentual: 0
            },
            pas: {
                count: 0,
                percentual: 0
            },
            pro: {
                count: 0,
                percentual: 0
            }
        };

        function populaNivel () {
            $scope.nivel_satisfacao = {};
            if ($localStorage.NivelConfig) $scope.nivel_satisfacao = $localStorage.NivelConfig;
            else {
                $scope.nivel_satisfacao = [
                    {
                        nome: $scope.getWord('Muito Satisfeito'),
                        notas: ['10'],
                        cor: '#4da044'
                    },
                    {
                        nome: $scope.getWord('Satisfeito'),
                        notas: ['8', '9'],
                        cor: '#f8911b'
                    },
                    {
                        nome: $scope.getWord('Insatisfeito'),
                        notas: ['0', '1', '2', '3', '4', '5', '6', '7'],
                        cor: '#c00002'
                    }
                ]
            }
            for (let nivel of $scope.nivel_satisfacao) {
                $scope.nivel.niveis.push(nivel.nome);
                $scope.nivel.notas[nivel.nome] = nivel.notas;
                $scope.nivel.cor[nivel.nome] = nivel.cor;
            }
        }

        $scope.updateCSAT = () => {
            if (!$scope.nivel_satisfacao || !$scope.csat_nivel) return
            const t = $scope.nivel_satisfacao.find(n => n.nome == $scope.csat_nivel.text);
            const notasValidas = $scope.nivel_satisfacao.flatMap(ns => [...ns.notas])
            const pesquisasValidas = $scope.nps.filter(n => notasValidas.includes(n.nps || parseInt(n.nps)))
            const pesquisasValidasCount = pesquisasValidas.reduce((acc, cur) => acc + cur.count, 0)
            $scope.csat = ((t.count / pesquisasValidasCount) * 100).toFixed(2);
            $scope.csatInvalid = ($scope.notasTotais - pesquisasValidasCount) > 0 ? $scope.notasTotais - pesquisasValidasCount : 0
        }

        $scope.updateGrafNivel = () => {
            populaGrafNivel();
        }

        async function populaGrafNivel () {
            const { pars } = getFiltrosToQuery();
            await $http.get(options.api.restv4 + `/pesquisas/registros/totaisPeriodo?periodo=${$scope.grafico.valor}` + pars)
                .success((data) => {
                    $scope.notasPorPeriodo = data
                })

            $scope.nivel_satisfacao = $scope.nivel_satisfacao.map(nivel => {
                nivel.count = 0;
                nivel.categoria = {};
                nivel.serie = [];
                return nivel;
            })

            for (let nivel of $scope.nivel_satisfacao) {
                for (let reg of Object.keys($scope.notasPorPeriodo)) {
                    let cat = 0;
                    switch ($scope.grafico.valor) {
                        case 'mes':
                            cat = $scope.getWord(getNomeMes(parseInt(reg) - 1));
                            break;
                        case 'dia_semana':
                            cat = $scope.getWord(getDiaDaSemana(parseInt(reg) - 1));
                            break;
                        case 'ano':
                            cat = reg;
                            break;
                        default:
                            cat = reg;
                            break;
                    }

                    if (!nivel.categoria[cat]) nivel.categoria[cat] = 0;
                    const notasValidas = $scope.notasPorPeriodo[reg].filter(n => nivel.notas.includes(n.nps))
                    nivel.count = notasValidas.reduce((acc, cur) => acc + cur.count, nivel.count)
                    nivel.categoria[cat] = notasValidas.reduce((acc, cur) => acc + cur.count, 0)
                }
            }

            let categorias = []
            for (let n of $scope.nivel_satisfacao) {
                for (let c of Object.keys(n.categoria)) {
                    categorias.push(c)
                }
            }
            // Deixar unico
            categorias = Array.from(new Set(categorias));

            // Ordenar de acordo com o tipo
            switch ($scope.grafico.valor) {
                case 'mes':
                    categorias = ordenarPorMes(categorias);
                    break;
                case 'dia_semana':
                    categorias = ordenarPorDiaSemana(categorias);
                    break;
                default:
                    categorias = categorias.sort((a, b) => { return parseInt(a) - parseInt(b); });
                    break;
            }
            $scope.graficoNivel.xAxis.categories = categorias;

            for (let cat of categorias) {
                for (let nivel of $scope.nivel_satisfacao) {
                    let s = getDadosDaCategoria(cat, nivel.nome);
                    nivel.serie.push(s);
                }
            }

            $scope.graficoNivel.series = $scope.nivel_satisfacao.map(nivel => {
                return {
                    name: nivel.nome,
                    type: 'column',
                    color: nivel.cor,
                    data: nivel.serie
                }
            })

            $scope.updateCSAT()
        }

        function getDadosDaCategoria (categoria, grupo) {
            let gp = $scope.nivel_satisfacao.find(gr => gr.nome == grupo);
            if (gp && gp.categoria[categoria]) {
                return gp.categoria[categoria];
            }
            else {
                return 0;
            }
        }

        $scope.saveNivelConfig = () => {
            const niveis = [];
            for (let nivel of $scope.nivel.niveis) {
                const ob = {
                    nome: nivel.text,
                    notas: $scope.nivel.notas[nivel.text].map(n => n.text),
                    cor: $scope.nivel.cor[nivel.text]
                }
                niveis.push(ob);
            }

            $localStorage.NivelConfig = niveis;
            $scope.nivel_satisfacao = $localStorage.NivelConfig;

            populaGrafNivel();

            $.smallBox({
                title: "SUCESSO!!",
                content: "<i class='fa fa-check'></i> <i>Configurações salvas!</i>",
                color: "#739E73",
                timeout: 2000
            });
        }

        $scope.RemoveNivel = (nivel) => {
            console.log(`Remove o nivel ${nivel}`);
            const idx = $scope.nivel.niveis.findIndex(n => n.text == nivel);
            if (idx) {
                $scope.nivel.niveis.splice(idx, 1);
            }
        }

        $scope.onAddNivel = (tag) => {
            console.log(`Add tag ${tag}`);
        }

        $scope.onDelNivel = (tag) => {
            console.log(`Del tag ${tag}`);
        }

        $scope.getNPSCalc = () => {
            let val = $scope.nps_grupo.pro.percentual - $scope.nps_grupo.det.percentual;
            return val >= 0 ? { valor: `+${val}`, style: "color: #4da044; font-weight:800; font-size: 650%;" } : { valor: `${val}`, style: "color: #f33e2a; font-weight:800; font-size: 650%;" };
        }

        $scope.NPSConf = [
            { id: 0, text: '0', color: '#b61719', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 1, text: '1', color: '#d32322', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 2, text: '2', color: '#f33e2a', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 3, text: '3', color: '#f16726', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 4, text: '4', color: '#f78d1d', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 5, text: '5', color: '#fdaf1a', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 6, text: '6', color: '#ffd400', group: $scope.getWord('Detractors'), style: "text-align: center;", valor: 0 },
            { id: 7, text: '7', color: '#e6e7e9', group: $scope.getWord('Passive'), style: "color: black; text-align: center;", valor: 0 },
            { id: 8, text: '8', color: '#e6e7e9', group: $scope.getWord('Passive'), style: "color: black; text-align: center;", valor: 0 },
            { id: 9, text: '9', color: '#88c546', group: $scope.getWord('Promoters'), style: "text-align: center;", valor: 0 },
            { id: 10, text: '10', color: '#4da044', group: $scope.getWord('Promoters'), style: "text-align: center;", valor: 0 }
        ]

        $scope.getPercentual = function (v1, v2) {
            if (!v1 || !v2) return 0;
            return Number(parseFloat(((parseInt(v1) / parseInt(v2)) * 100)).toFixed(2));
        };

        $scope.graficoNivel = {
            chart: {
                type: 'column'
            },
            title: {
                text: $scope.getWord('Nível de satisfação')
            },
            xAxis: {
                categories: ['Abril', 'Maio', 'Junho', 'Julho', 'Agosto']
            },
            yAxis: {
                min: 0,
                title: {
                    text: '%'
                }
            },
            tooltip: {
                pointFormat: '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.percentage:.2f}%)<br/>',
                shared: true
            },
            plotOptions: {
                column: {
                    stacking: 'percent'
                }
            }
        }

        $scope.NPSGraf = {
            chart: {
                type: 'gauge',
                plotBackgroundColor: null,
                plotBackgroundImage: null,
                plotBorderWidth: 0,
                plotShadow: false
            },

            credits: {
                enabled: false,
            },

            plotOptions: {
                gauge: {
                    innerRadius: '175%',
                    dial: {
                        radius: '70%',
                        backgroundColor: 'black',
                        borderWidth: 0,
                        baseWidth: 5,
                        topWidth: 1,
                        baseLength: '40%', // of radius
                        rearLength: '10%'
                    }
                }
            },

            title: {
                text: null
            },

            pane: {
                startAngle: -100,
                endAngle: 100,
                background: [{
                    backgroundColor: '#FFF',
                    borderWidth: 0,
                }]
            },

            // the value axis
            yAxis: {
                min: -100,
                max: 100,

                minorTickInterval: 'auto',
                minorTickWidth: 1,
                minorTickLength: 10,
                minorTickPosition: 'inside',
                minorTickColor: '#fff',

                tickPixelInterval: 30,
                tickWidth: 2,
                tickPosition: 'inside',
                tickLength: 10,
                tickColor: '#fff',
                labels: {
                    step: 2,
                    rotation: 'auto'
                },
                title: {
                    text: 'NPS'
                },
                plotBands: [
                    {
                        from: -100,
                        to: 0,
                        color: '#DF5353' // red
                    },
                    {
                        from: 0,
                        to: 100,
                        color: '#55BF3B' // green
                    }]
            },

            series: [{
                name: 'NPS',
                data: [0],
                tooltip: {
                    valueSuffix: ' %'
                },
                dataLabels: {
                    enabled: false
                },
            }]

        }

        $scope.chartConfig = {
            chart: {
                type: 'column'
            },
            title: {
                text: $scope.getWord('Nenhum registro encontrado')
            },
            subtitle: {
                text: $scope.getWord('Verifique os filtros e use o botão "Listar"')
            },
            xAxis: {
                type: 'category',
                labels: {
                    style: {
                        fontSize: '16px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            yAxis: {
                min: 0,
                allowDecimals: false,
                title: {
                    text: $scope.getWord('Qtd'),
                    style: {
                        fontSize: '9px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            legend: {
                enabled: false
            },
            tooltip: {
                pointFormat: $scope.getWord('Total:') + ' <b>{point.y}</b>'
            },
            dataLabels: {
                enabled: true,
                color: '#FFFFFF',
                align: 'right',
                format: '{point.y:.1f}', // one decimal
                y: 10, // 10 pixels down from the top
                style: {
                    fontSize: '13px',
                    fontFamily: 'Verdana, sans-serif'
                }
            },
            credits: {
                enabled: false,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            }
        };

        $scope.chartConfig2 = {
            chart: {
                type: 'Pie',
                plotBackgroundColor: null,
                plotBorderWidth: null,
                plotShadow: false
            },
            credits: {
                enabled: false,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            },
            title: {
                text: ' '
            },
            subtitle: {
                text: ' '
            },
            tooltip: {
                pointFormat: '<b>{point.percentage:.2f}%</b>'
            },
            plotOptions: {
                pie: {
                    allowPointSelect: true,
                    cursor: 'pointer',
                    dataLabels: {
                        enabled: true,
                        format: '<b>{point.name}</b>: {point.percentage:.2f} %',
                        style: {
                            color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
                        }
                    }
                }
            }
        };

        $scope.load = function () {
            $scope.filas = [];
            $scope.agentes = [];
            $scope.filtro = {};
            $scope.filtro.dataini = new Date();
            $scope.filtro.datafim = new Date();
            $scope.filtro.gravacao = false;
            $scope.filtro.page = 1;
            $scope.table = {};
            $scope.table.rows = {};

            $scope.getFilas();
            $scope.getGruposFila();
            $scope.getAgentes();
            $scope.getPesquisas();
            $scope.getRespostas();
            $scope.getTags();
            populaNivel();
            $scope.getRel()
        };

        $scope.sortBy = [
            { nome: `${$scope.getWord('Data')} (Asc)`, campo: 'a.data_registro', ordem: 'ASC' },
            { nome: `${$scope.getWord('Data')} (Desc)`, campo: 'a.data_registro', ordem: 'DESC' },
            { nome: `${$scope.getWord('Fila')} (Asc)`, campo: 'a.fila', ordem: 'ASC' },
            { nome: `${$scope.getWord('Fila')} (Desc)`, campo: 'a.fila', ordem: 'DESC' },
            { nome: `${$scope.getWord('Agente')} (Asc)`, campo: 'a.agente', ordem: 'ASC' },
            { nome: `${$scope.getWord('Agente')} (Desc)`, campo: 'a.agente', ordem: 'DESC' },
            { nome: `${$scope.getWord('Classificação')} (Asc)`, campo: 'c.nome', ordem: 'ASC' },
            { nome: `${$scope.getWord('Classificação')} (Desc)`, campo: 'c.nome', ordem: 'DESC' },
            { nome: `${$scope.getWord('De - Origen')} (Asc)`, campo: 'a.src', ordem: 'ASC' },
            { nome: `${$scope.getWord('De - Origen')} (Desc)`, campo: 'a.src', ordem: 'DESC' },
            { nome: `${$scope.getWord('Pesquisa')} (Asc)`, campo: 'b.nome', ordem: 'ASC' },
            { nome: `${$scope.getWord('Pesquisa')} (Desc)`, campo: 'b.nome', ordem: 'DESC' },
            { nome: `${$scope.getWord('Tipo')} (Asc)`, campo: 'cl.isBusiness', ordem: 'ASC' },
            { nome: `${$scope.getWord('Tipo')} (Desc)`, campo: 'cl.isBusiness', ordem: 'DESC' },
            { nome: `${$scope.getWord('Nota')} (Asc)`, campo: 'a.nota', ordem: 'ASC' },
            { nome: `${$scope.getWord('Nota')} (Desc)`, campo: 'a.nota', ordem: 'DESC' },
        ];

        $scope.filterOrdem = function (query) {
            return $scope.sortBy.filter(function (s) {
                return s.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getFilas = function () {
            $scope.filas = $localStorage.User.filas;
            $http.get(options.api.restv3 + "/fila?max=-1&page=1").
                success(function (data) {
                    data.dados = data.dados.filter(f => $localStorage.User.filas.includes(f.name))
                    $scope.filasObj = data.dados;
                });
        };

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.filterPesquisas = function (query) {
            return $scope.pesquisas.filter(function (f) {
                return f.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getFilterAgente = function (query) {
            return _.sortBy($scope.agentes, 'fullname').filter(function (agente) {
                if ($scope.filtro.filas && $scope.filtro.filas.length > 0) {
                    for (var i = 0; i < $scope.filtro.filas.length; i++) {
                        if (agente.filas && agente.filas.indexOf($scope.filtro.filas[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                        else if (agente.fila && agente.fila.indexOf($scope.filtro.filas[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                    }
                }
                else {
                    return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                }
            });
        };

        $scope.getFilterRespostas = function (query) {
            let respostas = []
            if ($scope.filtro.pesquisa && $scope.filtro.pesquisa.length > 0) {
                const idsPesquisas = $scope.filtro.pesquisa.map(p => p.id)
                respostas =
                    $scope.respostas
                        .filter(r => idsPesquisas.includes(r.id_pesquisa))
                        .filter((r) => { return r.nota.toLowerCase().indexOf(query.toLowerCase()) !== -1 })
            } else {
                respostas = $scope.respostas.filter((r) => { return r.nota.toLowerCase().indexOf(query.toLowerCase()) !== -1 })
            }
            return respostas
        };

        $scope.getFilterTags = function (query) {
            return $scope.tags.filter(function (f) {
                return f.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getAgentes = function (fila) {
            var link = "/agentes?max=0&order[0]=fullname";
            if (options.anyagent) {
                link = "/agente/all";
                if (fila) link = "/agente/all/" + JSON.stringify([fila]);
            }
            else {
                if (fila) {
                    link = "/agentespelafila/" + fila;
                }
            }
            if (options.multserver) {
                var servers = ($scope.filtro.server && $scope.filtro.server.length > 0) ? $scope.filtro.server : options.multserver;
                var agentes = [];
                angular.forEach(servers, function (s) {
                    $http.get("http://" + s.endereco + "/xcontact-rest" + link).
                        success(function (data) {
                            angular.forEach(data.itens, function (f) {
                                agentes.push(f);
                            });
                        });
                });
                $scope.agentes = agentes;
            }
            else {
                $http.get((options.anyagent ? options.api.restv3 : options.api.rest) + link).
                    success(function (data) {
                        if (options.anyagent)
                            $scope.agentes = data.dados;
                        else $scope.agentes = data.itens;
                    });
            }
        };

        $scope.getPesquisas = function () {
            $http.get(options.api.restv3 + "/pesquisa?max=-1&order=" + encodeURI(JSON.stringify([{ "campo": "name", "ordem": "ASC" }]))).
                success(function (data) {
                    $scope.pesquisas = data.dados;
                });
        };

        $scope.getRespostas = function () {
            $http.get(options.api.restv3 + "/pesquisa/resposta?max=-1&order=" + encodeURI(JSON.stringify([{ "campo": "nota", "ordem": "ASC" }])))
                .success(function (data) {
                    $scope.respostas = data.dados.map((x) => {
                        x.fullname = x.nota + " - " + x.significado;
                        return x;
                    });
                });
        };

        $scope.getLinkGravacao = function (l1) {
            if (!l1) return;
            if (l1.indexOf('/var', 10) > 0) {
                let user = encodeURI($localStorage.User.token);
                let arquivo = encodeURI(l1.substring(0, l1.indexOf('/var', 10)));
                return `${options.api.restv4}/filas/gravacao?access_token=${user}&caminho=${arquivo}`;
            }
            else {
                let user = encodeURI($localStorage.User.token);
                let arquivo = encodeURI(l1);
                return `${options.api.restv4}/filas/gravacao?access_token=${user}&caminho=${arquivo}`;
                //return options.api.gravacao + l1;
            }
        };

        $scope.downloadGravacao = async function (l1) {
            let user = encodeURI($localStorage.User.token);
            let filename = l1.substring(l1.lastIndexOf('/') + 1);
            let link = `${options.api.restv4}/filas/downloadgravacao?access_token=${user}&caminho=${l1}`;
            await exportFilesService.gerarDownload(link, 'audio/mpeg', filename);
            return `${options.api.restv4}/filas/downloadgravacao?access_token=${user}&caminho=${l1}`;
        }

        $scope.getTags = function () {
            $http.get(options.api.restv4 + "/tags").
                success(function (data) {
                    $scope.tags = data;
                });
        };

        $scope.getLigacoes = async function (ids) {
            if (ids.length === 0) return [];

            try {
                const response = await $http.post(options.api.restv4 + '/filas/ligacoesTagsIds', {
                    max: -1,
                    callids: ids
                });

                const data = response.data;
                data.rows.map((row) => {
                    row.tagsNomes = row.queueCallsTags.map((tag) => tag.tags[0].nome);
                });

                return data.rows;
            } catch (error) {
                console.error('Erro ao buscar ligações:', error);
                return [];
            }
        };


        $scope.getChats = async function (ids) {
            if (ids.length === 0) return []
            let chats
            await $http.post(options.api.restv4 + `/chatsTagsIds`, {
                protocolos: ids
            }).
                success(function (data) {
                    data.map((row) => {
                        row.tagsNomes = row.queueChatTags.map((tag) => tag.tags[0].nome)
                    })
                    chats = data;
                });
            return chats
        }

        //Função para eximir a necessidade de Join no banco para apresentar o significado do valor da pesquisa.
        //Por hora sem necessidade, pois estamos usando o joi e retornando o Campo notafullname
        $scope.getNota = function (row) {
            var r = $scope.respostas.filter((x) => {
                return x.id_pesquisa == row.id_pesquisa && (x.nota ?? '').trim() == (row.nota ?? '').trim();
            });
            if (r && Array.isArray(r) && r.length > 0)
                return r[0].fullname.replace('\n', '');
            else return row.nota ? row.nota.replace(/\n/g, '') : ''
        };

        $scope.getRel = async function () {
            $scope.loading = true;
            $scope.getDados(undefined, async function (data) {
                $scope.loading = false;
                $scope.table = data;
                $scope.table.rows = $scope.table.rows.map(e => {
                    e.nota_n = parseInt(e.nota);
                    return e;
                });

                populaGrafNivel();

                const { pars } = getFiltrosToQuery();
                await $http.get(options.api.restv4 + `/pesquisas/registros/totais?` + pars).
                    success(async function (data) {
                        $scope.notasTotais = data.find(d => d.nps == 'Total')?.count;
                        $scope.notasInvalidas = data.find(d => d.nps == 'Fora').count;
                        $scope.nps = data.filter(d => d.nps != 'Total' && d.nps != 'Fora' && d.count > 0);
                    });

                $scope.categories = $scope.nps.map(n => { return [n.nps, n.count] })

                $scope.chartConfig.title = {
                    text: ' '
                };
                $scope.chartConfig.subtitle = {
                    text: ' '
                };

                $scope.chartConfig.series = [{
                    name: $scope.getWord('Notas'),
                    colorByPoint: true,
                    data: $scope.categories
                }];
                $scope.chartConfig2.series = [{
                    type: 'pie',
                    name: 'Classificação / Notas',
                    data: $scope.categories
                }];

                $scope.nps_grupo = {
                    det: {
                        count: 0,
                        percentual: 0
                    },
                    pas: {
                        count: 0,
                        percentual: 0
                    },
                    pro: {
                        count: 0,
                        percentual: 0
                    }
                };
                for (let i = 0; i < $scope.nps.length; i++) {
                    let n = $scope.nps[i];
                    if (['0', '1', '2', '3', '4', '5', '6'].includes(n.nps)) {
                        $scope.nps_grupo.det.percentual += n.percentual;
                        $scope.nps_grupo.det.count += n.count;
                    }
                    else if (['7', '8'].includes(n.nps)) {
                        $scope.nps_grupo.pas.percentual += n.percentual;
                        $scope.nps_grupo.pas.count += n.count;
                    }
                    else if (['9', '10'].includes(n.nps)) {
                        $scope.nps_grupo.pro.percentual += n.percentual;
                        $scope.nps_grupo.pro.count += n.count;
                    }
                }
                $scope.NPSGraf.series[0].data = [parseInt($scope.getNPSCalc().valor)];
            });
        };

        $scope.getNPS = (np) => {
            if ($scope.nps.length <= 0) return { nps: np, count: 0, percentual: 0 }
            let ret = $scope.nps.find(n => n.nps == np);
            return ret ? ret : { nps: np, count: 0, percentual: 0 };
        }

        $scope.getDados = async function (max, callback) {
            $scope.loading = true;
            const { pars, ord } = getFiltrosToQuery(true);
            $http.get(options.api.restv4 + `/pesquisas/registros?max=${max || 10}` + pars + ord).
                success(async function (data) {
                    callback(data);
                });
        };

        $scope.ToCSV = function () {
            $scope.exportando_csv = true;

            const { pars } = getFiltrosToQuery();
            $http.get(options.api.restv4 + `/pesquisas/registrosExport?` + pars).
                success(async function (data) {
                    {
                        let arr = [];
                        let cabDesagrupados = $scope.filtro.desagrupar?.map(d => d.text) || []
                        let cab = [
                            $scope.getWord('Data'),
                            $scope.getWord('Fila'),
                            $scope.getWord('Agente'),
                            $scope.getWord('Protocolo'),
                            $scope.getWord('De'),
                            $scope.getWord('Tipo'),
                            $scope.getWord('Pesquisa'),
                            $scope.getWord('Nota'),
                            $scope.getWord('Tags'),
                            ...cabDesagrupados,
                        ];
                        arr.push(cab);

                        for (let i in data.rows) {
                            let a1 = [
                                $filter('datahora')(data.rows[i].data_registro),
                                data.rows[i].fila,
                                data.rows[i].agente,
                                !data.rows[i].uniqueid ? '' : data.rows[i].uniqueid.replace('\n', ''),
                                data.rows[i].src,
                                $scope.returnClienteTipo(data.rows[i]),
                                $scope.returnPesquisaNome(data.rows[i]),
                                $scope.getNota(data.rows[i]),
                                data.rows[i].atendimento?.tagsNomes.join(', '),
                            ];

                            for (let desagrupado of cabDesagrupados) {
                                let campoAdicional = data.rows[i].cliente?.CamposAdicionais.find(campo => campo.nome === desagrupado);
                                let valorCampo = campoAdicional ? campoAdicional.valor : '';
                                a1.push(valorCampo);
                            }

                            arr.push(a1);
                        }

                        let csv = arr.map(function (d) {
                            return d.join(';');
                        }).join('\n');

                        var exportedFilenmae = 'Pesquisa de satisfação.csv';

                        var blob = new Blob([csv], { type: 'text/csv;charset=utf-8;' });
                        if (navigator.msSaveBlob) { // IE 10+
                            navigator.msSaveBlob(blob, exportedFilenmae);
                        } else {
                            var link = document.createElement("a");
                            if (link.download !== undefined) { // feature detection
                                // Browsers that support HTML5 download attribute
                                var url = URL.createObjectURL(blob);
                                link.setAttribute("href", url);
                                link.setAttribute("download", exportedFilenmae);
                                link.style.visibility = 'hidden';
                                document.body.appendChild(link);
                                link.click();
                                document.body.removeChild(link);
                            }
                        }
                        $scope.loading = false;
                        $scope.exportando_csv = false;
                    }
                });
        };

    });
});