define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-file-upload'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.listas', [
        'ui.router',
        'ngResource',
        'ngFileUpload'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.listas', {
                url: '/listas',                
                data: {
                    title: 'Listas',
                    permissions: {
                        only: ['app.listas']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/lista.html',
                        controller: 'ListaCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ListaCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.listas.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Lista',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/form.html',
                        controller: 'ListaCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ListaCtrl',
                                'modules/listas/directives/listaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Lista',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/form.html',
                        controller: 'ListaCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ListaCtrl',
                                'modules/listas/directives/listaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Lista',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/form.html',
                        controller: 'ListaCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ListaCtrl',
                                'modules/listas/directives/listaFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.contatos', {
                url: '/contatos/:lista',
                data: {
                    title: 'Contatos',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/contatos_lista.html',
                        controller: 'ContatoCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ContatoCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.contatoseditar', {
                url: '/contatoseditar/:lista/:id',
                data: {
                    title: 'Contato',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/contato.html',
                        controller: 'ContatoCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ContatoCtrl',
                                'modules/listas/directives/contatoFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.contatosnovo', {
                url: '/contatosnovo/:lista',
                data: {
                    title: 'Contato',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/contato.html',
                        controller: 'ContatoCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/ContatoCtrl',
                                'modules/listas/directives/contatoFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.listas.upload', {
                url: '/contatosupload/:lista',
                data: {
                    title: 'Contato',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/listas/views/upload.html',
                        controller: 'UploadCtrl',
                        parent: 'app.listas',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/listas/controllers/UploadCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$rootScope) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.listas', function () {            
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.listas",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: $rootScope.getWord("Você não tem acesso a Lista.")
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});