"use strict";
define(['modules/log/module'], function (module) {
    return module.registerController('logCtrl', function ($scope, $http, $filter, $localStorage) {
        $scope.sortBy = 'DataHora';
        $scope.sortDescending = false;
        $scope.searchText = '';

        $scope.order = function (keyname, frist) {
            if (!frist) {
                $scope.sortBy = keyname;
                $scope.sortDescending = !$scope.sortDescending;
            }

            $scope.filteredRT.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.filter = function () {
            $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
            if ($scope.searchText && $scope.searchText.length > 0) {
                $scope.filteredRT = filterIt($scope.filteredRT, $scope.searchText);
            }
        };

        $scope.init = () => {
            let opa = BuscaValorPorAtributo($localStorage.User.modulos, 'nome', 'app.log');
            $scope.campos = "";
            if (opa && opa.length > 0) {
                $scope.campos = opa[0].extra;
            }

            $scope.server_filtro = {};
            $scope.server_filtro.data_ini = new Date();
            $scope.server_filtro.data_fim = new Date();
            getUsuarios();
            $scope.load();
        }

        async function getUsuarios() {
            let t = await $http.get(`${options.api.restv4}/usuarios`);
            $scope.usuarios = t.data;
        }

        $scope.filterUsuarios = (query) => {
            return $scope.usuarios.filter(function (o) {
                return o.fullname.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.isArray = angular.isArray;
        $scope.isEmpy = (obj) => {
            return (!obj || Object.keys(obj).length == 0) ? true : false;
        }

        $scope.ver = (dados) => {
            $scope.pop = dados;

            $scope.showDadosPopup = true;
        }

        $scope.load = function () {
            var pars = "";
            if ($scope.server_filtro) {
                var filtros = {};
                if ($scope.server_filtro.data_ini) {
                    filtros.data_ini = $filter('date')($scope.server_filtro.data_ini, 'yyyy-MM-dd');
                }
                if ($scope.server_filtro.data_fim) {
                    filtros.data_fim = $filter('date')($scope.server_filtro.data_fim, 'yyyy-MM-dd');
                }

                if ($scope.server_filtro.hora_ini) {
                    filtros.hora_ini = $scope.server_filtro.hora_ini + ':00';
                }
                if ($scope.server_filtro.hora_fim) {
                    filtros.hora_fim = $scope.server_filtro.hora_fim + ':59';
                }

                if ($scope.server_filtro.usuarios && $scope.server_filtro.usuarios.length > 0) {
                    filtros.usuarios = JSON.stringify($scope.server_filtro.usuarios.map(e => e.nome));
                }

                var par = [];
                angular.forEach(filtros, function (v, k) {
                    if (v) par.push(k + "=" + encodeURI(v));
                });

                if (par.length > 0) {
                    pars = "?" + par.join("&");
                }
            }

            $scope.searchText = '';

            // Valtar para pagina 1
            $scope.filteredRT = {};
            $scope.filteredRT.length = 0;

            $scope.carregando = true;
            $http.get(`${options.api.restv4}/logs${pars}`).
                success(function (data) {
                    if (data) {
                        $scope.reminderTypes = data;
                        $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                        $scope.order(null, true);
                    }
                    $scope.carregando = false;
                });
        };

        $scope.metodoClass = (metodo) => {
            switch (metodo) {
                case 'get':
                    return 'label label-primary';
                case 'post':
                    return 'label label-success';
                case 'put':
                    return 'label label-warning';
                case 'delete':
                    return 'label label-danger';
                default:
                    return 'label label-default';
            }
        }

        $scope.statusClass = (status) => {
            switch (status) {
                case 200:
                    return 'label label-success';
                case 401:
                    return 'label label-warning';
                case 500:
                    return 'label label-danger';
                default:
                    return 'label label-default';
            }
        }
    })
});
