define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.membros', [
        'ui.router',
        'ngResource'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.membros', {
                url: '/membros',                
                data: {
                    title: 'Agentes estáticos',
                    permissions: {
                        only: ['app.membros']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/membros/views/lista.html',
                        controller: 'MembrosCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/membros/controllers/MembrosCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.membros.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Editar agente estático',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/membros/views/form.html',
                        controller: 'MembrosCtrl',
                        parent: 'app.membros',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/membros/controllers/MembrosCtrl',
                                'modules/membros/directives/membrosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.membros.novo', {
                url: '/novo',
                data: {
                    title: 'Novo agente estático',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/membros/views/form.html',
                        controller: 'MembrosCtrl',
                        parent: 'app.membros',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/membros/controllers/MembrosCtrl',
                                'modules/membros/directives/membrosFormValidate'
                            ])
                        }
                    }
                }
            })
            
            .state('app.membros.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Ver agente estático',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/membros/views/form.html',
                        controller: 'MembrosCtrl',
                        parent: 'app.membros',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/membros/controllers/MembrosCtrl',
                                'modules/membros/directives/membrosFormValidate'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$filter) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.membros', function () {
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.membros",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: "Você não tem acesso ao módulo de Membros."
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});