"use strict";
define(["modules/online/module"], function (module) {
  return module.registerController(
    "onlineCtrl",
    function ($scope, $Xcmobx, $localStorage) {
      $scope.Xcmobx = $Xcmobx;

      // Usado para ativar/desativar funções em desenvolvimento
      $scope.beta = false;

      $scope.showFilas = "graph";

      $scope.filtro = {
        agentes: [],
        filas: [],
      };

      $scope.opcoes_naoatendidas = {
        barColor: "#ff021a",
        scaleColor: false,
        trackColor: "#e5e5e5",
        lineWidth: 10,
        lineCap: "circle",
        size: 125,
      };
      $scope.opcoes_atendidas = {
        barColor: "#71843f",
        scaleColor: false,
        trackColor: "#e5e5e5",
        lineWidth: 10,
        lineCap: "circle",
        size: 125,
      };
      $scope.opcoes_efetuadas = {
        barColor: "#57889c",
        scaleColor: false,
        trackColor: "#e5e5e5",
        lineWidth: 10,
        lineCap: "circle",
        size: 125,
      };
      $scope.opcoes_realizados = {
        barColor: "#92a2a8",
        scaleColor: false,
        trackColor: "#e5e5e5",
        lineWidth: 10,
        lineCap: "circle",
        size: 125,
      };

      $scope.showFilaMode = (modo) => {
        $scope.showFilas = modo;
      };

      $scope.load = function () {
        $scope.filtro.filas = $localStorage.Onlinefilas
          ? $localStorage.Onlinefilas
          : [];
      };

      $scope.filterFilas = function (query) {
        return $localStorage.User.filas.filter(function (f) {
          return f.toLowerCase().indexOf(query.toLowerCase()) != -1;
        });
      };

      $scope.filterNg = function (item) {
        if ($scope.filtro.filas && $scope.filtro.filas.length > 0) {
          let ar = $scope.filtro.filas.map((item) => item.text);
          return ar.includes(item.nome);
        } else {
          return item;
        }
      };

      $scope.filterSave = () => {
        $localStorage.Onlinefilas = $scope.filtro.filas.map((f) => f.text);
      };

      $scope.TotalAgentesOnline = (fila) => {
        return fila.agentes.filter(
          (agente) => parseInt(agente.status) === 1 && !agente.pausa
        ).length;
      };
      $scope.TotalAgentesEmPausa = (fila) => {
        return fila.agentes.filter((agente) => agente.pausa).length;
      };
      $scope.TotalAgentesOcupados = (fila) => {
        return fila.agentes.filter((agente) => parseInt(agente.status) === 2)
          .length;
      };

      $scope.TotalAgentesOffline = (fila) => {
        return fila.agentes.filter((agente) => parseInt(agente.status) === 12)
          .length;
      };

      $scope.TotalAgentesLogados = (fila) => {
        return fila.agentes.length - $scope.TotalAgentesOffline(fila);
      };

      $scope.TotalAOnline = (agentes) => {
        return agentes.filter(
          (agente) => parseInt(agente.status) === 1 && !agente.pausa
        ).length;
      };
      $scope.TotalEmPausa = (agentes) => {
        return agentes.filter(
          (agente) =>
            agente.pausa &&
            parseInt(agente.status) !== 2 &&
            (!agente.active_calls || agente.active_calls.length === 0)
        ).length;
      };
      $scope.TotalOcupados = (agentes) => {
        return agentes.filter((agente) => parseInt(agente.status) === 2).length;
      };
      $scope.TotalTocando = (agentes) => {
        return agentes.filter(
          (agente) =>
            parseInt(agente.status) === 6 || parseInt(agente.status) === 7
        ).length;
      };
      $scope.TotalIndisponivel = (agentes) => {
        return agentes.filter(
          (agente) =>
            parseInt(agente.status) === 0 ||
            parseInt(agente.status) === 4 ||
            parseInt(agente.status) === 5
        ).length;
      };

      $scope.TotalOffline = (agentes) => {
        return agentes.filter((agente) => parseInt(agente.status) === 12)
          .length;
      };

      $scope.TotalLogados = (agentes) => {
        return agentes.length - $scope.TotalOffline(agentes);
      };

      $scope.getAgenteStatusLb = function (agente) {
        const status = parseInt(agente.status);
        if (status === 2 && agente.active_calls) return "label-danger";
        if (agente.pausa && status === 6) return "label-info";
        if (agente.pausa) return "label-warning";

        switch (status) {
          case 1:
            return "label-success";
          case 2:
            return "label-danger";
          case 3:
            return "label-danger";
          case 6: // Ringing
            return "label-info";
          case 7:
            return "label-danger";
          case 8:
            return "label-warning";
          default:
            return "label-default";
        }
      };

      $scope.getAgenteStatus = function (agente) {
        const status = parseInt(agente.status);
        if (status === 2 && agente.active_calls)
          return $scope.getWord("Ocupado");
        if (agente.pausa && status === 2) return $scope.getWord("Ocupado");
        if (agente.pausa && status === 6) return $scope.getWord("Tocando");
        if (agente.pausa) return $scope.getWord("Pausa");
        switch (status) {
          case 1:
            return "Disponível";
          case 2:
            return "Ocupado";
          case 3:
            return "Ocupado";
          case 6: // Ringing
            return "Tocando";
          case 7:
            return "Tocando";
          case 8:
            return "Ocupado";
          default:
            return $scope.getWord("Indisponível");
        }
      };

      function padTime (time) {
        return (time < 10 ? "0" : "") + time;
      }

      $scope.Relogio = function (datainicial) {
        var currentTime = new Date(),
          timeElapsed = new Date(currentTime - new Date(datainicial)),
          hour = timeElapsed.getUTCHours(),
          min = timeElapsed.getUTCMinutes(),
          sec = timeElapsed.getUTCSeconds();

        return padTime(hour) + ":" + padTime(min) + ":" + padTime(sec);
      };

      $scope.TimeAgo = function (ini) {
        var moment = require("moment");
        // //console.log(ini + " -> " + moment(ini).locale('pt-br').fromNow())
        return moment(ini).locale("pt-br").fromNow();
      };

      $scope.getAgenteClass = function (agente) {
        const status = parseInt(agente.status);
        if (agente.pausa && status === 2) return $scope.getWord("bg-color-red");
        if (agente.pausa && status === 6)
          return $scope.getWord("bg-color-orange");
        if (agente.pausa && status !== 2 && status !== 5 && status !== 12)
          return "bg-color-yellow";

        switch (status) {
          case 0:
            return "bg-color-blueLight";
          case 1:
            return "bg-color-green";
          case 2:
            return "bg-color-red";
          case 3:
            return "bg-color-red";
          case 4:
            return "bg-color-blueLight";
          case 5:
            return "bg-color-blueLight";
          case 6: // Ringing
            return "bg-color-orange";
          case 7:
            return "bg-color-orange";
          case 8:
            return "bg-color-yellow";

          // Pausa -- Nao existe no asterisk, criado para o paused = 1
          case 10: {
            // Verifica se o agente tem ligações em curso
            if (agente.active_calls && agente.active_calls.length > 0) {
              // Se está em pausa, mas possui ligações, o cliente sip o colocou em AutoPausa para atendimento e classificação
              // Mas devemos apresenta-lo como ocupado
              return "bg-color-red";
            } else {
              return "bg-color-yellow";
            }
          }

          default:
            return "bg-color-blueLight";
        }
      };

      $scope.getPercentual = function (v1, v2) {
        if (typeof v1 === undefined || typeof v2 === undefined) return 0;
        v1 = parseInt(v1);
        v2 = parseInt(v2);
        if (v1 === 0) return 0;
        if (v2 === 0) return 100;
        return Math.round((v1 / v2) * 100);
      };

      $scope.getSoma = function (v1, v2) {
        if (typeof v1 === undefined || typeof v2 === undefined) return 0;
        return parseInt(v1) + parseInt(v2);
      };
    }
  );
});
