define(['modules/painelsupervisor/module'], function (module) {
    "use strict";

    return module.registerController('SuperDashboardCtrl_old', function ($scope, $http, $filter, $localStorage) {
        $scope.chartConfig = {
            chart: {
                type: 'column'
            },
            title: {
                text: `${$scope.getWord('Informe os filtros e use o botão')} "${$scope.getWord('Atualizar')}"`
            },
            xAxis: {
                type: 'category',
                labels: {
                    style: {
                        fontSize: '9px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            yAxis: {
                min: 0,
                allowDecimals: false,
                title: {
                    text: $scope.getWord('Ligações (Qtd)'),
                    style: {
                        fontSize: '9px',
                        fontFamily: 'Verdana, sans-serif'
                    }
                }
            },
            legend: {
                enabled: false
            },
            tooltip: {
                pointFormat: $scope.getWord('Valor:') + ' <b>{point.y}</b>'
            },
            dataLabels: {
                enabled: true,
                rotation: -90,
                color: '#FFFFFF',
                align: 'right',
                format: '{point.y:.1f}', // one decimal
                y: 10, // 10 pixels down from the top
                style: {
                    fontSize: '13px',
                    fontFamily: 'Verdana, sans-serif'
                }
            },
            credits: {
                enabled: true,
                text: 'X5 Soluções - Xcontact',
                href: 'http://www.x5s.com.br'
            }
        };

        $scope.status = [
            { valor: 'Atendida', nome: $scope.getWord('Atendida') },
            { valor: 'Não atendida', nome: $scope.getWord('Não atendida') },
            { valor: 'Abandonada', nome: $scope.getWord('Abandonada') },
            { valor: 'Falha', nome: $scope.getWord('Falha') },
            { valor: 'Ocupado', nome: $scope.getWord('Ocupado') }
        ];

        $scope.filterStatus = function (query) {
            return $scope.status.filter(function (st) {
                return st.nome.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getFilas = function () {
            $scope.filas = $localStorage.User.filas;
        };

        $scope.filterFilas = function (query) {
            var filas = $localStorage.User.filas;
            return filas.filter(function (fila) {
                return fila.toLowerCase().indexOf(query.toLowerCase()) != -1;
            });
        };

        $scope.getSelfila = function () {
            if (!$scope.filtro.fila || $scope.filtro.fila.length === 0) return 'Todas';
            else return _.values(_.pluck($scope.filtro.fila, "text")).join(",");
        };

        $scope.getSelfila2 = function () {
            if (!$scope.filtros.fila || $scope.filtros.fila.length === 0) return 'Todas';
            else return _.values(_.pluck($scope.filtros.fila, "text")).join(",");
        };

        $scope.getSelClassificacoes = function () {
            if (!$scope.filtro.classificacao || $scope.filtro.classificacao.length === 0) return 'Todas';
            else return _.values(_.pluck($scope.filtro.classificacao, "nome")).join(",");
        };

        $scope.getClassificacoes = function () {
            $http.get(options.api.restv2 + "/classificacoes?max=-1").
                success(function (data) {
                    $scope.classificacoes = data.dados;
                });
        };

        $scope.filterClassificacoes = function (query) {
            return $scope.classificacoes.filter(function (ob) {
                //console.log(query);
                return ob.nome.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.getAgentes = function () {
            $http.get(options.api.restv3 + "/agente/all").
                success(function (data) {
                    $scope.agentes = data.dados;
                });
        };

        $scope.filterAgentes = function (query) {
            return _.sortBy($scope.agentes, 'fullname').filter(function (agente) {
                if ($scope.filtro.fila && $scope.filtro.fila.length > 0) {
                    for (var i = 0; i < $scope.filtro.fila.length; i++) {
                        if (agente.filas && agente.filas.indexOf($scope.filtro.fila[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                        else if (agente.fila && agente.fila.indexOf($scope.filtro.fila[i].text) !== -1) {
                            return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                        }
                    }
                }
                else {
                    return agente.fullname.toLowerCase().indexOf(query.toLowerCase()) !== -1;
                }
            });
        };

        $scope.load = function () {
            $scope.filas = [];
            $scope.agentes = [];
            $scope.classificacoes = [];
            $scope.filtro = {};
            $scope.total = 0;
            $scope.tipoGrafico = "0";

            //$scope.filtro.dataini = hoje.getDate();
            $scope.filtro.status = [$scope.status[0]];
            $scope.filtro.dataini = new Date();
            $scope.filtro.datafim = new Date();

            $scope.filtro.horaini = '00:00';
            $scope.filtro.horafim = '23:59';

            $scope.filtros = angular.copy($scope.filtro);

            $scope.getClassificacoes();
            $scope.getFilas();
            $scope.getAgentes();
            $scope.getGrafico();
        };

        $scope.getGrafico = function () {
            $scope.loading = true;
            $scope.total = 0;
            let filtros = angular.copy($scope.filtro);
            filtros.data_ini = $filter('date')(filtros.dataini, 'yyyy-MM-dd') + ' ' + filtros.horaini + ':00';
            filtros.data_fim = $filter('date')(filtros.datafim, 'yyyy-MM-dd') + ' ' + filtros.horafim + ':59';
            delete filtros.dataini;
            delete filtros.datafim;
            delete filtros.horaini;
            delete filtros.horafim;
            //Filas
            if (filtros.fila && filtros.fila.length > 0) {
                // filtros.filas = JSON.stringify($scope.filas.filter((x)=> {return filtros.fila.includes(x.id);}));
                filtros.filas = JSON.stringify($scope.filas.filter(x => filtros.fila.find(y => y.text === x)));
            }
            delete filtros.fila;

            //Agentes
            if (filtros.agente && filtros.agente.length > 0) {
                filtros.agentes = JSON.stringify(filtros.agente.map((x) => { return x.fullname || x.nome; }));
            }
            delete filtros.agente;

            if (filtros.status && filtros.status.length > 0) {
                let sts = [];
                angular.forEach($scope.filtro.status, function (st) {
                    sts.push(st.valor);
                });
                filtros.status = JSON.stringify(sts);
            } else {
                delete filtros.status;
            }

            if (filtros.classificacao && filtros.classificacao.length > 0) {
                filtros.classificacao = JSON.stringify(filtros.classificacao.map(x => x.nome));
            } else {
                delete filtros.classificacao;
            }

            var par = [];
            if (filtros) {
                angular.forEach(filtros, function (v, k) {
                    if (v) par.push(k + "=" + encodeURI(v));
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }

            var moment = require('moment');

            $http.get(options.api.restv2 + "/filas/ligacoes?max=-1" + pars).success(function (data) {
                $scope.rows = _.chain(data.dados)
                    .sortBy('classificacao')
                    .groupBy('classificacao')
                    .map((v, k) => {
                        let cls = { classificacao: k == 'null' ? 'Não Classificada' : k, tme: 0, tma: 0, total: v.length }
                        let tme = 0;
                        let tma = 0;
                        v.forEach((e) => { return tme = moment.duration(e.espera).add(tme); });
                        v.forEach((a) => { return tma = moment.duration(a.duracao).add(tma); });
                        cls.tme = moment.utc(moment.duration(tme / v.length, 'ms').as('milliseconds')).format('HH:mm:ss');
                        cls.tma = moment.utc(moment.duration(tma / v.length, 'ms').as('milliseconds')).format('HH:mm:ss');
                        return cls;
                    }).value();

                $scope.total = data.dados.length;

                var titulo = '';

                if ($scope.total > 0) {
                    if ($scope.filtro.fila && $scope.filtro.fila.length > 1) {
                        titulo = $scope.getWord('Classificações') + '-' + $scope.getWord('Filas') + '<span style="text-transform: uppercase">: ' + $scope.getSelfila() + '</span>';
                    }
                    else {
                        titulo = $scope.getWord('Classificações') + '-' + $scope.getWord('Fila') + '<span style="text-transform: uppercase">: ' + $scope.getSelfila() + '</span>';
                    }

                    $scope.chartConfig.title = {
                        text: titulo
                    };

                    var sub = $scope.getWord('De') + ' ' + $filter('date')($scope.filtro.dataini, 'dd-MM-yyyy') + ' ' + $scope.filtro.horaini + ':00  ' + $scope.getWord('até') + ' ' + $filter('date')($scope.filtro.datafim, 'dd-MM-yyyy') + ' ' + $scope.filtro.horafim;

                    if (filtros.classificacao && filtros.classificacao.length > 0) {
                        sub = sub + ' - ' + $scope.getWord('Classificações') + ': ' + $scope.getSelClassificacoes();
                    }

                    $scope.chartConfig.subtitle = {
                        text: sub
                    };

                    var d = [];
                    switch ($scope.tipoGrafico) {
                        case "1":
                            d = _.chain($scope.rows).sortBy("tme").reverse().map((x) => { return [x.classificacao, moment.duration(x.tme).asSeconds()] }).value();
                            $scope.chartConfig.yAxis.title.text = $scope.getWord('T.M.E. (segundos)');
                            break;
                        case "2":
                            d = _.chain($scope.rows).sortBy("tma").reverse().map((x) => { return [x.classificacao, moment.duration(x.tma).asSeconds()] }).value();
                            $scope.chartConfig.yAxis.title.text = $scope.getWord('T.M.A. (segundos)');
                            break;
                        case "0":
                        default:
                            d = _.chain($scope.rows).sortBy("total").reverse().map((x) => { return [x.classificacao, x.total] }).value();
                            $scope.chartConfig.yAxis.title.text = $scope.getWord('Quantidade de Ligações');
                            break;
                    }

                    $scope.chartConfig.series = [{
                        name: $scope.getWord('Classificações'),
                        colorByPoint: true,
                        data: d
                    }];

                    /* $scope.chartConfig.series = [{
                        name: $scope.getWord('Classificações (Qtd)'),
                        data: $scope.rows.map((x)=>{ return {name: x.classificacao, y: x.total};})
                    },
                    {
                        name: $scope.getWord('T.M.E. (s)'),
                        data: $scope.rows.map((x)=>{ return {name: x.classificacao, y: moment.duration(x.tme).asSeconds()};})
                    },
                    {
                        name: $scope.getWord('T.M.A. (s)'),
                        data: $scope.rows.map((x)=>{ return {name: x.classificacao, y: moment.duration(x.tma).asSeconds()};})
                    }];  */
                }
                else {
                    $scope.chartConfig.title = {
                        text: $scope.getWord('Nada encontrado para os filtros informados')
                    };
                    $scope.chartConfig.series = [];
                }

                $scope.loading = false;
            });
        };
    });
});
