define([
    'angular',
    'angular-couch-potato',
    'angular-ui-router',
    'angular-resource',
    'angular-easypie',
    'angular-ws',
    'raphael',
    'jusgage',
    'highcharts',
    'angular-highcharts',
    'ngTagsInput',
    'moment',
    'moment-br',
    'moment-timezone',
    'ui.bootstrap',
    'angular-socket-io',
    'angular-desktop-notification'
], function (ng, couchPotato) {
    'use strict';

    var module = ng.module('app.painelsupervisor', [
        'ui.router',
        'ngResource',
        'easypiechart',
        'ws',
        'highcharts-ng',
        'ngTagsInput',
        'ui.bootstrap',
        'btford.socket-io',
        'ngDesktopNotification'
    ]);

    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.painelsupervisor', {
                abstract: true,
                data: {
                    title: 'Dashboards',
                    permissions: {
                        only: ['app.painelsupervisor']
                    }
                }
            })

            .state('app.painelsupervisor.dashborad', {
                url: '/painelsupervisor/dashborad',
                data: {
                    title: 'Classificações'
                },
                views: {
                    "content@app": {
                        controller: 'SuperDashboardCtrl',
                        templateUrl: 'app/modules/painelsupervisor/view/dashboard.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/SuperDashboardCtrl',
                                'modules/forms/directives/input/smartSelect2',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })

            .state('app.painelsupervisor.dashborad_old', {
                url: '/painelsupervisor/dashborad_old',
                data: {
                    title: 'Classificações (OLD)'
                },
                views: {
                    "content@app": {
                        controller: 'SuperDashboardCtrl_old',
                        templateUrl: 'app/modules/painelsupervisor/view/dashboard_old.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/SuperDashboardCtrl_old',
                                'modules/forms/directives/input/smartSelect2',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })

            .state('app.painelsupervisor.ligacoes', {
                url: '/painelsupervisor/ligacoes',
                data: {
                    title: 'Ligações por tempo'
                },
                views: {
                    "content@app": {
                        controller: 'PainelSupervisorCtrl',
                        templateUrl: 'app/modules/painelsupervisor/view/graficolinha.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/PainelSupervisorCtrl',
                                'modules/forms/directives/input/smartSelect2',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })

            .state('app.painelsupervisor.performancefila', {
                url: '/painelsupervisor/performancefila',
                data: {
                    title: 'Performance de Filas'
                },
                views: {
                    "content@app": {
                        controller: 'PerformanceFilaCtrl',
                        templateUrl: 'app/modules/painelsupervisor/view/performance_fila.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/PerformanceFilaCtrl',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })

            .state('app.painelsupervisor.classificacao', {
                url: '/painelsupervisor/classificacao',
                data: {
                    title: 'Classificação'
                },
                views: {
                    "content@app": {
                        controller: 'ClassificacoesCtrl',
                        templateUrl: 'app/modules/painelsupervisor/view/classificacao.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/ClassificacoesCtrl',
                                'modules/forms/directives/input/smartDatepicker',
                                'modules/forms/directives/input/smartTimepicker',
                                'modules/forms/directives/input/smartClockpicker'
                            ])
                        }
                    }
                }
            })

            .state('app.painelsupervisor.hardware', {
                url: '/painelsupervisor/hardware',
                data: {
                    title: 'Hardware'
                },
                views: {
                    "content@app": {
                        controller: 'HardwareCtrl',
                        templateUrl: 'app/modules/painelsupervisor/view/hardware.html',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/painelsupervisor/controllers/HardwareCtrl',
                                'modules/x5graficos/directives/justGage',
                                'modules/graphs/directives/inline/sparklineContainer'
                            ])
                        }
                    }
                }
            })

    });

    module.run(function ($couchPotato, Permission, $localStorage, ModuloPermissao) {
        module.lazy = $couchPotato;
        Permission.defineRole('app.painelsupervisor', function () {
            if (!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.painelsupervisor", $localStorage.User.modulos)) {
                $localStorage.msg = {
                    tipo: "Erro",
                    txt: "Você não tem acesso as paineis."
                };
                return false;
            }
            else {
                return true;
            }
        });
    });

    return module;
});
