define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'ng-file-upload'

], function (ng, couchPotato, _) {
    'use strict';

    var module = ng.module('app.pin', [
        'ui.router',
        'ngResource',
        'ngTagsInput'
    ]);


    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {

        $stateProvider
            .state('app.pin', {
                url: '/pin',                
                data: {
                    title: 'PINs',
                    permissions: {
                        only: ['app.pin']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/lista.html',
                        controller: 'PinCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.pin_inativos', {
                url: '/pin/inativos',                
                data: {
                    title: 'PINs Inativos',
                    permissions: {
                        only: ['app.pin']
                    }
                },                
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/lista.inativos.html',
                        controller: 'PinInativoCtrl',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinInativoCtrl'
                            ])
                        },                        
                    }
                }
            })
            
            .state('app.pin.editar', {
                url: '/editar/:id',
                data: {
                    title: 'Pin',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/form.html',
                        controller: 'PinCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.novo', {
                url: '/novo',
                data: {
                    title: 'Nova Pin',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/form.html',
                        controller: 'PinCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.ver', {
                url: '/ver/:id',
                data: {
                    title: 'Pin',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/form.html',
                        controller: 'PinCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.upload', {
                url: '/upload',
                data: {
                    title: 'Importar contatos',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/upload.html',
                        controller: 'PinCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.grupos', {
                url: '/grupos',
                data: {
                    title: 'PIN - Grupos'
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/lista.grupos.html',
                        controller: 'PinGruposCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinGruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.grupos_editar', {
                url: '/grupos/editar/:id',
                data: {
                    title: 'Editar Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/form.grupos.html',
                        controller: 'PinGruposCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinGruposCtrl'
                            ])
                        }
                    }
                }
            })
            
            .state('app.pin.grupos_novo', {
                url: '/grupos/novo',
                data: {
                    title: 'Novo Grupo',
                },
                views: {
                    "content@app": {
                        templateUrl: 'app/modules/pin/views/form.grupos.html',
                        controller: 'PinGruposCtrl',
                        parent: 'app.pin',
                        resolve: {
                            deps: $couchPotatoProvider.resolveDependencies([
                                'modules/pin/controllers/PinGruposCtrl'
                            ])
                        }
                    }
                }
            })
    });

    module.run(function ($couchPotato,Permission,ModuloPermissao,$localStorage,$rootScope) {
        module.lazy = $couchPotato;        
        Permission.defineRole('app.pin', function () {            
            if(!$localStorage.User || !ModuloPermissao.UsuarioTemModulo("app.pin",$localStorage.User.modulos)){
               $localStorage.msg = {
                   tipo: "Erro",
                   txt: $rootScope.getWord("Você não tem acesso a Pin.")
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });

    return module;

});