define(['modules/tags/module'], function (module) {
    "use strict";

    return module.registerController('TagsCtrl', function ($scope, $http, $localStorage, paginationService) {
        $scope.carregando = false;
        $scope.sortBy = 'nome';
        $scope.sortDescending = false;
        $scope.searchText = '';

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.new = () => {
            $scope.popup = {};
            $scope.showTagPopup = true;
        };

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.filter = function () {
            var results = $scope.filteredRT;
            results.length = 0;
            var searchText = $scope.searchText;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].nome && reminderTypes[i].nome.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].fila && reminderTypes[i].fila.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    results.push(reminderTypes[i]);
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        $scope.init = function () {
            $scope.load();
        };

        $scope.load = async function () {
            $scope.carregando = true;
            await $http.get(options.api.restv4 + "/tags/grupos").
                success(function (data) {
                    $scope.grupos = data;
                    $scope.carregando = false;
                })

            await $http.get(options.api.restv4 + "/tags").
                success(function (data) {
                    $scope.reminderTypes = data;
                    $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                    $scope.filter();
                });
        };

        $scope.findGroupName = function (id) {
            if (!id || id == '') return ''
            const group = $scope.grupos.find((g) => g.id == id)
            return group.nome
        }

        $scope.editar = (ob) => {
            ob.loading = true;
            $http.get(options.api.restv4 + "/tags/" + ob.id).
                success(function (data) {
                    ob.loading = false;
                    $scope.showTagPopup = true;
                    $scope.popup = data;
                });
        };

        $scope.updateColor = function (newColor) {
            $scope.popup.cor = newColor;
            $scope.$apply();
        };
        window.updateColor = $scope.updateColor;

        $scope.Salvar = function (form) {
            var pop = angular.copy($scope.popup);
            const idTag = pop.id
            delete pop.id
            if (idTag) {
                $http.put(options.api.restv4 + `/tags/${idTag}`, pop).
                    success(function (data) {
                        $scope.showTagPopup = false;
                        $.smallBox({
                            title: "SUCESSO!!",
                            content: "<i class='fa fa-check'></i> <i>Tag " + pop.nome + " atualizada!</i>",
                            color: "#739E73",
                            timeout: 2000
                        });
                        $scope.load();
                    });
            } else {
                $http.post(options.api.restv4 + `/tags`, pop).
                    success(function (data) {
                        $scope.showTagPopup = false;
                        $.smallBox({
                            title: "SUCESSO!!",
                            content: "<i class='fa fa-check'></i> <i>Tag " + pop.nome + " criada!</i>",
                            color: "#739E73",
                            timeout: 2000
                        });
                        $scope.load();
                    });
            }
        }

        $scope.Voltar = function () {
            $rootScope.$state.transitionTo('app.tags');
        };

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover " + row.nome + "? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv4 + "/tags/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Tag apagada com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });

                    $scope.load();
                })
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };
    });
});