"use strict";
define(['modules/tempos/module'], function (module) {
    return module.registerController('FeriadoCtrl', function ($scope, $http, $stateParams, $rootScope) {
        $scope.carregando = false;
        $scope.filtro = {};
        $scope.order = [{ "campo": "nome", "ordem": "ASC" }];

        $scope.dias_semana = ['Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sáb'];
        $scope.dias = [];
        for (var i = 1; i <= 31; i++) {
            $scope.dias.push(i);
        }
        $scope.messes = ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez'];

        $scope.filterDiasSemana = function (query) {
            return $scope.dias_semana.filter(function (t) {
                return t.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.filterDias = function (query) {
            return $scope.dias.filter(function (t) {
                return t.toString().indexOf(query.toString()) !== -1;
            });
        };

        $scope.filterMes = function (query) {
            return $scope.messes.filter(function (t) {
                return t.toLowerCase().indexOf(query.toLowerCase()) !== -1;
            });
        };

        $scope.init = function () {
            $scope.filtro.page = { valor: 1 };
            $scope.load();
        };

        $scope.load = function () {
            $scope.carregando = true;
            var par = [];
            if ($scope.filtro) {
                angular.forEach($scope.filtro, function (v, k) {
                    if (v) par.push(k + "=" + v.valor);
                });
            }
            var pars = "";
            if (par.length > 0) {
                pars = "&" + par.join("&");
            }
            $http.get(options.api.restv2 + "/horarios?max=15&tipo=F&order=" + encodeURI(JSON.stringify($scope.order)) + pars).
                success(function (data) {
                    // Converte o sistema numérico para String
                    data.dados = data.dados.map(turno => {
                        if (turno.dias_semana)// Verifica se a propriedade não é nula
                            turno.dias_semana = turno.dias_semana.map(x => $scope.dias_semana[x]);
                        
                        if (turno.messes)// Verifica se a propriedade não é nula
                            turno.messes = turno.messes.map(x => $scope.messes[x]);

                        return turno;
                    });

                    $scope.table = data;
                    $scope.carregando = false;
                });
        };

        $scope.showTurno = function (r) {
            $('#myTab3 a[href="#pop2s1"]').tab('show');
            if (r) {
                $scope.turno = angular.copy(r);
            } else {
                $scope.turno = {};
            }
        };

        $scope.SalvarTurno = function (f1) {
            if (f1.$valid) {
                var turno = angular.copy($scope.turno);
                turno.tipo = 'F';

                if ($scope.turno.dias_semana && $scope.turno.dias_semana.length > 0) {
                    // POSTGRES https://www.postgresql.org/docs/8.1/functions-datetime.html DOCS -> (0 - 6; Sunday is 0) 
                    turno.dias_semana = $scope.turno.dias_semana.map(x => $scope.dias_semana.indexOf(x.text));

                } else {
                    turno.dias_semana = null;
                }

                if ($scope.turno.dias && $scope.turno.dias.length > 0) {
                    turno.dias = $scope.turno.dias.map(x => parseInt(x.text));
                } else {
                    turno.dias = null;
                }

                if ($scope.turno.messes && $scope.turno.messes.length > 0) {
                    turno.messes = $scope.turno.messes.map(x => $scope.messes.indexOf(x.text));
                } else {
                    turno.messes = null;
                }

                if ($scope.turno.id) {
                    delete turno.id;
                    $http.put(options.api.restv2 + "/horario/" + $scope.turno.id, turno).
                        success(function (data) {
                            $scope.load();
                            $('#myModal').modal('hide');
                            $.smallBox({
                                title: $scope.getWord('SUCESSO') + "!!",
                                content: "<i class='fa fa-check'></i> <i>" + $scope.getWord('Turno editado com sucesso.') + "</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                        });
                }
                else {
                    $http.post(options.api.restv2 + "/horario", turno).
                        success(function (data) {
                            $scope.load();
                            $('#myModal').modal('hide');
                            $.smallBox({
                                title: $scope.getWord('SUCESSO') + "!!",
                                content: "<i class='fa fa-check'></i> <i>" + $scope.getWord('Turno cadastrado com sucesso.') + "</i>",
                                color: "#739E73",
                                timeout: 2000
                            });
                        });
                }
            }
        };

        $scope.ConfirmaRemoveTurno = function (r) {
            $.x5Box({
                title: $scope.getWord('Remover feriado'),
                content: $scope.getWord('<p>Tem certeza que quer remover o feriado ') + '<b>' + r.nome + "</b>?</p><p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Sim</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Não</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Sim") {
                    $scope.RemoveTurno(r.id);
                }
            });
        };

        $scope.RemoveTurno = function (id) {
            $http.delete(options.api.restv2 + "/horario/" + id).
                success(function (s) {
                    $.smallBox({
                        title: $scope.getWord('SUCESSO') + "!!",
                        content: "<i class='fa fa-check'></i> <i>" + $scope.getWord('Turno removido com sucesso.') + "</i>",
                        color: "#739E73",
                        timeout: 2000
                    });
                    $scope.load();
                })
        };

        $scope.gotoPage = function (page) {
            $scope.filtro.page = { valor: page };
            $scope.load();
        };

        $scope.getClass = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                return $scope.order[0].ordem === 'ASC' ? 'sorting_asc' : 'sorting_desc';
            } else {
                return 'sorting';
            }
        };

        $scope.setOrder = function (coluna) {
            if ($scope.order[0].campo === coluna) {
                $scope.order[0].ordem = $scope.order[0].ordem === 'ASC' ? 'DESC' : 'ASC';
            } else {
                $scope.order = [{ "campo": coluna, "ordem": "ASC" }];
            }
            $scope.load();
        };

        $scope.$watch('Pesquisa', function (t1, t2) {
            // Se os dois valores estirem em branco nao faz nada, isso aconte no load.
            if (!t1 && !t2)
                return false;

            setTimeout(function () {
                if (t1 === $scope.Pesquisa) {
                    if (t1 !== '') {
                        $scope.filtro.nome = { valor: t1 };
                    } else {
                        $scope.filtro.nome = undefined;
                    }

                    $scope.load();
                }
            }, 1000);
        });
    });
});
