define(['modules/troncos/module'], function (module) {
    "use strict";

    return module.registerController('TroncosCtrl', function ($scope, $http, paginationService) {
        $scope.row = {};
        $scope.sip = {};

        $scope.sortBy = 'nome';
        $scope.sortDescending = false;
        $scope.searchText = '';

        $scope.getCurPage = () => {
            return paginationService.getCurrentPage('pag');
        }

        $scope.getTotalPages = () => {
            return Math.ceil(paginationService.getCollectionLength('pag') / paginationService.getItemsPerPage('pag'))
        }

        $scope.getTotalItens = () => {
            return paginationService.getCollectionLength('pag');
        }

        $scope.order = function (keyname) {
            $scope.sortBy = keyname;
            $scope.sortDescending = !$scope.sortDescending;
            $scope.filter();
        };

        $scope.getClass = function (coluna) {
            if ($scope.sortBy === coluna) {
                return $scope.sortDescending ? 'sorting_asc' : 'sorting_desc';
            }
            else {
                return 'sorting';
            }
        };

        $scope.filter = function () {
            var results = $scope.filteredRT;
            results.length = 0;
            var searchText = $scope.searchText;
            var reminderTypes = $scope.reminderTypes;

            for (var i = 0; i < reminderTypes.length; ++i) {
                if (searchText.length > 0) {
                    if (reminderTypes[i].nome && reminderTypes[i].nome.toLowerCase().includes(searchText.toLowerCase())
                        || (reminderTypes[i].rota && reminderTypes[i].rota.toLowerCase().includes(searchText.toLowerCase()))) {
                        results.push(reminderTypes[i]);
                    }
                } else {
                    results.push(reminderTypes[i]);
                }
            }
            results.sort(function (a, b) {
                var sortBy = $scope.sortBy;
                if (a[sortBy] < b[sortBy]) {
                    return $scope.sortDescending ? 1 : -1;
                }
                if (a[sortBy] > b[sortBy]) {
                    return $scope.sortDescending ? -1 : 1;
                }
                return 0;
            });
        };

        // Chamada para primeira pagina
        $scope.load = function () {
            $scope.carregando = true;
            $http.get(`${options.api.restv4}/troncos`).
                success(function (data) {
                    if (data) {
                        $scope.reminderTypes = data;
                        $scope.filteredRT = JSON.parse(JSON.stringify($scope.reminderTypes));
                        $scope.filter();
                    }
                    $scope.carregando = false;
                });
        };

        $scope.AddCampo = () => {
            if ($scope.pop.conf.length <= 0) $scope.pop.conf = [];
            $scope.pop.conf.push({ parametro: '', valor: '' });
        }

        $scope.DelCampo = (i) => {
            $scope.pop.conf.splice(i, 1);
        };

        $scope.novo = async () => {
            $scope.carregando_novo = true;
            $scope.sip_disable = false;
            $scope.pop = {};
            await getRotas();
            $scope.pop.conf = [
                { parametro: 'host', valor: 'dynamic' },
                { parametro: 'fromdomain', valor: 'xcontactcenter.com.br' },
                { parametro: 'username', valor: 'usuario' },
                { parametro: 'secret', valor: 'senha' },
                { parametro: 'qualify', valor: 'yes' },
                { parametro: 'insecure', valor: 'invite,port' },
                { parametro: 'nat', valor: 'force_rport,comedia' },
                { parametro: 'disallow', valor: 'all' },
                { parametro: 'allow', valor: 'ulaw' },
                { parametro: 'allow', valor: 'alaw' },
                { parametro: 'allow', valor: 'g729' },
                { parametro: 'directmedia', valor: 'no' },
                { parametro: 'type', valor: 'peer' }
            ]
            $scope.carregando_novo = false;
            $scope.showTroncoPopup = true;
        }

        async function getRotas() {
            let c = await $http.get(`${options.api.restv2}/contexto?max=-1&sentido=%5B%22Entrada%22%5D`);
            $scope.rotas = c.data.dados;
        }

        $scope.editar = async (tronco) => {
            tronco.load = true;
            $scope.sip_disable = true;
            await getRotas();

            let t = await $http.get(`${options.api.restv4}/tronco/${tronco.id}`);
            $scope.pop = t.data;

            if ($scope.pop.conf) {
                $scope.pop.conf = $scope.pop.conf.filter(function (obj) {
                    return obj.parametro !== 'context';
                });
            }
            tronco.load = false;
            $scope.showTroncoPopup = true;
        }

        $scope.Salvar = async (f) => {
            if (f.$valid) {
                let dados = JSON.parse(JSON.stringify($scope.pop));
                dados.conf.push({ parametro: 'context', valor: $scope.pop.rota });
                delete dados.id;

                try {
                    if ($scope.pop.id) {
                        await $http.put(`${options.api.restv4}/tronco/${$scope.pop.id}`, dados);
                    }
                    else {
                        await $http.post(`${options.api.restv4}/tronco`, dados);
                    }

                    $scope.showTroncoPopup = false;
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Tronco atualizado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 3000
                    });
                    $scope.load();
                } catch (error) {
                    console.log(error);
                }
            }
        }

        $scope.confirma = function (row) {
            $.x5Box({
                title: "Remover TRONCO <b>" + row.nome + "</b>?",
                content: "Tem certeza que quer remover o tronco <b>" + row.nome + "</b>? <p class='text-align-right'><a id='yes' href-void class='btn btn-danger btn-sm btncallback'>Apagar</a> <a id='no' href-void class='btn btn-primary btn-sm btncallback'>Cancelar</a></p>",
                color: "#C46A69",
                //timeout: 8000,
                icon: "fa fa-warning shake animated"
            }, function (ButtonPressed) {
                if (ButtonPressed === "Apagar") {
                    $scope.delete(row.id);
                }
            });
        };

        $scope.delete = function (id) {
            $http.delete(options.api.restv4 + "/tronco/" + id).
                success(function (s) {
                    $.smallBox({
                        title: "SUCESSO!!",
                        content: "<i class='fa fa-check'></i> <i>Tronco apagado com sucesso!</i>",
                        color: "#739E73",
                        timeout: 6000
                    });
                    $scope.load();
                })
        };

    });
});