define(['app'], function (app) {
    "use strict";
    app.service('exportFilesService', function ($localStorage, $http) {
        this.gerarCSV = function (dados, filename) {
            return new Promise((resolve, reject) => {
                try {
                    let csv = dados.map(function (d) {
                        return d.join(',');
                    }).join('\n');

                    var blob = new Blob([csv], { type: 'text/csv;charset=utf-8;' });
                    if (navigator.msSaveBlob) { // IE 10+
                        navigator.msSaveBlob(blob, filename);
                    } else {
                        var link = document.createElement("a");
                        if (link.download !== undefined) { // feature detection
                            // Browsers that support HTML5 download attribute
                            var url = URL.createObjectURL(blob);
                            link.setAttribute("href", url);
                            link.setAttribute("download", filename);
                            link.style.visibility = 'hidden';
                            document.body.appendChild(link);
                            link.click();
                            document.body.removeChild(link);
                        }
                    }
                    resolve('CSV gerado com sucesso!');
                } catch (error) {
                    reject(error);
                }
            });
        };
        this.gerarDownload = function (caminho, tipo, filename) {
            return new Promise((resolve, reject) => {
                try {
                    var xhr = new XMLHttpRequest();
                    xhr.onload = function () {
                        if (this.status == 200) {
                            var myBlob = this.response;
                            var blob = new Blob([myBlob], { type: tipo });
                            if (navigator.msSaveBlob) { // IE 10+
                                navigator.msSaveBlob(blob, filename);
                            } else {
                                var link = document.createElement("a");
                                if (link.download !== undefined) { // feature detection
                                    // Browsers that support HTML5 download attribute
                                    var url = URL.createObjectURL(blob);
                                    link.setAttribute("href", url);
                                    link.setAttribute("download", filename);
                                    link.style.visibility = 'hidden';
                                    document.body.appendChild(link);
                                    link.click();
                                    document.body.removeChild(link);
                                }
                            }
                            resolve('Download gerado');
                        }
                    };
                    xhr.open('GET', caminho);
                    xhr.responseType = 'blob';
                    xhr.send();
                } catch (error) {
                    reject(error);
                }
            });
        };
        function getWord(key) {
            if ($localStorage && $localStorage.lang && $localStorage.lang[key]) {
                return $localStorage.lang[key];
            }
            else {
                return key;
            }
        };
        function getLogo() {
            return new Promise((resolve, reject) => {
                try {
                    var xhr = new XMLHttpRequest();
                    xhr.onload = function () {
                        var reader = new FileReader();
                        reader.onloadend = function () {
                            resolve(reader.result);
                        }
                        reader.readAsDataURL(xhr.response);
                    };
                    xhr.open('GET', 'styles/img/LogoComSite_160dpi.jpg');
                    xhr.responseType = 'blob';
                    xhr.send();
                } catch (error) {
                    reject(error);
                }

            });
        }
        this.gerarPDF = async function (cabecalho, linhas, titulo, subtitulo, filename) {
            try {
                var jsPDF = require('jspdf');
                var doc = new jsPDF('l', 'pt', 'a4');
                if (titulo) {
                    doc.setFontSize(18);
                    doc.text(titulo.texto, titulo.x, titulo.y);
                    doc.setFontSize(8);
                    doc.setTextColor(100);
                }
                else {
                    doc.setFontSize(18);
                    doc.text('Sem titulo', 40, 60);
                    doc.setFontSize(8);
                    doc.setTextColor(100);
                }

                if (subtitulo) {
                    subtitulo.forEach((subt) => {
                        doc.text(subt.texto, subt.x, subt.y);
                    });
                }

                let imagen = await getLogo();
                doc.addImage(imagen, 'JPEG', 630, 40, 168, 38);

                var totalPagesExp = "{total_pages_count_string}";
                var footer = function (data) {
                    var str = getWord("Página") + ' ' + data.pageCount;
                    // Total page number plugin only available in jspdf v1.0+
                    if (typeof doc.putTotalPages === 'function') {
                        str = str + ' ' + getWord("de") + ' ' + totalPagesExp;
                    }
                    doc.setFontSize(8);
                    doc.setTextColor(60);
                    doc.setFontStyle('normal');
                    doc.text(str, data.settings.margin.left, doc.internal.pageSize.height - 30);
                };

                doc.autoTable(cabecalho, linhas, {
                    startY: 120,
                    styles: { cellPadding: 2 },
                    headerStyles: { verticalCellPadding: 14, fontSize: 8, halign: 'center', valign: 'middle', fillColor: [59, 118, 145] },
                    bodyStyles: { verticalCellPadding: 12, fontSize: 8, halign: 'center', valign: 'middle' },
                    createdCell: function (cell, data) {
                        if (data.row.raw[0] === 'Total') {
                            //cell.styles.halign = 'right';
                            cell.styles.fillColor = [239, 123, 21];
                            cell.styles.textColor = [255, 255, 255];
                            cell.styles.fontStyle = 'bold';
                            //cell.text = '$' + cell.text;
                        }
                    },
                    addPageContent: footer
                });

                if (typeof doc.putTotalPages === 'function') {
                    doc.putTotalPages(totalPagesExp);
                }
                doc.save(filename);
                return ('PDF gerado com sucesso!');
            } catch (error) {
                return (error);
            }
        }
    });
    return app;
});