function groupBy(array, key) {
    return array.reduce((result, currentValue) => {
        (result[currentValue[key]] = result[currentValue[key]] || []).push(
            currentValue
        );
        return result;
    }, {});
};

/**
 * Remove itens duplicados em um array de objetos baseados em um atributo.
 * @param {*} array 
 * @param {*} key 
 * @returns Array
 */
function ArrayUnique(array, key) {
    return [...new Map(array.map(item => [item[key], item])).values()];
}

/**
 * Essa função busca recursivamente um valor no atributo informado
 * @param {*} obj Array ou Objeto para procurar 
 * @param {*} key Nome o atributo
 * @param {*} value Valor a pesquisar
 * @returns Array
 */
function BuscaValorPorAtributo(obj, key, value) {
    let list = [];
    if (!obj) return list;
    if (obj instanceof Array) {
        for (var i in obj) {
            list = list.concat(BuscaValorPorAtributo(obj[i], key, value));
        }
        return list;
    }
    if (obj[key] == value) list.push(obj);

    if ((typeof obj == "object") && (obj !== null)) {
        let children = Object.keys(obj);
        if (children.length > 0) {
            for (let i = 0; i < children.length; i++) {
                list = list.concat(BuscaValorPorAtributo(obj[children[i]], key, value));
            }
        }
    }
    return list;
}

/**
 * Função recursiva que busca um valor em qualquer atributo
 * @param {*} arr  Array ou Objeto onde buscar
 * @param {*} searchKey String valor a buscar
 * @returns Array
 */
function filterIt(arr, searchKey) {
    return arr.filter(function (obj) {
        return Object.keys(obj).some(function (key) {
            if (obj[key]) {
                if (Array.isArray(obj[key])) {
                    let find = filterIt(obj[key], searchKey);
                    if (find && find.length > 0) {
                        return find;
                    }
                }
                else {
                    return obj[key].toString().toUpperCase().includes(searchKey.toString().toUpperCase());
                }
            }
        })
    });
}