/**
 * angular-X5ControleAcesso
 * Modulo em AngularJs para fazer controle de acesso ao front-end utilizando ui-route
 * Baseado em https://github.com/Narzerus/angular-permission
 * @version v0.0.1 - 18-11-2014
 * @link http://www.x5s.com.br
 * @author Fernando Rossato <fernando.rossato@x5s.com.br>
 * @license MIT License, http://www.opensource.org/licenses/MIT
 * 
 * USO:
 * Primeiramente voce deve criar um modulo para tratar a rota /login que sera redirecionada em caso de não logado ou sem permissão.
 define([
    'angular',
    'angular-couch-potato',
    'lodash',
    'angular-ui-router',
    'angular-resource',
    'angular-x5controleacesso'

 var module = ng.module('app.MODULO', [
        'ui.router',
        'ngResource',
        'X5ControleAcesso'
    ]);

.state('app.classificacoes', {
                url: '/classificacoes',                
                data: {
                    title: 'Classificações',
                    permissions: {
                        only: ['app.classificacoes']
                    }
                },

 module.run(function ($couchPotato,Permission,User,$filter) {
        module.lazy = $couchPotato;
        
        Permission.defineRole('app.pausas', function () {
            if(!User.modulos || $filter('filter')(User.modulos, {nome:"app.pausas"}).length <= 0){
               User.msg = {
                   tipo: "Erro",
                   txt: "Você não tem acesso ao módulo de Pausas."
               };               
               return false;
            }            
            else {                
                return true;         
            }
        });
    });
 */

define([
    'angular',
    'angular-couch-potato',
    'angular-ui-router',
    'angular-storage'
], function (ng, couchPotato) {

    "use strict";

    var module = ng.module('X5ControleAcesso', [
        'ui.router',
        'ngStorage'
    ]);

    couchPotato.configureApp(module);

    module.config(function ($stateProvider, $couchPotatoProvider) {
        $stateProvider
            .state('X5ControleAcesso', {
                abstract: true,
                data: {
                    title: 'X5ControleAcesso'
                }
            });
    });
    var tem = false;
    function BuscaModulo(modulo, modulos) {
        for (var i in modulos) {
            if (modulos[i].nome && modulos[i].nome.indexOf(modulo) >= 0) {
                tem = true;
                return;
            }
            else if (modulos[i].modulos && modulos[i].modulos.length > 0) {
                BuscaModulo(modulo, angular.copy(modulos[i].modulos));
            }
        }
    };

    module.registerFactory('ModuloPermissao', function () {
        return {
            UsuarioTemModulo: function (modulo, modulos) {
                BuscaModulo(modulo, modulos);
                return tem;
            }
        };
    });

    module.registerFactory('User', function ($q) {
        var dfd = $q.defer();

        var UserModel = {
            initialized: dfd.promise,
            id: undefined,
            nome: undefined,
            foto: undefined,
            tocken: undefined,
            modulos: undefined
        };
        /**
         * modulos: [
                {nome: "app.pausas", descricao: "Pausas", icon: "fa fa-history"}
            ]
         $http.get('api/user.json').then(function(response){
             UserModel.username = response.data.username;
             UserModel.picture= response.data.picture;
             UserModel.modulos = ["app.classificacoes","app.pausas"];
             dfd.resolve(UserModel);
         });
        **/
        return UserModel;
    });

    module.run(function ($rootScope, Permission, $state) {
        $rootScope.$on('$stateChangeStart',
            function (event, toState, toParams, fromState, fromParams) {
                // If there are permissions set then prevent default and attempt to authorize
                var permissions;
                if (toState.data && toState.data.permissions) {
                    permissions = toState.data.permissions;
                } else if (toState.permissions) {
                    /**
                     * This way of defining permissions will be depracated in v1. Should use
                     * `data` key instead
                     */
                    console.log('Deprecation Warning: permissions should be set inside the `data` key ');
                    console.log('Setting permissions for a state outside `data` will be depracated in' +
                        ' version 1');
                    permissions = toState.permissions;
                }

                if (permissions) {
                    event.preventDefault();

                    Permission.authorize(permissions, toParams).then(function () {

                        // If authorized, use call state.go without triggering the event.
                        // Then trigger $stateChangeSuccess manually to resume the rest of the process
                        // Note: This is a pseudo-hacky fix which should be fixed in future ui-router versions
                        if (!$rootScope.$broadcast('$stateChangeStart', toState.name, toParams, fromState.name, fromParams).defaultPrevented) {
                            $rootScope.$broadcast('$stateChangePermissionAccepted');

                            $state.go(toState.name, toParams, { notify: false }).then(function () {
                                $rootScope
                                    .$broadcast('$stateChangeSuccess', toState, toParams, fromState, fromParams);
                            });
                        }

                    }, function () {
                        if (!$rootScope.$broadcast('$stateChangeStart', toState.name, toParams, fromState.name, fromParams).defaultPrevented) {
                            $rootScope.$broadcast('$stateChangePermissionDenied');

                            // If not authorized, redirect to wherever the route has defined, if defined at all
                            var redirectTo = permissions.redirectTo;
                            if (redirectTo) {
                                $state.go(redirectTo, {}, { notify: false }).then(function () {
                                    $rootScope
                                        .$broadcast('$stateChangeSuccess', toState, toParams, fromState, fromParams);
                                });
                            }
                            else {
                                $state.go("login");
                            }
                        }
                    });
                }
            });
    });

    module.provider('Permission', function () {
        var roleValidationConfig = {};
        var validateRoleDefinitionParams = function (roleName, validationFunction) {
            if (!angular.isString(roleName)) {
                throw new Error('Role name must be a string');
            }
            if (!angular.isFunction(validationFunction)) {
                throw new Error('Validation function not provided correctly');
            }
        };

        this.defineRole = function (roleName, validationFunction) {
            /**
             This method is only available in config-time, and cannot access services, as they are
             not yet injected anywere which makes this kinda useless.
             Should remove if we cannot find a use for it.
             **/
            validateRoleDefinitionParams(roleName, validationFunction);
            roleValidationConfig[roleName] = validationFunction;

            return this;
        };

        this.$get = function ($q) {
            var Permission = {
                _promiseify: function (value) {
                    /**
                     Converts a value into a promise, if the value is truthy it resolves it, otherwise
                     it rejects it
                     **/
                    if (value && angular.isFunction(value.then)) {
                        return value;
                    }

                    var deferred = $q.defer();
                    if (value) {
                        deferred.resolve();
                    } else {
                        deferred.reject();
                    }
                    return deferred.promise;
                },
                _validateRoleMap: function (roleMap) {
                    if (typeof (roleMap) !== 'object' || roleMap instanceof Array) {
                        throw new Error('Role map has to be an object');
                    }
                    if (roleMap.only === undefined && roleMap.except === undefined) {
                        throw new Error('Either "only" or "except" keys must me defined');
                    }
                    if (roleMap.only) {
                        if (!(roleMap.only instanceof Array)) {
                            throw new Error('Array of roles expected');
                        }
                    } else if (roleMap.except) {
                        if (!(roleMap.except instanceof Array)) {
                            throw new Error('Array of roles expected');
                        }
                    }
                },
                _findMatchingRole: function (rolesArray, toParams) {
                    var roles = angular.copy(rolesArray);
                    var deferred = $q.defer();
                    var currentRole = roles.shift();

                    // If no roles left to validate reject promise
                    if (!currentRole) {
                        deferred.reject();
                        return deferred.promise;
                    }
                    // Validate role definition exists
                    if (!angular.isFunction(Permission.roleValidations[currentRole])) {
                        throw new Error('undefined role or invalid role validation');
                    }

                    var validatingRole = Permission.roleValidations[currentRole](toParams);
                    validatingRole = Permission._promiseify(validatingRole);

                    validatingRole.then(function () {
                        deferred.resolve();
                    }, function () {
                        Permission._findMatchingRole(roles, toParams).then(function () {
                            deferred.resolve();
                        }, function () {
                            deferred.reject();
                        });
                    });

                    return deferred.promise;
                },
                defineRole: function (roleName, validationFunction) {
                    /**
                     Service-available version of defineRole, the callback passed here lives in the
                     scope where it is defined and therefore can interact with other modules
                     **/
                    validateRoleDefinitionParams(roleName, validationFunction);
                    roleValidationConfig[roleName] = validationFunction;

                    return Permission;
                },
                resolveIfMatch: function (rolesArray, toParams) {
                    var roles = angular.copy(rolesArray);
                    var deferred = $q.defer();
                    Permission._findMatchingRole(roles, toParams).then(function () {
                        // Found role match
                        deferred.resolve();
                    }, function () {
                        // No match
                        deferred.reject();
                    });
                    return deferred.promise;
                },
                rejectIfMatch: function (roles, toParams) {
                    var deferred = $q.defer();
                    Permission._findMatchingRole(roles, toParams).then(function () {
                        // Role found
                        deferred.reject();
                    }, function () {
                        // Role not found
                        deferred.resolve();
                    });
                    return deferred.promise;
                },
                roleValidations: roleValidationConfig,
                authorize: function (roleMap, toParams) {
                    // Validate input
                    Permission._validateRoleMap(roleMap);

                    var authorizing;

                    if (roleMap.only) {
                        authorizing = Permission.resolveIfMatch(roleMap.only, toParams);
                    } else {
                        authorizing = Permission.rejectIfMatch(roleMap.except, toParams);
                    }

                    return authorizing;
                }
            };

            return Permission;
        };
    });

    return module;
});